@@ -74,8 +74,8 @@ vector (int nl, int nh)
   float *v;
 
   v = (float *) malloc ((unsigned) (nh - nl + 1) * sizeof (float));
-  if (!v)
-    fprintf (stderr, "memory allocation failure"), exit (1);
+  if (v == NULL)
+    pm_error("Unable to allocate memory for a vector.");
   return v - nl;
 }
 
@@ -90,16 +90,17 @@ matrix (int nrl, int nrh, int ncl, int nch)
 
   /* allocate pointers to rows */
   m = (float **) malloc ((unsigned) (nrh - nrl + 1) * sizeof (float *));
-  if (!m)
-    fprintf (stderr, "memory allocation failure"), exit (1);
+  if (m == NULL)
+    pm_error("Unable to allocate memory for a matrix.");
+
   m -= ncl;
 
   /* allocate rows and set pointers to them */
   for (i = nrl; i <= nrh; i++)
   {
     m[i] = (float *) malloc ((unsigned) (nch - ncl + 1) * sizeof (float));
-    if (!m[i])
-      fprintf (stderr, "memory allocation failure"), exit (2);
+    if (m[i] == NULL)
+      pm_error("Unable to allocate memory for a matrix row.");
     m[i] -= ncl;
   }
   /* return pointer to array of pointers to rows */
@@ -747,7 +748,7 @@ f13_icorr (float **P, int Ng)
     hy -= py[i] * log10 (py[i] + EPSILON);
   }
 /*  fprintf(stderr,"hx=%f\thxy2=%f\n",hx,hxy2); */
-  return (sqrt (fabs (1 - exp (-2.0 * (hxy2 - hxy)))));
+  return (sqrt (abs (1 - exp (-2.0 * (hxy2 - hxy)))));
 }
 
 static float 
