@@ -2,12 +2,8 @@
 
 require 5.000;
 
-#Note: mkdir() must have 2 arguments as late as 5.005.
-
 use strict;
 use English;
-use Fcntl;
-use File::Basename;
 
 my ($TRUE, $FALSE) = (1,0);
 
@@ -193,9 +189,9 @@ sub getBinDir($) {
 
 
 
-sub installProgram($$$) {
+sub installProgram($$) {
 
-    my ($pkgdir, $prefix, $bindirR) = @_;
+    my ($pkgdir, $prefix) = @_;
 
     my $binDir = getBinDir($prefix);
 
@@ -209,7 +205,6 @@ sub installProgram($$$) {
     } else {
         print("Done.\n");
     }
-    $$bindirR = $binDir;
 }
 
 
@@ -303,19 +298,12 @@ execLdconfig() {
 
 
 
-sub ldconfigExists() {
-
-    return (system("ldconfig -? 2>/dev/null") >> 8) != 127;
-}
-
-
-
 sub
 doLdconfig() {
 #-----------------------------------------------------------------------------
 #  Run Ldconfig where appropriate.
 #-----------------------------------------------------------------------------
-    if ($OSNAME eq "linux" || ldconfigExists()) {
+    if ($OSNAME eq "linux" || system("ldconfig -? 2>/dev/null") != 127) {
         # This is a system where Ldconfig makes sense
 
         print("In order for the Netpbm shared library to be found when " .
@@ -352,9 +340,9 @@ doLdconfig() {
 
 
 
-sub installSharedLib($$$) {
+sub installSharedLib($$) {
 
-    my ($pkgdir, $prefix, $libdirR) = @_;
+    my ($pkgdir, $prefix) = @_;
 
     if (-d("$pkgdir/lib")) {
         my $libDir = getLibDir($prefix);
@@ -371,7 +359,6 @@ sub installSharedLib($$$) {
             print("\n");
             doLdconfig();
         }
-        $$libdirR = $libDir;
     } else {
         print("You did not build a shared library, so I will not " .
               "install one.\n");
@@ -415,9 +402,9 @@ sub getLinkDir($) {
 
 
 
-sub installStaticLib($$$) {
+sub installStaticLib($$) {
 
-    my ($pkgdir, $prefix, $linkdirR) = @_;
+    my ($pkgdir, $prefix) = @_;
 
     if (-d("$pkgdir/link")) {
         my $linkDir = getLinkDir($prefix);
@@ -432,7 +419,6 @@ sub installStaticLib($$$) {
         } else {
             print("done.\n");
         }
-        $$linkdirR = $linkDir;
     } else {
         print("You did not build a static library, so I will not " .
               "install one \n");
@@ -509,9 +495,9 @@ sub getHdrDir($) {
 
 
 
-sub installDataFile($$$) {
+sub installDataFile($$) {
 
-    my ($pkgdir, $prefix, $datadirR) = @_;
+    my ($pkgdir, $prefix) = @_;
 
     my $dataDir = getDataDir($prefix);
 
@@ -524,16 +510,15 @@ sub installDataFile($$$) {
               "failed.\n");
         print("cp exit code is $rc\n");
     } else {
-        $$datadirR = $dataDir;
         print("done.\n");
     }
 }
 
 
 
-sub installHeader($$$) {
+sub installHeader($$) {
 
-    my ($pkgdir, $prefix, $includedirR) = @_;
+    my ($pkgdir, $prefix) = @_;
 
     my $hdrDir = getHdrDir($prefix);
 
@@ -548,7 +533,6 @@ sub installHeader($$$) {
     } else {
         print("done.\n");
     }
-    $$includedirR = $hdrDir;
 }
 
 
@@ -640,15 +624,9 @@ sub tryToCreateManwebConf($) {
 
 
 sub getWebdir($) {
+
     my ($manweb_conf_filename) = @_;
-#-----------------------------------------------------------------------------
-#  Return the value of the Manweb "web directory," as indicated by the
-#  Manweb configuration file $manweb_conf_filename.
-#
-#  If that file doesn't exist, or doesn't have a 'webdir' value, or
-#  the 'webdir' value is a chain of directories instead of just one,
-#  we return an undefined value.
-#-----------------------------------------------------------------------------
+
     my $webdir;
 
     my $success = open(MANWEB_CONF, "<$manweb_conf_filename");
@@ -666,11 +644,7 @@ sub getWebdir($) {
                 #It looks like "keyword=value"
                 my ($keyword, $value) = ($1, $2);
                 if ($keyword eq "webdir") {
-                    # We can't handle a multi-directory path; we're looking
-                    # only for a webdir statement naming a sole directory.
-                    if ($value !~ m{:}) {
-                        $webdir = $value;
-                    }
+                    $webdir = $value;
                 }
             }
         }
@@ -727,7 +701,6 @@ sub makeInManwebPath($) {
                 if (userWantsManwebSymlink($webdir, $netpbmWebdir)) {
                     my $old = "$netpbmWebdir/netpbm.url";
                     my $new = "$webdir/netpbm.url";
-                    mkdir($webdir, 0777);
                     my $success = symlink($old, $new);
                     if (!$success) {
                         print("Failed to create symbolic link from $new to " .
@@ -741,12 +714,12 @@ sub makeInManwebPath($) {
 
 
 
-sub installManPage($$$) {
+sub installManPage($$) {
 
 
 # Note: This installs the pointer man pages and the netpbm.url file for Manweb.
 
-    my ($pkgdir, $prefix, $mandirR) = @_;
+    my ($pkgdir, $prefix) = @_;
 
     my $manDir = getManDir($prefix);
 
@@ -766,109 +739,11 @@ sub installManPage($$$) {
     removeObsoleteManPage($manDir);
 
     makeInManwebPath("$manDir/web");
-    
-    $$mandirR = $manDir;
+
 }
 
 
 
-sub 
-processTemplate($$$$$$$$$) {
-    my ($templateR, $version, $bindir, $libdir, $linkdir, $datadir, 
-        $includedir, $mandir, $outputR) = @_;
-
-    my @output;
-
-    foreach (@{$templateR}) {
-        if (m{^@}) {
-            # Comment -- ignore it.
-        } else {
-            if (defined($version)) {
-                s/\@VERSION\@/$version/;
-            }
-            if (defined($bindir)) {
-                s/\@BINDIR@/$bindir/;
-            }
-            if (defined($libdir)) {
-                s/\@LIBDIR@/$libdir/;
-            }
-            if (defined($linkdir)) {
-                s/\@LINKDIR@/$linkdir/;
-            }
-            if (defined($datadir)) {
-                s/\@DATADIR@/$datadir/;
-            }
-            if (defined($includedir)) {
-                s/\@INCLUDEDIR@/$includedir/;
-            }
-            if (defined($mandir)) {
-                s/\@MANDIR@/$mandir/;
-            }
-            push(@output, $_);
-        }
-    }
-    $$outputR = \@output;
-}
-
-
-
-sub
-installConfig($$$$$$$) {
-    my ($pkgdir, 
-        $bindir, $libdir, $linkdir, $datadir, $includedir, $mandir) = @_;
-
-    my $error;
-
-    my $configTemplateFilename = dirname($0) . "/config_template";
-
-    my $templateOpened = open(TEMPLATE, "<$configTemplateFilename");
-    if (!$templateOpened) {
-        $error = "Can't open template file '$configTemplateFilename'.\n";
-    } else {
-        my @template = <TEMPLATE>;
-
-        close(TEMPLATE);
-
-        my $versionOpened = open(VERSION, "<$pkgdir/VERSION");
-
-        my $version;
-        if (!$versionOpened) {
-            $error = "Unable to open $pkgdir/VERSION for reading.  " .
-                "Errno=$ERRNO\n";
-        } else {
-            $version = <VERSION>;
-            chomp($version);
-            close(VERSION);
-
-            processTemplate(\@template, $version, $bindir, $libdir,
-                            $linkdir, $datadir, $includedir, $mandir,
-                            \my $fileContentsR);
-
-            # TODO: Really, this ought to go in an independent directory,
-            # because you might want to have the Netpbm executables in
-            # some place not in the PATH and use this program, via the
-            # PATH, to find them.
-            
-            my $filename = "$bindir/netpbm-config";
-            
-            my $success = open(NETPBM_CONFIG, ">$filename");
-            if ($success) {
-                chmod(0755, $filename);
-                foreach (@{$fileContentsR}) { print NETPBM_CONFIG; }
-                close(NETPBM_CONFIG);
-            } else {
-                $error = "Unable to open the file " .
-                    "'$filename' for writing.  Errno=$ERRNO\n";
-            }
-        }
-    }
-    if ($error) {
-        print(STDERR "Failed to create the Netpbm configuration program.  " .
-              "$error\n");
-    }
-}
-
-
 
 #******************************************************************************
 #
@@ -891,27 +766,24 @@ my $prefix = getPrefix();
 
 $cpCommand = getCpCommand();
 
-installProgram($pkgdir, $prefix, \my $bindir);
+installProgram($pkgdir, $prefix);
 print("\n");
 
-installSharedLib($pkgdir, $prefix, \my $libdir);
+installSharedLib($pkgdir, $prefix);
 print("\n");
 
-installStaticLib($pkgdir, $prefix, \my $linkdir);
+installStaticLib($pkgdir, $prefix);
 print("\n");
 
-installDataFile($pkgdir, $prefix, \my $datadir);
+installDataFile($pkgdir, $prefix);
 print("\n");
 
-installHeader($pkgdir, $prefix, \my $includedir);
+installHeader($pkgdir, $prefix);
 print("\n");
 
-installManPage($pkgdir, $prefix, \my $mandir);
+installManPage($pkgdir, $prefix);
 print("\n");
 
-installConfig($pkgdir, 
-              $bindir, $libdir, $linkdir, $datadir, $includedir, $mandir);
-
 print("Installation is complete (except where previous error messages have\n");
 print("indicated otherwise).\n");
 
