@@ -216,7 +216,7 @@ BMPreadinfoheader(FILE *             const ifP,
            bits are divided among red, green, and blue, so we can't handle it.
         */
         else
-            pm_error("Unrecognized bits per pixel in BMP file header: %d",
+            pm_error("Unrecognized bits per pixel in OS/2 BMP file header: %d",
                      cBitCount);
 
         break;
@@ -247,20 +247,21 @@ BMPreadinfoheader(FILE *             const ifP,
            */
         colorsimportant = GetLong(ifP);  /* ColorsImportant */
 
-        if (colorsused != 0)
-            headerP->cmapsize = colorsused;
-        else {
-            if (cBitCount <= 8)
+        if (cBitCount <= 8) {
+            if (colorsused != 0) {
+                if (colorsused > 1 << cBitCount)
+                    pm_error("Invalid BMP header.  Says %u bits per pixel, "
+                             "but %d colors used", cBitCount, colorsused);
+                else
+                    headerP->cmapsize = colorsused;
+            } else 
                 headerP->cmapsize = 1 << cBitCount;
-            else if (cBitCount == 24)
-                headerP->cmapsize = 0;
-            /* There is a 16 bit truecolor format, but we don't know
-               how the bits are divided among red, green, and blue, so
-               we can't handle it.  */
-            else
-                pm_error("Unrecognized bits per pixel in BMP file header: %d",
-                         cBitCount);
-        }
+        } else if (cBitCount == 24 || cBitCount == 16)
+            headerP->cmapsize = 0;
+        else
+            pm_error("Unrecognized bits per pixel in "
+                     "Windows BMP file header: %d",
+                     cBitCount);
     }
         break;
     default:
@@ -304,13 +305,18 @@ static void
 BMPreadcolormap(FILE *         const ifP, 
                 unsigned int   const cBitCount, 
                 int            const class, 
-                xel                  colormap[], 
+                xel **         const colormapP, 
                 int            const cmapsize,
                 unsigned int * const bytesReadP) {
 
     int i;
+
+    xel * colormap;
+    unsigned int bytesRead;
+
+    colormap = pnm_allocrow(cmapsize);
     
-    *bytesReadP = 0;  /* initial value */
+    bytesRead = 0;  /* initial value */
 
     for (i = 0; i < cmapsize; ++i) {
         /* There is a document that says the bytes are ordered R,G,B,Z,
@@ -324,17 +330,74 @@ BMPreadcolormap(FILE *         const ifP,
 
         PNM_ASSIGN(colormap[i], r, g, b);
 
-        *bytesReadP += 3;
+        bytesRead += 3;
 
         if (class == C_WIN) {
             GetByte(ifP);
-            *bytesReadP += 1;
+            bytesRead += 1;
         }
     }
+    *colormapP = colormap;
+    *bytesReadP = bytesRead;
 }
 
 
 
+static void
+convertRow16(unsigned char const bmprow[],
+             xel                 xelrow[],
+             int           const cols) {
+    /* It's truecolor.  We haven't seen a spec for this format, but we
+       have seen one example of it, produced by National Instruments
+       Lookout, on 2003.06.09, and the encoding of the 16 bit raster
+       element was apparently as follows: It's a signed little-endian
+       integer, which is never negative, and the 15 bit binary
+       representation of that integer can be decomposed into 5 bits
+       each of red, green, and blue going from msb to lsb.  
+    */
+    unsigned int col;
+    unsigned int cursor;
+    cursor = 0;
+    for (col=0; col < cols; ++col) {
+        unsigned short const rasterval = (unsigned short)
+            bmprow[cursor+1] << 8 | bmprow[cursor+0];
+        unsigned int const rbits = (rasterval >> 10) & 0x1F;
+        unsigned int const gbits = (rasterval >> 5)  & 0x1F;
+        unsigned int const bbits = (rasterval >> 0)  & 0x1F;
+
+        pixval const r = (unsigned int) rbits * 255 / 31;
+        pixval const g = (unsigned int) gbits * 255 / 31;
+        pixval const b = (unsigned int) bbits * 255 / 31;
+        
+        PNM_ASSIGN(xelrow[col], r, g, b);
+        
+        cursor += 2;
+    }
+}
+
+
+
+static void
+convertRow24(unsigned char const bmprow[],
+             xel                 xelrow[],
+             int           const cols) {
+    
+    /* It's truecolor */
+    /* There is a document that gives a much different format for
+       24 bit BMPs.  But this seems to be the de facto standard.
+    */
+    unsigned int col;
+    unsigned int cursor;
+    cursor = 0;
+    for (col = 0; col < cols; ++col) {
+        PNM_ASSIGN(xelrow[col], 
+                   bmprow[cursor+2], bmprow[cursor+1], bmprow[cursor+0]);
+        cursor += 3;
+    }
+} 
+
+
+
 static void
 convertRow(unsigned char  const bmprow[], 
            xel                  xelrow[],
@@ -350,21 +413,11 @@ convertRow(unsigned char  const bmprow[],
    If the image is colormapped, colormap[] is the colormap
    (colormap[i] is the color with color index i).
 -----------------------------------------------------------------------------*/
-    if (cBitCount == 24) {
-        /* It's truecolor */
-        /* There is a document that gives a much different format for
-           24 bit BMPs.  But this seems to be the de facto standard.
-        */
-        unsigned int col;
-        unsigned int cursor;
-        cursor = 0;
-        for (col = 0; col < cols; ++col)
-        {
-            PNM_ASSIGN(xelrow[col], 
-                       bmprow[cursor+2], bmprow[cursor+1], bmprow[cursor+0]);
-            cursor += 3;
-        }
-    } else if (cBitCount == 8) {            
+    if (cBitCount == 24) 
+        convertRow24(bmprow, xelrow, cols);
+    else if (cBitCount == 16) 
+        convertRow16(bmprow, xelrow, cols);
+    else if (cBitCount == 8) {            
         /* It's a whole byte colormap index */
         unsigned int col;
         for (col = 0; col < cols; ++col)
@@ -463,10 +516,22 @@ BMPreadraster(FILE * const ifP,
 
 static void
 writeRaster(unsigned char ** const BMPraster,
-            int const cols, int const rows, int format,
-            unsigned int const cBitCount, enum bmpClass const class, 
-            xel colormap[]) {
+            int              const cols, 
+            int              const rows, 
+            int              const format,
+            unsigned int     const cBitCount, 
+            enum bmpClass    const class, 
+            xel                    colormap[]) {
+/*----------------------------------------------------------------------------
+  Write the PNM raster to Standard Output, corresponding to the raw BMP
+  raster BMPraster.  Write the raster assuming the PNM image has 
+  dimensions 'cols' by 'rows' and format 'format'.
 
+  The BMP image is of class 'class' and  has 'cBitCount' bits per pixel.
+
+  If the image is colormapped, colormap[] is the colormap
+  (colormap[i] is the color with color index i).
+-----------------------------------------------------------------------------*/
     xel * xelrow;
     int row;
 
@@ -536,13 +601,18 @@ analyzeColors(xel colormap[], unsigned int const cmapsize, xelval const maxval,
 
 
 static void
-readBmp(FILE * const ifP, unsigned char *** const BMPrasterP, 
-        int * const colsP, int * const rowsP,
-        bool * const grayPresentP, bool * const colorPresentP,
-        unsigned int * const cBitCountP, unsigned int * const classP, 
-        xel * const colormap,
-        bool const verbose) {
+readBmp(FILE *            const ifP, 
+        unsigned char *** const BMPrasterP, 
+        int *             const colsP, 
+        int *             const rowsP,
+        bool *            const grayPresentP, 
+        bool *            const colorPresentP,
+        unsigned int *    const cBitCountP, 
+        enum bmpClass *   const classP, 
+        xel **            const colormapP,
+        bool              const verbose) {
 
+    xel * colormap;  /* malloc'ed */
     unsigned int pos;
 
     /* The following are all information from the BMP headers */
@@ -569,7 +639,11 @@ readBmp(FILE * const ifP, unsigned char *** const BMPrasterP,
 
     if(offBits != BMPoffbits(BMPheader.class, BMPheader.cBitCount, 
                              BMPheader.cmapsize)) {
-        pm_message("warning: offBits is %u, expected %u"
+        pm_message("warning: the BMP header says the raster starts "
+                   "at offset %u bytes into the file (offbits), "
+                   "but that there are %u bytes of information before "
+                   "the raster.  This inconsistency probably means the "
+                   "input file is not a legal BMP file and is unusable."
                    , offBits
                    , BMPoffbits(BMPheader.class, BMPheader.cBitCount, 
                                 BMPheader.cmapsize));
@@ -577,7 +651,7 @@ readBmp(FILE * const ifP, unsigned char *** const BMPrasterP,
     {
         unsigned int bytesRead;
         BMPreadcolormap(ifP, BMPheader.cBitCount, BMPheader.class, 
-                        colormap, BMPheader.cmapsize, &bytesRead);
+                        &colormap, BMPheader.cmapsize, &bytesRead);
         pos += bytesRead;
         if (bytesRead != BMPlencolormap(BMPheader.class, BMPheader.cBitCount, 
                                         BMPheader.cmapsize)) {
@@ -614,6 +688,7 @@ readBmp(FILE * const ifP, unsigned char *** const BMPrasterP,
     *rowsP = BMPheader.rows;
     *cBitCountP = BMPheader.cBitCount;
     *classP = BMPheader.class;
+    *colormapP = colormap;
 }
 
 
@@ -639,7 +714,10 @@ main(int argc, char ** argv) {
         */
     unsigned int cBitCount;
     enum bmpClass class;
-    xel colormap[MAXCOLORS];
+    xel *colormap;
+        /* Malloc'ed colormap (palette) from the BMP.  Undefined if
+           not a colormapped BMP.
+        */
 
     pnm_init(&argc, argv);
 
@@ -652,7 +730,7 @@ main(int argc, char ** argv) {
         ifname = cmdline.input_filespec;
 
     readBmp(ifP, &BMPraster, &cols, &rows, &grayPresent, &colorPresent, 
-            &cBitCount, &class, colormap,
+            &cBitCount, &class, &colormap,
             cmdline.verbose);
     pm_close(ifP);
 
@@ -671,7 +749,11 @@ main(int argc, char ** argv) {
     writeRaster(BMPraster, cols, rows, outputType, cBitCount, class,
                 colormap);
 
+    free(colormap);
     free(BMPraster);
 
     exit(0);
 }
+
+
+
