@@ -10,6 +10,12 @@
 
 /* There is a copy of the GIF89 specification, as defined by its
    inventor, Compuserve, in 1989, at http://members.aol.com/royalef/gif89a.txt
+
+   This covers the high level format, but does not cover how the "data"
+   contents of a GIF image represent the raster of color table indices.
+   An appendix describes extensions to Lempel-Ziv that GIF makes (variable
+   length compression codes and the clear and end codes), but does not
+   describe the Lempel-Ziv base.
 */
 
 #include <string.h>
@@ -33,16 +39,21 @@
 
 #define LM_to_uint(a,b)                        (((b)<<8)|(a))
 
+static int const maxnum_lzwCode = (1<<MAX_LZW_BITS);
+
 struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
     const char *input_filespec;  /* Filespecs of input files */
-    int verbose;    /* -verbose option */
-    int comments;   /* -comments option */
-    int image_no;   /* image number he wants from input */
+    unsigned int verbose;    /* -verbose option */
+    unsigned int comments;   /* -comments option */
+    bool all_images;  /* He wants all the images */
+    unsigned int image_no;
+        /* image number he wants from input, starting at 0.  Undefined
+           if all_images is TRUE
+        */
     const char *alpha_filename;
-    int alpha_stdout;
 };
 
 
@@ -53,35 +64,58 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
+    optEntry *option_def = malloc(100*sizeof(optEntry));
+        /* Instructions to optParseOptions3 on how to parse our options.
          */
-    optStruct2 opt;
+    optStruct3 opt;
+    
+    unsigned int alphaSpec, imageSpec;
+    const char * image;
 
     unsigned int option_def_index;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "verbose",     OPT_FLAG,   &cmdlineP->verbose,         0);
-    OPTENTRY(0,   "comments",    OPT_FLAG,   &cmdlineP->comments,        0);
-    OPTENTRY(0,   "image",       OPT_UINT,   &cmdlineP->image_no,        0);
-    OPTENTRY(0,   "alphaout",    OPT_STRING, &cmdlineP->alpha_filename, 0);
-
-    /* Set the defaults */
-    cmdlineP->verbose = FALSE;
-    cmdlineP->comments = FALSE;
-    cmdlineP->image_no = 1;
-    cmdlineP->alpha_filename = NULL;
+    OPTENT3(0, "verbose",     OPT_FLAG, NULL, 
+            &cmdlineP->verbose,         0);
+    OPTENT3(0, "comments",    OPT_FLAG, NULL,
+            &cmdlineP->comments,        0);
+    OPTENT3(0, "image",       OPT_STRING, &image,
+            &imageSpec,                  0);
+    OPTENT3(0, "alphaout",    OPT_STRING, &cmdlineP->alpha_filename, 
+            &alphaSpec,                 0);
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    optParseOptions2(&argc, argv, opt, 0);
+    optParseOptions3( &argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
-    if (cmdlineP->image_no < 1)
-        pm_error("Invalid -image option: %d.  Must be at least 1.",
-                 cmdlineP->image_no);
+    if (!imageSpec) {
+        cmdlineP->image_no = 0;
+        cmdlineP->all_images = FALSE;
+    } else {
+        if (strcasecmp(image, "all") == 0) 
+            cmdlineP->all_images = TRUE;
+        else {
+            char * tailptr;
+
+            long const imageNo = strtol(image, &tailptr, 10);
+
+            if (*tailptr != '\0')
+                pm_error("Invalid value for '-image' option.  Must be either "
+                         "a number or 'all'.  You specified '%s'", image);
+            else if (imageNo < 0)
+                pm_error("Invalid value for '-image' option.  Must be "
+                         "positive.  You specified %ld", imageNo);
+            else if (imageNo == 0)
+                pm_error("Invalid value for 'image' option.  You specified "
+                         "zero.  The first image is 1.");
+
+            cmdlineP->all_images = FALSE;
+            cmdlineP->image_no = (unsigned int) imageNo - 1;
+        }
+    }
     
     if (argc-1 == 0) 
         cmdlineP->input_filespec = "-";
@@ -91,11 +125,8 @@ parse_command_line(int argc, char ** argv,
     else
         cmdlineP->input_filespec = argv[1];
 
-    if (cmdlineP->alpha_filename && 
-        strcmp(cmdlineP->alpha_filename, "-") == 0 )
-        cmdlineP->alpha_stdout = 1;
-    else 
-        cmdlineP->alpha_stdout = 0;
+    if (!alphaSpec) 
+        cmdlineP->alpha_filename = NULL;
 }
 
 
@@ -512,123 +543,333 @@ getCode(FILE * const ifP,
 }
 
 
+struct stack {
+    /* Stack grows from low addresses to high addresses */
+    int * stack;  /* malloc'ed array */
+    int * sp;     /* stack pointer */
+    int * top;    /* next word above top of stack */
+};
+
+
+
+static void 
+initStack(struct stack * const stackP, unsigned int const size) {
+
+    stackP->stack = malloc(size * sizeof(int));
+    if (stackP->stack == NULL)
+        pm_error("Unable to allocate %d-word stack.", size);
+    stackP->sp = stackP->stack;
+    stackP->top = stackP->stack + size;
+}
+
+
+
+static void
+pushStack(struct stack * const stackP, int const value) {
+
+    if (stackP->sp >= stackP->top)
+        pm_error("stack overflow");
+
+    *(stackP->sp++) = value;
+}
+
+
+
+static bool
+stackIsEmpty(const struct stack * const stackP) {
+    return stackP->sp == stackP->stack;
+}
+
+
 
 static int
-lzwReadByte(FILE * const ifP, bool const first, int const input_codeSize) {
-/*----------------------------------------------------------------------------
-  Return the next byte of the decompressed image.
+popStack(struct stack * const stackP) {
 
-  Return -1 if we hit EOF prematurely (i.e. before an "end" code.  We
-  forgive the case that the "end" code is followed by EOF instead of
-  an EOD marker (zero length DataBlock)).
-
-  Return -2 if there are no more bytes in the image.
------------------------------------------------------------------------------*/
-    static int      fresh = FALSE;
-    int             code, incode;
-    static int      codeSize, set_codeSize;
-    static int      max_code, max_codeSize;
-    static int      firstcode, oldcode;
-    static int      clear_code, end_code;
-    static int      table[2][(1<< MAX_LZW_BITS)];
-    static int      stack[(1<<(MAX_LZW_BITS))*2], *sp;
-
-    if (first) {
-        int    i;
-
-        set_codeSize = input_codeSize;
-        codeSize = set_codeSize+1;
-        clear_code = 1 << set_codeSize ;
-        end_code = clear_code + 1;
-        max_codeSize = 2*clear_code;
-        max_code = clear_code+2;
-
-        getCode(ifP, 0, TRUE);
-               
-        fresh = TRUE;
-
-        for (i = 0; i < clear_code; ++i) {
-            table[0][i] = 0;
-            table[1][i] = i;
-        }
-        for (; i < (1<<MAX_LZW_BITS); ++i)
-            table[0][i] = table[1][i] = 0;
-
-        sp = stack;
-
-        return 0;
-    } else if (fresh) {
-        fresh = FALSE;
-        do {
-            firstcode = oldcode = getCode(ifP, codeSize, FALSE);
-        } while (firstcode == clear_code);
-        return firstcode;
-    }
-
-    if (sp > stack)
-        return *--sp;
-
-    while ((code = getCode(ifP, codeSize, FALSE)) >= 0) {
-        if (code == clear_code) {
-            int    i;
-            for (i = 0; i < clear_code; ++i) {
-                table[0][i] = 0;
-                table[1][i] = i;
-            }
-            for (; i < (1<<MAX_LZW_BITS); ++i)
-                table[0][i] = table[1][i] = 0;
-            codeSize = set_codeSize+1;
-            max_codeSize = 2*clear_code;
-            max_code = clear_code+2;
-            sp = stack;
-            firstcode = oldcode = getCode(ifP, codeSize, FALSE);
-            return firstcode;
-        } else if (code == end_code) {
-            if (zeroDataBlock)
-                return -2;
-
-            readThroughEod(ifP);
-                       
-            return -2;
-        }
-
-        incode = code;
-
-        if (code >= max_code) {
-            *sp++ = firstcode;
-            code = oldcode;
-        }
-
-        while (code >= clear_code) {
-            *sp++ = table[1][code];
-            if (code == table[0][code])
-                pm_error("circular table entry BIG ERROR");
-            code = table[0][code];
-        }
-
-        *sp++ = firstcode = table[1][code];
-
-        if ((code = max_code) <(1<<MAX_LZW_BITS)) {
-            table[0][code] = oldcode;
-            table[1][code] = firstcode;
-            ++max_code;
-            if ((max_code >= max_codeSize) &&
-                (max_codeSize < (1<<MAX_LZW_BITS))) {
-                max_codeSize *= 2;
-                ++codeSize;
-            }
-        }
-
-        oldcode = incode;
-
-        if (sp > stack)
-            return *--sp;
-    }
-    return code;
+    if (stackP->sp <= stackP->stack)
+        pm_error("stack underflow");
+    
+    return *(--stackP->sp);
 }
 
 
 
+static void
+termStack(struct stack * const stackP) {
+    free(stackP->stack);
+    stackP->stack = NULL;
+}
+
+
+/*----------------------------------------------------------------------------
+   Some notes on LZW.
+
+   LZW is an extension of Limpel-Ziv.  The two extensions are:
+
+     1) in Limpel-Ziv, codes are all the same number of bits.  In
+        LZW, they start out small and increase as the stream progresses.
+
+     2) LZW has a clear code that resets the string table and code
+        size.
+
+   The LZW code space is allocated as follows:
+
+   The true data elements are dataWidth bits wide, so the maximum
+   value of a true data element is 2**dataWidth-1.  We call that
+   max_dataVal.  The first byte in the stream tells you what dataWidth
+   is.
+
+   LZW codes 0 - max_dataVal are direct codes.  Each on represents
+   the true data element whose value is that of the LZW code itself.
+   No decompression is required.
+
+   max_dataVal + 1 and up are compression codes.  They encode
+   true data elements:
+
+   max_dataVal + 1 is the clear code.
+         
+   max_dataVal + 2 is the end code.
+
+   max_dataVal + 3 and up are string codes.  Each string code 
+   represents a string of true data elements.  The translation from a
+   string code to the string of true data elements varies as the stream
+   progresses.  In the beginning and after every clear code, the
+   translation table is empty, so no string codes are valid.  As the
+   stream progresses, the table gets filled and more string codes 
+   become valid.
+
+-----------------------------------------------------------------------------*/
+
+
+struct decompressor {
+    struct stack stack;
+    int      fresh;
+        /* The stream is right after a clear code or at the very beginning */
+    int      codeSize;
+        /* The current code size -- each LZW code in this part of the image
+           is this many bits.  Ergo, we read this many bits at a time from
+           the stream.
+        */
+    int      maxnum_code;
+        /* The maximum number of LZW codes that can be represented with the 
+           current code size.  (1 << codeSize)
+        */
+    int      next_tableSlot;
+        /* Index in the code translation table of the next free entry */
+    int      firstcode;
+        /* This is always a true data element code */
+    int      prevcode;
+        /* The code just before, in the image, the one we're processing now */
+    int      table[2][(1 << MAX_LZW_BITS)];
+
+    /* The following are constant for the life of the decompressor */
+    FILE * ifP;
+    int init_codeSize;
+    int max_dataVal;
+    int clear_code;
+    int end_code; 
+};
+
+
+
+static void
+resetDecompressor(struct decompressor * const decompP) {
+
+    decompP->codeSize = decompP->init_codeSize+1;
+    decompP->maxnum_code = 1 << decompP->codeSize;
+    decompP->next_tableSlot = decompP->max_dataVal + 3;
+    decompP->fresh = 1;
+}
+
+
+
+static void
+lzwInit(struct decompressor * const decompP, 
+        FILE *                const ifP,
+        int                   const init_codeSize) {
+
+    if (verbose)
+        pm_message("Image says the initial compression code size is "
+                   "%d bits", 
+                   init_codeSize);
+    
+    decompP->ifP = ifP;
+    decompP->init_codeSize = init_codeSize;
+
+    decompP->max_dataVal = (1 << init_codeSize) - 1;
+    decompP->clear_code = decompP->max_dataVal + 1;
+    decompP->end_code = decompP->max_dataVal + 2;
+    
+    /* The entries in the translation table for true data codes are
+       constant throughout the stream.  We set them now and they never
+       change.
+    */
+    {
+        unsigned int i;
+        for (i = 0; i <= decompP->max_dataVal; ++i) {
+            decompP->table[0][i] = 0;
+            decompP->table[1][i] = i;
+        }
+    }
+    resetDecompressor(decompP);
+
+    getCode(decompP->ifP, 0, TRUE);
+    
+    decompP->fresh = TRUE;
+    
+    initStack(&decompP->stack, maxnum_lzwCode*2);
+}
+
+
+
+static void
+lzwTerm(struct decompressor * const decompP) {
+
+    termStack(&decompP->stack);
+}
+
+
+
+static void
+expandCodeOntoStack(struct decompressor * const decompP,
+                    int                   const incode,
+                    bool *                const errorP) {
+/*----------------------------------------------------------------------------
+   'incode' is an LZW string code.  It represents a string of true data
+   elements, as defined by the string translation table in *decompP.
+
+   Expand the code to a string of LZW direct codes and push them onto the
+   stack such that the leftmost code is on top.
+
+   Also add to the translation table where appropriate.
+
+   Iff the translation table contains a cycle (which means the LZW stream
+   from which it was built is invalid), return *errorP == TRUE.
+-----------------------------------------------------------------------------*/
+    int code;
+    bool error;
+
+    error = FALSE;
+
+    if (incode < decompP->next_tableSlot) 
+        code = incode;
+    else {
+        /* It's a code that isn't in our translation table yet */
+        pushStack(&decompP->stack, decompP->firstcode);
+        code = decompP->prevcode;
+    }
+
+    {
+        /* Get the whole string that this compression code
+           represents and push it onto the code stack so the
+           leftmost code is on top.  Set decompP->firstcode to the
+           first (leftmost) code in that string.
+            */
+
+        unsigned int stringCount;
+        stringCount = 0;
+
+        while (code > decompP->max_dataVal && !error) {
+            if (stringCount > maxnum_lzwCode) {
+                pm_message("Error in GIF image: contains LZW string loop");
+                error = TRUE;
+            } else {
+                ++stringCount;
+                pushStack(&decompP->stack, decompP->table[1][code]);
+                code = decompP->table[0][code];
+            }
+        }
+        decompP->firstcode = decompP->table[1][code];
+        pushStack(&decompP->stack, decompP->firstcode);
+    }
+
+    if (decompP->next_tableSlot < maxnum_lzwCode) {
+        decompP->table[0][decompP->next_tableSlot] = decompP->prevcode;
+        decompP->table[1][decompP->next_tableSlot] = decompP->firstcode;
+        ++decompP->next_tableSlot;
+        if (decompP->next_tableSlot >= decompP->maxnum_code) {
+            /* We've used up all the codes of the current code size.
+               Future codes in the stream will have codes one bit longer.
+               But there's an exception if we're already at the LZW
+               maximum, in which case the codes will simply continue
+               the same size.
+            */
+            if (decompP->codeSize < MAX_LZW_BITS) {
+                ++decompP->codeSize;
+                decompP->maxnum_code = 1 << decompP->codeSize;
+            }
+        }
+    }
+
+    decompP->prevcode = incode;
+    *errorP = error;
+}
+
+
+
+static int
+lzwReadByte(struct decompressor * const decompP) {
+/*----------------------------------------------------------------------------
+  Return the next data element of the decompressed image.  In the context
+  of a GIF, a data element is the color table index of one pixel.
+
+  We read and return the next byte of the decompressed image:
+
+    Return -1 if we hit EOF prematurely (i.e. before an "end" code.  We
+    forgive the case that the "end" code is followed by EOF instead of
+    an EOD marker (zero length DataBlock)).
+
+    Return -2 if there are no more bytes in the image.
+
+    Return -3 if we encounter errors in the LZW stream.
+-----------------------------------------------------------------------------*/
+    int retval;
+
+    if (!stackIsEmpty(&decompP->stack)) 
+        retval = popStack(&decompP->stack);
+    else if (decompP->fresh) {
+        decompP->fresh = FALSE;
+        /* Read off all initial clear codes, read the first non-clear code,
+           and return it.  There are no strings in the table yet, so the next
+           code must be a direct true data code.
+        */
+        do {
+            decompP->firstcode =
+                getCode(decompP->ifP, decompP->codeSize, FALSE);
+            decompP->prevcode = decompP->firstcode;
+        } while (decompP->firstcode == decompP->clear_code);
+        retval = decompP->firstcode;
+    } else {
+        int code;
+        code = getCode(decompP->ifP, decompP->codeSize, FALSE);
+        if (code < 0)
+            retval = code;
+        else {
+            if (code == decompP->clear_code) {
+                resetDecompressor(decompP);
+                retval = lzwReadByte( decompP );
+            } else if (code == decompP->end_code) {
+                if (!zeroDataBlock)
+                    readThroughEod(decompP->ifP);
+                retval = -2;
+            } else {
+                bool error;
+                expandCodeOntoStack(decompP, code, &error);
+                if (error)
+                    retval = -3;
+                else
+                    retval = popStack(&decompP->stack);
+            }
+        }
+    }
+    return retval;
+}
+
+
+
+
+
+
+
 static void
 readImageData(FILE * const ifP, 
               xel ** const xels, 
@@ -640,6 +881,7 @@ readImageData(FILE * const ifP,
     unsigned char lzwMinCodeSize;      
     int v;
     int xpos, ypos, pass;
+    struct decompressor decomp;
 
     pass = 0;
     xpos = 0;
@@ -654,15 +896,13 @@ readImageData(FILE * const ifP,
                  "right after an image separator; no "
                  "image data follows.");
 
-    if (lzwReadByte(ifP, TRUE, lzwMinCodeSize) < 0)
-        pm_error("GIF stream ends (or read error) right after the "
-                 "minimum lzw code size field; no image data follows.");
+    lzwInit(&decomp, ifP, lzwMinCodeSize);
 
     if (verbose)
         pm_message("reading %d by %d%s GIF image",
                    len, height, interlace ? " interlaced" : "" );
 
-    while ((v = lzwReadByte(ifP,FALSE,lzwMinCodeSize)) >= 0 ) {
+    while ((v = lzwReadByte(&decomp)) >= 0 ) {
         PPM_ASSIGN(xels[ypos][xpos], 
                    cmap[CM_RED][v], cmap[CM_GRN][v], cmap[CM_BLU][v]);
 
@@ -700,11 +940,14 @@ readImageData(FILE * const ifP,
         if (ypos >= height)
             break;
     }
-    
 fini:
-    if (lzwReadByte(ifP,FALSE,lzwMinCodeSize)>=0)
-        pm_message("too much input data, ignoring extra...");
-
+    if (v == -3)
+        pm_error("Error in GIF input stream");
+    else {
+        if (lzwReadByte(&decomp)>=0)
+            pm_message("too much input data, ignoring extra...");
+    }
+    lzwTerm(&decomp);
 }
 
 
@@ -713,6 +956,12 @@ static void
 writePnm(FILE *outfile, xel ** const xels, 
          const int cols, const int rows,
          const int hasGray, const int hasColor) {
+/*----------------------------------------------------------------------------
+   Write a PNM image to the current position of file 'outfile' with
+   dimensions 'cols' x 'rows' and raster 'xels'.
+   
+   Make it PBM, PGM, or PBM according to 'hasGray' and 'hasColor'.
+-----------------------------------------------------------------------------*/
     int format;
     const char *format_name;
            
@@ -934,7 +1183,13 @@ convertImage(FILE *           const ifP,
              FILE *           const alphafile, 
              struct gifScreen       gifScreen,
              struct gif89     const gif89) {
+/*----------------------------------------------------------------------------
+   Read a single GIF image from the current position of file 'ifP'.
 
+   If 'skipIt' is TRUE, don't do anything else.  Otherwise, write the
+   image to the current position of files 'imageout_file' and 'alphafile'.
+   If 'alphafile' is NULL, though, don't write any alpha information.
+-----------------------------------------------------------------------------*/
     unsigned char buf[16];
     bool useGlobalColormap;
     xel **xels;  /* The image raster, in libpnm format */
@@ -988,10 +1243,21 @@ convertImage(FILE *           const ifP,
 
 static void
 convertImages(FILE * const ifP, 
+              bool   const allImages,
               int    const requestedImageSeq, 
               FILE * const imageout_file, 
               FILE * const alphafile) {
+/*----------------------------------------------------------------------------
+   Read a GIF stream from file 'ifP' and write one or more images from
+   it as PNM images to file 'imageout_file'.  If the images have transparency
+   and 'alphafile' is non-NULL, write PGM alpha masks to file 'alphafile'.
 
+   'allImages' means Caller wants all the images in the stream.  
+
+   'requestedImageSeq' is meaningful only when 'allImages' is FALSE.  It 
+   is the sequence number of the one image Caller wants from the stream,
+   with the first image being 0.
+-----------------------------------------------------------------------------*/
     int imageSeq;
         /* Sequence within GIF stream of image we are currently processing.
            First is 0.
@@ -1011,14 +1277,16 @@ convertImages(FILE * const ifP,
 
         if (eod) {
             /* GIF stream ends before image with sequence imageSeq */
-            if (imageSeq <= requestedImageSeq)
+            if (!allImages && (imageSeq <= requestedImageSeq))
                 pm_error("You requested Image %d, but "
                          "only %d image%s found in GIF stream",
                          requestedImageSeq+1,
                          imageSeq, imageSeq>1?"s":"" );
-        } else 
-            convertImage(ifP, imageSeq != requestedImageSeq, 
+        } else {
+            pm_message("Reading Image Sequence %d", imageSeq);
+            convertImage(ifP, !allImages && (imageSeq != requestedImageSeq), 
                          imageout_file, alphafile, gifScreen, gif89);
+        }
     }
 }
 
@@ -1039,20 +1307,18 @@ main(int argc, char **argv) {
    
     ifP = pm_openr(cmdline.input_filespec);
 
-    if (cmdline.alpha_stdout)
-        alpha_file = stdout;
-    else if (cmdline.alpha_filename == NULL) 
+    if (cmdline.alpha_filename == NULL)
         alpha_file = NULL;
-    else {
+    else
         alpha_file = pm_openw(cmdline.alpha_filename);
-    }
 
-    if (cmdline.alpha_stdout) 
+    if (alpha_file && strcmp(cmdline.alpha_filename, "-") == 0)
         imageout_file = NULL;
     else
         imageout_file = stdout;
 
-    convertImages(ifP, cmdline.image_no-1, imageout_file, alpha_file);
+    convertImages(ifP, cmdline.all_images, cmdline.image_no, 
+                  imageout_file, alpha_file);
 
     pm_close(ifP);
     if (imageout_file != NULL) 
