@@ -248,7 +248,8 @@ png_const_charp msg;
 
 #define SIG_CHECK_SIZE 4
 
-static int convertpng (FILE *ifp, FILE *tfp)
+static void 
+convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 {
   unsigned char sig_buf [SIG_CHECK_SIZE];
   png_struct *png_ptr;
@@ -272,16 +273,15 @@ static int convertpng (FILE *ifp, FILE *tfp)
    * in the absence of any other error condition) */
   static const char *type_string;
   static const char *alpha_string;
-  static int errorlevel;
 
-  errorlevel = 0;
+  *errorlevelP = 0;
   type_string = alpha_string = "";
 
   if (fread (sig_buf, 1, SIG_CHECK_SIZE, ifp) != SIG_CHECK_SIZE) {
     pm_closer (ifp);
     pm_error ("input file is empty or too short");
   }
-  if (png_sig_cmp (sig_buf, (png_size_t) 0, (png_size_t) SIG_CHECK_SIZE) != 0) {
+  if (png_sig_cmp(sig_buf, (png_size_t) 0, (png_size_t) SIG_CHECK_SIZE) != 0) {
     pm_closer (ifp);
     pm_error ("input file is not a PNG file");
   }
@@ -466,7 +466,7 @@ static int convertpng (FILE *ifp, FILE *tfp)
             alpha == none) {
 	  pm_message ("different bit depths for color channels not supported");
 	  pm_message ("writing file with %d bit resolution", info_ptr->bit_depth);
-	  errorlevel = PNMTOPNG_WARNING_LEVEL;
+	  *errorlevelP = PNMTOPNG_WARNING_LEVEL;
         } else {
           if ((info_ptr->color_type == PNG_COLOR_TYPE_PALETTE) &&
 	      (info_ptr->sig_bit.red < 255)) {
@@ -576,7 +576,7 @@ static int convertpng (FILE *ifp, FILE *tfp)
                   "to fix do a 'pamscale -%cscale %g'",
 		    r < 1.0 ? 'x' : 'y',
 		    r < 1.0 ? 1.0 / r : r );
-	  errorlevel = PNMTOPNG_WARNING_LEVEL;
+	  *errorlevelP = PNMTOPNG_WARNING_LEVEL;
     }
   }
 
@@ -713,10 +713,10 @@ static int convertpng (FILE *ifp, FILE *tfp)
     free (png_image[y]);
   free (png_image);
   png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-
-  return errorlevel;
 }
 
+
+
 int main (int argc, char *argv[])
 {
   FILE *ifp, *tfp;
@@ -782,7 +782,7 @@ int main (int argc, char *argv[])
   else
     tfp = NULL;
 
-  errorlevel = convertpng (ifp, tfp);
+  convertpng (ifp, tfp, &errorlevel);
 
   if (text)
     pm_closew (tfp);
