@@ -36,6 +36,16 @@
 static const char * progname;	/* program name for error messages */
 
 enum restart_unit {RESTART_MCU, RESTART_ROW, RESTART_NONE};
+enum density_unit {DEN_UNSPECIFIED, DEN_DOTS_PER_INCH, DEN_DOTS_PER_CM};
+
+struct density {
+    enum density_unit unit;
+        /* The units of density for 'horiz', 'vert' */
+    unsigned short horiz;  /* Not 0 */
+        /* Horizontal density, in units specified by 'unit' */
+    unsigned short vert;   /* Not 0 */
+        /* Same as 'horiz', but vertical */
+};
 
 struct cmdline_info {
     /* All the information the user supplied in the command line,
@@ -63,6 +73,11 @@ struct cmdline_info {
     char *restart;
     char *comment;              /* NULL if none */
     const char *exif_filespec;  /* NULL if none */
+    unsigned int density_spec;
+        /* boolean: JFIF should specify a density.  If false, 'density'
+           is undefined.
+        */
+    struct density density;
 } cmdline;
 
 static void 
@@ -126,6 +141,61 @@ interpret_restart(const char * const restart,
 
 
 
+
+static void
+interpret_density(const char *        const densityString,
+                  struct density *    const densityP) {
+/*----------------------------------------------------------------------------
+   Interpret the value of the "-density" option.
+-----------------------------------------------------------------------------*/
+    if (strlen(densityString) < 1)
+        pm_error("-density value cannot be null.");
+    else {
+        char * unitName;  /* malloc'ed */
+        int matched;
+        int horiz, vert;
+
+        unitName = malloc(strlen(densityString));
+    
+        matched = sscanf(densityString, "%dx%d%s", &horiz, &vert, unitName);
+
+        if (matched < 2)
+            pm_error("Invalid format for density option value '%s'.  It "
+                     "should follow the example '3x2' or '3x2dpi' or "
+                     "'3x2dpcm'.", densityString);
+        else {
+            if (horiz <= 0 || horiz >= 1<<16)
+                pm_error("Horizontal density %d is outside the range 1-65535",
+                         horiz);
+            else if (vert <= 0 || vert >= 1<<16)
+                pm_error("Vertical density %d is outside the range 1-65535",
+                         vert);
+            else {
+                densityP->horiz = horiz;
+                densityP->vert  = vert;
+
+                if (matched < 3) 
+                    densityP->unit = DEN_UNSPECIFIED;
+                else {
+                    if (strcmp(unitName, "dpi") == 0 || 
+                        strcmp(unitName, "DPI") == 0)
+                        densityP->unit = DEN_DOTS_PER_INCH;
+                    else if (strcmp(unitName, "dpcm") == 0 || 
+                        strcmp(unitName, "DPCM") == 0)
+                        densityP->unit = DEN_DOTS_PER_CM;
+                    else
+                        pm_error("Unrecognized unit '%s' in the density value "
+                                 "'%s'.  I recognize only 'dpi' and 'dpcm'",
+                                 unitName, densityString);
+                }
+            }
+        }
+        free(unitName);
+    }
+}
+
+
+
 static void
 parse_command_line(const int argc, char ** argv,
                    struct cmdline_info *cmdline_p) {
@@ -138,16 +208,17 @@ parse_command_line(const int argc, char ** argv,
    On the other hand, unlike other option processing functions, we do
    not change argv at all.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optStruct));
+    optEntry *option_def = malloc(100*sizeof(optEntry));
         /* Instructions to OptParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
 
     int i;  /* local loop variable */
 
-    char *dctval;
-    char *maxmemory;
-    char *restart;
+    const char *dctval;
+    const char *maxmemory;
+    const char *restart;
+    const char *density;
 
     unsigned int qualitySpec, smoothSpec;
 
@@ -175,12 +246,14 @@ parse_command_line(const int argc, char ** argv,
     OPTENT3(0, "sample",      OPT_STRING, &cmdline_p->sample,      NULL,    0);
     OPTENT3(0, "scans",       OPT_STRING, &cmdline_p->scans,       NULL,    0);
     OPTENT3(0, "smooth",      OPT_UINT,   &cmdline_p->smoothing_factor, 
-            &smoothSpec, 0);
+            &smoothSpec,  0);
     OPTENT3(0, "optimize",    OPT_FLAG,   NULL, &cmdline_p->optimize,       0);
     OPTENT3(0, "optimise",    OPT_FLAG,   NULL, &cmdline_p->optimize,       0);
     OPTENT3(0, "restart",     OPT_STRING, &restart, NULL,                   0);
     OPTENT3(0, "comment",     OPT_STRING, &cmdline_p->comment, NULL,        0);
     OPTENT3(0, "exif",        OPT_STRING, &cmdline_p->exif_filespec, NULL,  0);
+    OPTENT3(0, "density",     OPT_STRING, &density, 
+            &cmdline_p->density_spec, 0);
 
     /* Set the defaults */
     dctval = NULL;
@@ -236,7 +309,9 @@ parse_command_line(const int argc, char ** argv,
     interpret_maxmemory(maxmemory, &cmdline_p->max_memory_to_use);
     interpret_restart(restart, &cmdline_p->restart_value,
                       &cmdline_p->restart_unit);
-
+    if (cmdline_p->density_spec) 
+        interpret_density(density, &cmdline_p->density);
+    
     if (cmdline_p->smoothing_factor > 100)
         pm_error("Smoothing factor %d is greater than 100 (%%).",
                  cmdline_p->smoothing_factor);
@@ -327,6 +402,25 @@ setup_jpeg_source_parameters(struct jpeg_compress_struct * const cinfoP,
 
 
 
+static void
+setup_jpeg_density(struct jpeg_compress_struct * const cinfoP, 
+                   struct density                const density) {
+/*----------------------------------------------------------------------------
+   Set up in the compressor descriptor *cinfoP the density information
+   'density'.
+-----------------------------------------------------------------------------*/
+    switch(density.unit) {
+    case DEN_UNSPECIFIED:   cinfoP->density_unit = 0; break;
+    case DEN_DOTS_PER_INCH: cinfoP->density_unit = 1; break;
+    case DEN_DOTS_PER_CM:   cinfoP->density_unit = 2; break;
+    }
+    
+    cinfoP->X_density = density.horiz;
+    cinfoP->Y_density = density.vert;
+}
+
+
+
 static void
 setup_jpeg(struct jpeg_compress_struct * const cinfoP,
            struct jpeg_error_mgr       * const jerrP,
@@ -404,6 +498,9 @@ setup_jpeg(struct jpeg_compress_struct * const cinfoP,
     if (cmdline.progressive)
         jpeg_simple_progression(cinfoP);
 
+    if (cmdline.density_spec)
+        setup_jpeg_density(cinfoP, cmdline.density);
+
     if (cmdline.scans != NULL) {
         if (! read_scan_script(cinfoP, cmdline.scans)) {
             pm_message("Error in scan script '%s'.", cmdline.scans);
@@ -422,7 +519,6 @@ setup_jpeg(struct jpeg_compress_struct * const cinfoP,
 
 
 
-
 static void
 write_exif_header(struct jpeg_compress_struct * const cinfoP,
                   const char * const exif_filespec) {
@@ -826,7 +922,7 @@ read_scan_script (j_compress_ptr cinfo, char * filename)
          * want JPOOL_PERMANENT.  
          */
         const unsigned int scan_info_size = nscans * sizeof(jpeg_scan_info);
-        jpeg_scan_info *scan_info = 
+        jpeg_scan_info * const scan_info = 
             (jpeg_scan_info *)
             (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_IMAGE,
                                         scan_info_size);
