@@ -33,7 +33,7 @@
 
    This shouldn't cause I/O delays because the entire image ought to fit
    in the system's I/O cache (remember that the file is a lot smaller than
-   the xel array you'd get by doing a pnm_pnmread() of it).
+   the xel array you'd get by doing a pnm_readpnm() of it).
 
    However, it does introduce some delay because of all the system calls 
    required to read the file.  A future enhancement might read the entire
@@ -45,7 +45,7 @@
 
    Before May 2001, we did in fact read the whole image into an xel array,
    and we got complaints.  Before April 2000, it wasn't as big a problem
-   because xels were only 8 bits.  Now they're 32.
+   because xels were only 24 bits.  Now they're 96.
 */
    
 #define GRR_GRAY_PALETTE_FIX
@@ -59,8 +59,7 @@
 #include <png.h>	/* includes zlib.h and setjmp.h */
 #define VERSION "2.37.6 (21 July 2001) +netpbm"
 #include "pnm.h"
-
-#include "ppmcmap.h"
+#include "pngtxt.h"
 
 typedef struct cahitem {
     xel color;
@@ -93,7 +92,6 @@ typedef struct _jmpbuf_wrapper {
 #  define NONE 0
 #endif
 #define MAXCOLORS 256
-#define MAXCOMMENTS 256
 
 /* PALETTEMAXVAL is the maxval used in a PNG palette */
 #define PALETTEMAXVAL 255
@@ -101,11 +99,8 @@ typedef struct _jmpbuf_wrapper {
 #define PALETTEOPAQUE 255
 #define PALETTETRANSPARENT 0
 
-#define SQR(a) ((a)*(a))
-
 /* function prototypes */
 #ifdef __STDC__
-static void read_text (png_info *info_ptr, FILE *tfp);
 static void pnmtopng_error_handler (png_structp png_ptr, png_const_charp msg);
 int main (int argc, char *argv[]);
 #endif
@@ -115,7 +110,7 @@ static int interlace = FALSE;
 static int downscale = FALSE;
 static int transparent = -1;
 static char *transstring;
-static int alpha = FALSE;
+static bool alpha = FALSE;
 static char *alpha_file;
 static int background = -1;
 static char *backstring;
@@ -215,7 +210,7 @@ alloccoloralphahash(void)  {
     int i;
 
     caht = (coloralphahash_table) malloc(HASH_SIZE * sizeof(cahitem *));
-    if (caht == 0)
+    if (caht == NULL)
         pm_error( "out of memory allocating hash table" );
 
     for (i = 0; i < HASH_SIZE; ++i)
@@ -282,98 +277,6 @@ lookupColorAlpha(coloralphahash_table const caht,
 
 
 
-
-
-#ifdef __STDC__
-static void read_text (png_info *info_ptr, FILE *tfp)
-#else
-static void read_text (info_ptr, tfp)
-png_info *info_ptr;
-FILE *tfp;
-#endif
-{
-  char textline[256];
-  int textpos;
-  int i, j;
-  int c;
-  char *cp;
-  const char * const texterr = 
-      "pnmtopng:  unable to allocate memory for text chunks\n";
-
-  info_ptr->text = (png_text *)malloc (MAXCOMMENTS * sizeof (png_text));
-  if (!info_ptr->text) {
-    fprintf(stderr, texterr);
-    fflush(stderr);
-    return;   /* not much else can do */
-  }
-  j = 0;
-  textpos = 0;
-  while ((c = getc (tfp)) != EOF) {
-    if (c != '\n' && c != EOF) {
-      textline[textpos++] = c;
-    } else {
-      textline[textpos++] = '\0';
-      if (textline[0] == '\0') {   /* check for keyword-on-own-line case */
-        textpos = 0;
-        continue;
-      }
-      if ((textline[0] != ' ') && (textline[0] != '\t')) {
-        /* the following is a not that accurate check on Author or Title */
-        if ((!ztxt) || (textline[0] == 'A') || (textline[0] == 'T'))
-          info_ptr->text[j].compression = -1;
-        else
-          info_ptr->text[j].compression = 0;
-        cp = malloc (textpos);
-        if (!cp) {
-          fprintf(stderr, texterr);
-          fflush(stderr);
-          return;   /* not much else can do */
-        }
-        info_ptr->text[j].key = cp;
-        i = 0;
-        if (textline[0] == '"') {
-          i++;
-          while (textline[i] != '"' && textline[i] != '\n' &&
-                 textline[i] != '\0')
-            *(cp++) = textline[i++];
-          i++;
-        } else {
-          while (textline[i] != ' '  && textline[i] != '\t' &&
-                 textline[i] != '\n' && textline[i] != '\0')
-            *(cp++) = textline[i++];
-        }
-        *(cp++) = '\0';
-        cp = malloc (textpos);
-        if (!cp) {
-          fprintf(stderr, texterr);
-          fflush(stderr);
-          return;   /* not much else can do */
-        }
-        info_ptr->text[j].text = cp;
-        while (textline[i] == ' ' || textline[i] == '\t')
-          i++;
-        strcpy (cp, &textline[i]);
-        info_ptr->text[j].text_length = strlen (cp);
-        j++;
-      } else {
-        j--;
-        cp = malloc (info_ptr->text[j].text_length + textpos);
-        strcpy (cp, info_ptr->text[j].text);
-        strcat (cp, "\n");
-        info_ptr->text[j].text = cp;
-        i = 0;
-        while (textline[i] == ' ' || textline[i] == '\t')
-        i++;
-        strcat (cp, &textline[i]);
-        info_ptr->text[j].text_length = strlen (cp);
-        j++;
-      }
-      textpos = 0;
-    }
-  } /* end while */
-  info_ptr->num_text = j;
-}
-
 #ifdef __STDC__
 static void pnmtopng_error_handler (png_structp png_ptr, png_const_charp msg)
 #else
@@ -417,7 +320,7 @@ static int getChv_colors;
 
 static void
 getChv(FILE *             const ifP, 
-       int                const imagepos,
+       pm_filepos         const imagepos,
        int                const cols, 
        int                const rows, 
        xelval             const maxval,
@@ -446,7 +349,7 @@ getChv(FILE *             const ifP,
         if (verbose) 
             pm_message ("Finding colors in input image...");
 
-        pm_seek(ifP, imagepos);
+        pm_seek2(ifP, &imagepos, sizeof(imagepos));
         getChv_chv = ppm_computecolorhist2(ifP, cols, rows, maxval, format, 
                                            MAXCOLORS, &getChv_colors);
         
@@ -474,7 +377,7 @@ static void freeChv(void) {
 
 static void
 meaningful_bits_pgm(FILE *         const ifp, 
-                    int            const imagepos, 
+                    pm_filepos     const imagepos, 
                     int            const cols,
                     int            const rows,
                     xelval         const maxval,
@@ -500,7 +403,7 @@ meaningful_bits_pgm(FILE *         const ifp,
 
     if (maxval == 65535) {
         mayscale = TRUE;   /* initial assumption */
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
@@ -514,7 +417,7 @@ meaningful_bits_pgm(FILE *         const ifp,
     }
     if (maxval == 255 || *retvalP == 8) {
         mayscale = TRUE;  /* initial assumption */
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
@@ -529,7 +432,7 @@ meaningful_bits_pgm(FILE *         const ifp,
 
     if (maxval == 15 || *retvalP == 4) {
         mayscale = TRUE;   /* initial assumption */
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
@@ -544,7 +447,7 @@ meaningful_bits_pgm(FILE *         const ifp,
 
     if (maxval == 3 || *retvalP == 2) {
         mayscale = TRUE;   /* initial assumption */
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
@@ -562,7 +465,7 @@ meaningful_bits_pgm(FILE *         const ifp,
 
 static void
 meaningful_bits_ppm(FILE *         const ifp, 
-                    int            const imagepos, 
+                    pm_filepos     const imagepos, 
                     int            const cols,
                     int            const rows,
                     xelval         const maxval,
@@ -588,7 +491,7 @@ meaningful_bits_ppm(FILE *         const ifp,
 
     if (maxval == 65535) {
         mayscale = TRUE;   /* initial assumption */
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
@@ -608,14 +511,14 @@ meaningful_bits_ppm(FILE *         const ifp,
 
 
 static void
-alpha_trans(FILE *  const ifp, 
-            int     const imagepos, 
-            int     const cols, 
-            int     const rows, 
-            xelval  const maxval, int const format, 
-            gray ** const alpha_mask, gray alpha_maxval,
-            bool *  const alpha_can_be_transparency_indexP, 
-            pixel*  const alpha_transcolorP) {
+alpha_trans(FILE *     const ifp, 
+            pm_filepos const imagepos, 
+            int        const cols, 
+            int        const rows, 
+            xelval     const maxval, int const format, 
+            gray **    const alpha_mask, gray alpha_maxval,
+            bool *     const alpha_can_be_transparency_indexP, 
+            pixel*     const alpha_transcolorP) {
 /*----------------------------------------------------------------------------
   Check if the alpha mask can be represented by a single transparency
   value (i.e. all colors fully opaque except one fully transparent;
@@ -644,7 +547,7 @@ alpha_trans(FILE *  const ifp,
            image that the alpha mask says should be transparent.
         */
         found_transparent_pixel = FALSE;  /* initial assumption */
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         for (row = 0 ; row < rows && !found_transparent_pixel ; ++row) {
             int col;
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
@@ -659,7 +562,7 @@ alpha_trans(FILE *  const ifp,
 
     if (found_transparent_pixel) {
         int row;
-        pm_seek(ifp, imagepos);
+        pm_seek2(ifp, &imagepos, sizeof(imagepos));
         retval = TRUE;  /* initial assumption */
         
         for (row = 0 ; row < rows && retval == TRUE; ++row) {
@@ -705,6 +608,40 @@ alpha_trans(FILE *  const ifp,
 
 
 
+static void
+findRedundantBits(FILE * const ifp, 
+                  int const imagepos, 
+                  int const cols,
+                  int const rows,
+                  xelval const maxval,
+                  int const format,
+                  bool const alpha,
+                  bool const force,
+                  unsigned int * const meaningfulBitsP) {
+/*----------------------------------------------------------------------------
+   Find out if we can use just a subset of the bits from each input sample.
+   Often, people create an image with e.g. 8 bit samples from one that has
+   e.g. only 4 bit samples by just repeating the bits.  1011 becomes
+   10111011.  We detect this case.  We return as *meaningfulBitsP the
+   minimum number of bits, starting from the least significant end, that
+   contain original information.
+-----------------------------------------------------------------------------*/
+  if (!alpha && PNM_FORMAT_TYPE(format) == PGM_TYPE && !force) 
+      meaningful_bits_pgm(ifp, imagepos, cols, rows, maxval, format,
+                          meaningfulBitsP);
+  else if (PNM_FORMAT_TYPE(format) == PPM_TYPE && !force)
+      meaningful_bits_ppm(ifp, imagepos, cols, rows, maxval, format,
+                          meaningfulBitsP);
+  else 
+      *meaningfulBitsP = pm_maxvaltobits(maxval);
+
+  if (verbose && *meaningfulBitsP != pm_maxvaltobits(maxval))
+      pm_message("Using only %d rightmost bits of input samples.  The "
+                 "rest are just copies of those.", *meaningfulBitsP);
+}
+
+
+
 static void
 readOrderedPalette(FILE * const   pfp,
                    xel            ordered_palette[], 
@@ -817,7 +754,7 @@ compute_alpha_palette(FILE *         const ifP,
                       int            const rows,
                       xelval         const maxval,
                       int            const format,
-                      int            const imagepos,
+                      pm_filepos     const imagepos,
                       gray **        const alpha_mask,
                       pixel                palette_pnm[],
                       gray                 trans_pnm[],
@@ -882,7 +819,7 @@ compute_alpha_palette(FILE *         const ifP,
         alphas_of_color_cnt[color_index] = 0;
     }
  
-    pm_seek(ifP, imagepos);
+    pm_seek2(ifP, &imagepos, sizeof(imagepos));
 
     xelrow = pnm_allocrow(cols);
 
@@ -1036,25 +973,32 @@ static void
 findOrAddBackgroundInPalette(pixel          const backColor, 
                              pixel                palette_pnm[], 
                              unsigned int * const paletteSizeP,
-                             unsigned int   const transSize,
                              unsigned int * const backgroundIndexP) {
 /*----------------------------------------------------------------------------
-  Add the background color to the palette, unless there's no room for
-  it, in which case choose a background color that's already in the
-  palette.
+  Add the background color 'backColor' to the palette, unless
+  it's already in there.  If it's not present and there's no room to
+  add it, choose a background color that's already in the palette,
+  as close to 'backColor' as possible.
+
+  If we add an entry to the palette, make it opaque.  But in searching the 
+  existing palette, ignore transparency.
+
+  Note that PNG specs say that transparency of the background is meaningless;
+  i.e. a viewer must ignore the transparency of the palette entry when 
+  using the background color.
 -----------------------------------------------------------------------------*/
     int backgroundIndex;  /* negative means not found */
     unsigned int paletteIndex;
 
     backgroundIndex = -1;
-    for (paletteIndex = transSize; 
+    for (paletteIndex = 0; 
          paletteIndex < *paletteSizeP; 
          ++paletteIndex) 
         if (PPM_EQUAL(palette_pnm[paletteIndex], backColor))
             backgroundIndex = paletteIndex;
 
     if (backgroundIndex >= 0) {
-        /* The background color, opaque, is already in the palette. */
+        /* The background color is already in the palette. */
         *backgroundIndexP = backgroundIndex;
         if (verbose) {
             pixel const p = palette_pnm[*backgroundIndexP];
@@ -1066,6 +1010,9 @@ findOrAddBackgroundInPalette(pixel          const backColor,
         if (*paletteSizeP < MAXCOLORS) {
             /* There's room, so just add it to the end of the palette */
 
+            /* Because we're not expanding the transparency palette, this
+               entry is not in it, and is thus opaque.
+            */
             *backgroundIndexP = (*paletteSizeP)++;
             palette_pnm[*backgroundIndexP] = backColor;
             if (verbose) {
@@ -1094,18 +1041,22 @@ buildColorLookup(pixel                   palette_pnm[],
                  unsigned int      const paletteSize,
                  colorhash_table * const chtP) {
 /*----------------------------------------------------------------------------
-   Create a colorhash_table out of the palette palette_pnm[] (which has
-   'paletteSize' entries) so one can look up the palette index of a given
-   color.  
+   Create a colorhash_table out of the palette described by
+   palette_pnm[] (which has 'paletteSize' entries) so one can look up
+   the palette index of a given color.
 
-   Assume no color appears in palette_pnm[] twice.
+   Where the same color appears twice in the palette, the lookup table
+   finds an arbitrary one of them.  We don't consider transparency of
+   palette entries, so if the same color appears in the palette once
+   transparent and once opaque, the lookup table finds an arbitrary one
+   of those two.
 -----------------------------------------------------------------------------*/
     colorhash_table const cht = ppm_alloccolorhash();
     unsigned int paletteIndex;
 
-    for (paletteIndex = 0; paletteIndex < paletteSize; ++paletteIndex) 
+    for (paletteIndex = 0; paletteIndex < paletteSize; ++paletteIndex) {
         ppm_addtocolorhash(cht, &palette_pnm[paletteIndex], paletteIndex);
-
+    }
     *chtP = cht;
 }
 
@@ -1139,6 +1090,112 @@ buildColorAlphaLookup(pixel              palette_pnm[],
 
 
 
+static void computeColorMapLookupTable(
+    bool                   const colorMapped,
+    pixel                        palette_pnm[],
+    unsigned int           const palette_size,
+    gray                         trans_pnm[],
+    unsigned int           const trans_size,
+    bool                   const alpha,
+    xelval                 const alpha_maxval,
+    colorhash_table *      const chtP,
+    coloralphahash_table * const cahtP) {
+/*----------------------------------------------------------------------------
+   Compute applicable lookup tables for the palette index.  If there's no
+   alpha mask, this is just a standard Netpbm colorhash_table.  If there's
+   an alpha mask, it is the slower Pnmtopng-specific 
+   coloralphahash_table.
+
+   If a lookup table is not applicable to the image, return NULL as
+   its address.  (If the image is not colormapped, both will be NULL).
+-----------------------------------------------------------------------------*/
+    if (colorMapped) {
+        if (alpha) {
+            buildColorAlphaLookup(palette_pnm, palette_size, 
+                                  trans_pnm, trans_size, alpha_maxval, cahtP);
+            *chtP = NULL;
+        } else { 
+            buildColorLookup(palette_pnm, palette_size, chtP);
+            *cahtP = NULL;
+        }
+        if (verbose)
+            pm_message("PNG palette has %u entries, %u of them non-opaque",
+                       palette_size, trans_size);
+    } else {
+        *chtP = NULL;
+        *cahtP = NULL;
+    }
+}
+
+
+
+static void
+computePixelWidth(bool           const colorMapped,
+                  unsigned int   const palette_size,
+                  int            const pnm_type,
+                  unsigned int   const pnm_meaningful_bits,
+                  bool           const alpha,
+                  unsigned int * const bitsPerSampleP,
+                  unsigned int * const bitsPerPixelP) {
+/*----------------------------------------------------------------------------
+   Compute the number of bits per raster sample and per raster pixel:
+   *bitsPerSampleP and *bitsPerPixelP.  Note that a raster element may be a
+   palette index, or a gray value or color with or without alpha mask.
+-----------------------------------------------------------------------------*/
+    if (colorMapped) {
+        if (palette_size <= 2)
+            *bitsPerSampleP = 1;
+        else if (palette_size <= 4)
+            *bitsPerSampleP = 2;
+        else if (palette_size <= 16)
+            *bitsPerSampleP = 4;
+        else
+            *bitsPerSampleP = 8;
+        *bitsPerPixelP = *bitsPerSampleP;
+        if (verbose)
+            pm_message("Writing %d-bit color indexes", *bitsPerSampleP);
+    } else {
+        if (pnm_type == PPM_TYPE || alpha) {
+            /* PNG allows only depths of 8 and 16 for a truecolor image 
+               and for a grayscale image with an alpha channel.
+          */
+            if (pnm_meaningful_bits > 8)
+                *bitsPerSampleP = 16;
+            else 
+                *bitsPerSampleP = 8;
+        } else {
+            /* A grayscale, non-colormapped, no-alpha PNG may have any 
+             bit depth from 1 to 16
+          */
+            if (pnm_meaningful_bits > 8)
+                *bitsPerSampleP = 16;
+            else if (pnm_meaningful_bits > 4)
+                *bitsPerSampleP = 8;
+            else if (pnm_meaningful_bits > 2)
+                *bitsPerSampleP = 4;
+            else if (pnm_meaningful_bits > 1)
+                *bitsPerSampleP = 2;
+            else
+                *bitsPerSampleP = 1;
+        }
+        if (alpha) {
+            if (pnm_type == PPM_TYPE)
+                *bitsPerPixelP = 4 * *bitsPerSampleP;
+            else
+                *bitsPerPixelP = 2 * *bitsPerSampleP;
+        } else {
+            if (pnm_type == PPM_TYPE)
+                *bitsPerPixelP = 3 * *bitsPerSampleP;
+            else
+                *bitsPerPixelP = *bitsPerSampleP;
+        }
+        if (verbose)
+            pm_message("Writing %d bits per component per pixel", 
+                       *bitsPerSampleP);
+    }
+}
+
+
 static void
 createPngPalette(pixel              palette_pnm[], 
                  unsigned int const paletteSize, 
@@ -1201,7 +1258,7 @@ convertpnm (FILE * const ifp,
     /* boolean: the user wants only the exact color he specified to be
        transparent; not just something close to it.
     */
-  xelval pnm_meaningful_bits;
+  unsigned int pnm_meaningful_bits;
   pixel backcolor;
       /* The background color, with maxval equal to that of the input
          image.
@@ -1240,10 +1297,17 @@ convertpnm (FILE * const ifp,
   int alpha_can_be_transparency_index;
 
   int colors;
-  int fulldepth;
-      /* The total number of bits per pixel, including all channels */
+  unsigned int depth;
+      /* The number of bits per pixel in the (uncompressed) png 
+         raster -- if the raster contains palette indices, this is the
+         number of bits in the index.
+      */
+  unsigned int fulldepth;
+      /* The total number of bits per pixel in the (uncompressed) png
+         raster, including all channels 
+      */
   int x, y;
-  int imagepos;  
+  pm_filepos imagepos;  
       /* file position in input image file of start of image (i.e. after
          the header)
       */
@@ -1257,7 +1321,6 @@ convertpnm (FILE * const ifp,
   static int pnm_type;
   static xelval maxmaxval;
   static gray **alpha_mask;
-  static int depth;
   
 
   /* these guys are initialized to quiet compiler warnings: */
@@ -1287,7 +1350,7 @@ convertpnm (FILE * const ifp,
   }
 
   pnm_readpnminit (ifp, &cols, &rows, &maxval, &format);
-  imagepos = pm_tell(ifp);
+  pm_tell2(ifp, &imagepos, sizeof(imagepos));
   pnm_type = PNM_FORMAT_TYPE (format);
 
   xelrow = pnm_allocrow(cols);
@@ -1365,7 +1428,7 @@ convertpnm (FILE * const ifp,
   if (pnm_type == PPM_TYPE && !force) {
     int isgray = TRUE;
 
-    pm_seek(ifp, imagepos);
+    pm_seek2(ifp, &imagepos, sizeof(imagepos));
     for (y = 0 ; y < rows && isgray ; y++) {
       pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
       for (x = 0 ; x < cols && isgray ; x++) {
@@ -1387,21 +1450,8 @@ convertpnm (FILE * const ifp,
                 "(use -downscale to override");
     }
 
-  /* Find out if we can mask off redundant (repeated) bits in sample values */
-
-  if (!alpha && pnm_type == PGM_TYPE && !force) 
-      meaningful_bits_pgm(ifp, imagepos, cols, rows, maxval, format,
-                          &pnm_meaningful_bits);
-  else if (pnm_type == PPM_TYPE && !force)
-      meaningful_bits_ppm(ifp, imagepos, cols, rows, maxval, format,
-                          &pnm_meaningful_bits);
-  else 
-      pnm_meaningful_bits = pm_maxvaltobits(maxval);
-
-  if (verbose && pnm_meaningful_bits != pm_maxvaltobits(maxval))
-      pm_message("Using only %d rightmost bits of input samples.  The "
-                 "rest are just copies of those.", pnm_meaningful_bits);
-
+  findRedundantBits(ifp, imagepos, cols, rows, maxval, format, alpha,
+                    force, &pnm_meaningful_bits);
   
   /* -----------------------------------------------------------------
      Determine whether to do a colormapped or truecolor PNG (variable
@@ -1475,93 +1525,20 @@ convertpnm (FILE * const ifp,
               
               if (background > -1) {
                   findOrAddBackgroundInPalette(
-                      backcolor, palette_pnm, &palette_size, trans_size, 
+                      backcolor, palette_pnm, &palette_size, 
                       &background_index);
               }
           }
       }
   }
-
-  if (colorMapped) {
-      /* Compute a lookup table for the palette index.  If there's no alpha
-         mask, this is just a standard Netpbm colorhash_table.  If there's
-         an alpha mask, it is the slower Pnmtopng-specific 
-         coloralphahash_table.
-      */
-      if (alpha) {
-          buildColorAlphaLookup(palette_pnm, palette_size, 
-                                trans_pnm, trans_size, alpha_maxval, &caht);
-          cht = NULL;
-      } else { 
-          buildColorLookup(palette_pnm, palette_size, &cht);
-          caht = NULL;
-      }
-      if (verbose)
-          pm_message("PNG palette has %u entries, %u of them non-opaque",
-                     palette_size, trans_size);
-  } else {
-      cht = NULL;
-      caht = NULL;
-  }
-
   
-  
-/*----------------------------------------------------------------------------
-   Compute the number of bits per raster sample and per raster pixel:
-   'depth' and 'fulldepth'.  Note that a raster element may be a
-   palette index, or a gray value or color with or without alpha mask.
------------------------------------------------------------------------------*/
-  if (colorMapped) {
-      if (palette_size <= 2)
-          depth = 1;
-      else if (palette_size <= 4)
-          depth = 2;
-      else if (palette_size <= 16)
-          depth = 4;
-      else
-      depth = 8;
-      fulldepth = depth;
-      if (verbose)
-          pm_message("Writing %d-bit color indexes", depth);
-  } else {
-      if (pnm_type == PPM_TYPE || alpha) {
-          /* PNG allows only depths of 8 and 16 for a truecolor image 
-             and for a grayscale image with an alpha channel.
-          */
-          if (pnm_meaningful_bits > 8)
-              depth = 16;
-          else 
-              depth = 8;
-      } else {
-          /* A grayscale, non-colormapped, no-alpha PNG may have any 
-             bit depth from 1 to 16
-          */
-          if (pnm_meaningful_bits > 8)
-              depth = 16;
-          else if (pnm_meaningful_bits > 4)
-              depth = 8;
-          else if (pnm_meaningful_bits > 2)
-              depth = 4;
-          else if (pnm_meaningful_bits > 1)
-              depth = 2;
-          else
-              depth = 1;
-      }
-    if (alpha) {
-      if (pnm_type == PPM_TYPE)
-        fulldepth = 4 * depth;
-      else
-        fulldepth = 2 * depth;
-    } else {
-      if (pnm_type == PPM_TYPE)
-        fulldepth = 3 * depth;
-      else
-        fulldepth = depth;
-    }
-    if (verbose)
-      pm_message("Writing %d bits per component per pixel", depth);
-  }
+  computeColorMapLookupTable(colorMapped, palette_pnm, palette_size,
+                             trans_pnm, trans_size, alpha, alpha_maxval,
+                             &cht, &caht);
 
+  computePixelWidth(colorMapped, palette_size, pnm_type, 
+                    pnm_meaningful_bits, alpha,
+                    &depth, &fulldepth);
   if (verbose)
     pm_message ("writing a%s %d-bit %s%s file%s",
                 fulldepth == 8 ? "n" : "", fulldepth,
@@ -1747,7 +1724,7 @@ convertpnm (FILE * const ifp,
 
   /* tEXT and zTXT chunks */
   if ((text) || (ztxt)) {
-    read_text (info_ptr, tfp);
+    pnmpng_read_text (info_ptr, tfp, ztxt, verbose);
   }
 
   /* tIME chunk */
@@ -1791,6 +1768,8 @@ convertpnm (FILE * const ifp,
   /* let libpng take care of, e.g., bit-depth conversions */
   png_set_packing (png_ptr);
 
+  /* Write the raster */
+ 
   /* max: 3 color channels, one alpha channel, 16-bit */
   if ((line = (png_byte *) malloc (cols*8)) == NULL)
   {
@@ -1800,14 +1779,16 @@ convertpnm (FILE * const ifp,
   }
 
   for (pass = 0 ; pass < png_set_interlace_handling (png_ptr) ; pass++) {
-      pm_seek(ifp, imagepos);
-      for (y = 0 ; y < rows ; y++) {
+      unsigned int row;
+      pm_seek2(ifp, &imagepos, sizeof(imagepos));
+      for (row = 0; row < rows; ++row) {
+          unsigned int col;
           pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
           pnm_promoteformatrow(xelrow, cols, maxval, format, maxval, PPM_TYPE);
           pp = line;
-          for (x = 0 ; x < cols ; x++) {
+          for (col = 0 ; col < cols ; ++col) {
               xel p_png;
-              xel const p = xelrow[x];
+              xel const p = xelrow[col];
               PPM_DEPTH(p_png, p, maxval, png_maxval);
               if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
                   info_ptr->color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
@@ -1818,7 +1799,7 @@ convertpnm (FILE * const ifp,
                   unsigned int paletteIndex;
                   if (alpha)
                       paletteIndex = lookupColorAlpha(caht, &p, 
-                                                      &alpha_mask[y][x]);
+                                                      &alpha_mask[row][col]);
                   else
                       paletteIndex = ppm_lookupcolor(cht, &p);
                   *pp++ = paletteIndex;
@@ -1840,7 +1821,7 @@ convertpnm (FILE * const ifp,
               }
               if (info_ptr->color_type & PNG_COLOR_MASK_ALPHA) {
                   int const png_alphaval = (int)
-                      alpha_mask[y][x] * (float) png_maxval / maxval +0.5;
+                      alpha_mask[row][col] * (float) png_maxval / maxval +0.5;
                   if (depth == 16)
                       *pp++ = png_alphaval >> 8;
                   *pp++ = png_alphaval & 0xff;
