@@ -4,6 +4,14 @@
   Use Ghostscript to convert a Postscript file into a PBM, PGM, or PNM
   file.
 
+  Implementation note: This program feeds the input file to Ghostcript
+  directly (with possible statements preceding it), and uses
+  Ghostscript's PNM output device drivers.  As an alternative,
+  Ghostscript also comes with the Postscript program pstoppm.ps which
+  we could run and it would read the input file and produce PNM
+  output.  It isn't clear to me what pstoppm.ps adds to what you get
+  from just feeding your input directly to Ghostscript as the main program.
+
 -----------------------------------------------------------------------------*/
 
 #define _BSD_SOURCE 1   /* Make sure strdup() is in string.h */
@@ -250,36 +258,24 @@ enum postscript_language {COMMON_POSTSCRIPT, ENCAPSULATED_POSTSCRIPT};
 
 static enum postscript_language
 language_declaration(const char input_filespec[], int const verbose) {
-/*----------------------------------------------------------------------------
-  Return the Postscript language in which the file declares it is written.
-  (Except that if the file is on Standard Input or doesn't validly declare
-  a languages, just say it is Common Postscript).
------------------------------------------------------------------------------*/
-    enum postscript_language language;
 
-    if (strcmp(input_filespec, "-") == 0)
-        /* Can't read stdin, because we need it to remain positioned for the 
-           Ghostscript interpreter to read it.
-        */
+    FILE *infile;
+    enum postscript_language language;
+    char line[80];
+
+    infile = pm_openr(input_filespec);
+
+    if (fgets(line, sizeof(line), infile) == NULL)
         language = COMMON_POSTSCRIPT;
     else {
-        FILE *infile;
-        char line[80];
+        const char eps_header[] = "%!PS-Adobe EPSF-3.0";
 
-        infile = pm_openr(input_filespec);
-
-        if (fgets(line, sizeof(line), infile) == NULL)
+        if (strncmp(line, eps_header, sizeof(eps_header)) == 0)
+            language = ENCAPSULATED_POSTSCRIPT;
+        else
             language = COMMON_POSTSCRIPT;
-        else {
-            const char eps_header[] = "%!PS-Adobe EPSF-3.0";
-            
-            if (strncmp(line, eps_header, sizeof(eps_header)) == 0)
-                language = ENCAPSULATED_POSTSCRIPT;
-            else
-                language = COMMON_POSTSCRIPT;
-        }
-        fclose(infile);
     }
+    fclose(infile);
     if (verbose)
         pm_message("language is %s",
                    language == ENCAPSULATED_POSTSCRIPT ?
@@ -410,7 +406,13 @@ static struct box
 add_borders(const struct box input_box, 
             const float xborder_scale, float yborder_scale,
             const int verbose) {
+/*----------------------------------------------------------------------------
+   Return a box which is 'input_box' plus some borders.
 
+   Add left and right borders that are the fraction 'xborder_scale' of the
+   width of the input box; likewise for top and bottom borders with 
+   'yborder_scale'.
+-----------------------------------------------------------------------------*/
     struct box retval;
 
     const int left_right_border_size = 
@@ -576,7 +578,6 @@ execGhostscript(int const inputPipeFd,
     asprintfN(&ropt, "-r%dx%d", xres, yres);
 
     if (verbose) {
-        pm_message("about to message");
         pm_message("execing '%s' with args '%s' (arg 0), "
                    "'%s', '%s', '%s', '%s', '%s', '%s', '%s'",
                    ghostscriptProg, arg0,
