@@ -40,8 +40,10 @@
    it's possible that it can be more flexible or at least give better
    diagnostic information if there's something wrong with the TIFF.
 
-   So we try (1) first, and if we can't get enough memory for the decoded
-   image or TIFFRGBAImageGet() fails, we fall back to (2).
+   In Netpbm, we stress function over performance, so by default we
+   try (1) first, and if we can't get enough memory for the decoded
+   image or TIFFRGBAImageGet() fails, we fall back to (2).  But we
+   give the user the -byrow option to order (2) only.
 */
 
 #define _BSD_SOURCE 1
@@ -84,6 +86,7 @@ struct cmdlineInfo {
     char *alphaFilename;
     int alphaStdout;
     unsigned int respectfillorder;   /* -respectfillorder option */
+    unsigned int byrow;
 } cmdline;
 
 
@@ -109,6 +112,8 @@ parseCommandLine(int argc, char ** argv,
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "respectfillorder", 
             OPT_FLAG,   NULL, &cmdlineP->respectfillorder, 0);
+    OPTENT3(0,   "byrow",   
+            OPT_FLAG,   NULL, &cmdlineP->byrow, 0);
     OPTENT3('h', "headerdump", 
             OPT_FLAG,   NULL, &cmdlineP->headerdump, 0);
     OPTENT3(0,   "alphaout",   
@@ -165,7 +170,7 @@ read_directory(TIFF * const tif,
     rc = TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &tiff_bps);
     *bps_p = (rc == 0) ? 1 : tiff_bps;
 
-    if (*bps_p < 1 || (*bps_p > 8 && *bps_p != 16))
+    if (*bps_p < 1 || (*bps_p > 8 && *bps_p != 16 && *bps_p != 32))
         pm_error("This program can process Tiff images with only "
                  "1-8 or 16 bits per sample.  The input Tiff image "
                  "has %d bits per sample.", *bps_p);
@@ -237,11 +242,15 @@ read_directory(TIFF * const tif,
 
 
 static void
-readscanline(TIFF * const tif, unsigned short * const samplebuf,
-             unsigned char scanbuf[], int const row, int const plane,
-             unsigned int const cols, unsigned short const bps,
+readscanline(TIFF *         const tif, 
+             unsigned char  scanbuf[], 
+             int            const row, 
+             int            const plane,
+             unsigned int   const cols, 
+             unsigned short const bps,
              unsigned short const spp,
-             unsigned short const fillorder) {
+             unsigned short const fillorder,
+             unsigned int * const samplebuf) {
 /*----------------------------------------------------------------------------
    Read one scanline out of the Tiff input and store it into samplebuf[].
    Unlike the scanline returned by the Tiff library function, samplebuf[]
@@ -314,8 +323,18 @@ readscanline(TIFF * const tif, unsigned short * const samplebuf,
            bytes of each sample appear in a TIFF file, which is
            contrary to the TIFF spec.  
         */
-        memcpy((char *)&samplebuf[0], &scanbuf[0], cols*spp*2);
-    } else
+        uint16 * const scanbuf16 = (uint16 *) scanbuf;
+        unsigned int sample;
+
+        for (sample = 0; sample < cols*spp; ++sample)
+            samplebuf[sample] = scanbuf16[sample];
+    } else if (bps == 32) {
+        uint32 * const scanbuf32 = (uint32 *) scanbuf;
+        unsigned int sample;
+        
+        for (sample = 0; sample < cols*spp; ++sample)
+            samplebuf[sample] = scanbuf32[sample];
+    } else 
         pm_error("Internal error: invalid bits per sample passed to "
                  "readscanline()");
 }
@@ -323,7 +342,7 @@ readscanline(TIFF * const tif, unsigned short * const samplebuf,
 
 
 static void
-pick_cmyk_pixel(const unsigned short samplebuf[], const int sample_cursor,
+pick_cmyk_pixel(const unsigned int samplebuf[], const int sample_cursor,
                 xelval * const r_p, xelval * const b_p, xelval * const g_p) {
 
     unsigned char c, y, m, k;
@@ -380,7 +399,7 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
 
     if ( bps == 1 && spp == 1 ) {
         if ( cmdline.headerdump )
-            pm_message("monochrome" );
+            pm_message("bilevel" );
         grayscale = TRUE;
         *maxvalP = 1;
     } else {
@@ -389,17 +408,15 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
            */
         switch ( photomet ) {
         case PHOTOMETRIC_MINISBLACK:
-            grayscale = TRUE;
-            *maxvalP = (1 << bps) - 1;
-            if (headerdump)
-                pm_message( "%d graylevels (min=black)", *maxvalP + 1 );
-            break;
-            
         case PHOTOMETRIC_MINISWHITE:
             grayscale = TRUE;
-            *maxvalP = (1 << bps) - 1;
+            *maxvalP = pm_bitstomaxval(MIN(bps,16));
             if (headerdump)
-                pm_message( "%d graylevels (min=white)", *maxvalP + 1 );
+                pm_message("grayscale image, (min=%s) output maxval %u ", 
+                           photomet == PHOTOMETRIC_MINISBLACK ? 
+                           "black" : "white",
+                           *maxvalP
+                           );
             break;
             
         case PHOTOMETRIC_PALETTE: {
@@ -500,14 +517,17 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
 
 
 static void
-convertRow(unsigned short samplebuf[], xel xelrow[], gray alpharow[], 
-           int const cols, xelval const maxval, 
-           unsigned short const photomet, unsigned short const spp,
-           xel colormap[]) {
+convertRow(unsigned int   const samplebuf[], 
+           xel                  xelrow[], 
+           gray                 alpharow[], 
+           int            const cols, 
+           xelval         const maxval, 
+           unsigned short const photomet, 
+           unsigned short const spp,
+           xel            const colormap[]) {
 /*----------------------------------------------------------------------------
-   Assuming samplebuf[] is a scan line as returned by the Tiff library,
-   convert it to a libpnm row in xelrow[] and alpharow[].
-
+   Assuming samplebuf[] is an array of raster values as returned by the Tiff
+   library, convert it to a libnetpbm row in xelrow[] and alpharow[].
 -----------------------------------------------------------------------------*/
     switch (photomet) {
     case PHOTOMETRIC_MINISBLACK: {
@@ -574,6 +594,86 @@ convertRow(unsigned short samplebuf[], xel xelrow[], gray alpharow[],
 
 
 
+static void
+scale32to16(unsigned int       samplebuf[],
+            unsigned int const cols,
+            unsigned int const spp) {
+/*----------------------------------------------------------------------------
+  Convert every sample in samplebuf[] to something that can be expressed
+  in 16 bits, assuming it takes 32 bits now.
+-----------------------------------------------------------------------------*/
+    unsigned int i;
+    for (i = 0; i < cols * spp; ++i)
+        samplebuf[i] >>= 16; 
+}
+
+
+
+static void
+convertMultiPlaneRow(TIFF *         const tif,
+                     xel                   xelrow[],
+                     gray                  alpharow[],
+                     int             const cols,
+                     xelval          const maxval,
+                     int             const row,
+                     unsigned short  const photomet,
+                     unsigned short  const bps,
+                     unsigned short  const spp,
+                     unsigned short  const fillorder,
+                     unsigned char * const scanbuf,
+                     unsigned int *  const samplebuf) {
+
+    /* The input is in separate planes, so we need to read one
+       scanline for the reds, another for the greens, then another
+       for the blues.
+    */
+
+    int col;
+
+    if (photomet != PHOTOMETRIC_RGB)
+        pm_error("This is a multiple-plane file, but is not an RGB "
+                 "file.  This program does not know how to handle that.");
+    else {
+        /* First, clear the buffer so we can add red, green,
+           and blue one at a time.  
+                */
+        for (col = 0; col < cols; ++col) 
+            PPM_ASSIGN(xelrow[col], 0, 0, 0);
+
+        /* Read the reds */
+        readscanline(tif, scanbuf, row, 0, cols, bps, spp, fillorder, 
+                     samplebuf);
+        if (bps == 32)
+            scale32to16(samplebuf, cols, spp);
+        for (col = 0; col < cols; ++col) 
+            PPM_PUTR(xelrow[col], samplebuf[col]);
+                
+        /* Next the greens */
+        readscanline(tif, scanbuf, row, 1, cols, bps, spp, fillorder,
+                     samplebuf);
+        if (bps == 32)
+            scale32to16(samplebuf, cols, spp);
+        for (col = 0; col < cols; ++col) 
+            PPM_PUTG( xelrow[col], samplebuf[col] );
+            
+        /* And finally the blues */
+        readscanline(tif, scanbuf, row, 2, cols, bps, spp, fillorder,
+                     samplebuf);
+        if (bps == 32)
+            scale32to16(samplebuf, cols, spp);
+        for (col = 0; col < cols; ++col) 
+            PPM_PUTB(xelrow[col], samplebuf[col]);
+
+        /* Could there be an alpha plane?  (We assume no.  But if so,
+           here is where to read it) 
+        */
+        for (col = 0; col < cols; ++col) 
+            alpharow[col] = 0;
+    }
+}
+
+
+
 static void
 convertRasterByRows(FILE *         const imageoutFile, 
                     FILE *         const alphaFile,
@@ -600,7 +700,7 @@ convertRasterByRows(FILE *         const imageoutFile,
         /* Buffer for a raster line in the format returned by TIFF library's
            TIFFReadScanline
         */
-    unsigned short * samplebuf;
+    unsigned int * samplebuf;
         /* Same info as 'scanbuf' above, but with each raster column (sample)
            represented as single array element, so it's easy to work with.
         */
@@ -609,7 +709,7 @@ convertRasterByRows(FILE *         const imageoutFile,
     gray* alpharow;
         /* The pgm-format row representing the alpha values for the image 
            row we are presently converting.
-           */
+        */
 
     int row;
 
@@ -617,7 +717,7 @@ convertRasterByRows(FILE *         const imageoutFile,
     if (scanbuf == NULL)
         pm_error("can't allocate memory for scanline buffer");
 
-    samplebuf = (unsigned short *) malloc(cols * sizeof(unsigned short) * spp);
+    samplebuf = (unsigned int *) malloc(cols * sizeof(samplebuf[0]) * spp);
     if (samplebuf == NULL)
         pm_error ("can't allocate memory for row buffer");
 
@@ -628,58 +728,17 @@ convertRasterByRows(FILE *         const imageoutFile,
         /* Read one row of samples into samplebuf[] */
 
         if (planarconfig == PLANARCONFIG_CONTIG) {
-            readscanline(tif, samplebuf, scanbuf, row, 0, cols, bps, spp, 
-                         fillorder);
-
-
+            readscanline(tif, scanbuf, row, 0, cols, bps, spp, fillorder, 
+                         samplebuf);
+            if (bps == 32)
+                scale32to16(samplebuf, cols, spp);
             convertRow(samplebuf, xelrow, alpharow, cols, maxval, 
                        photomet, spp, colormap);
-        } else {
-            /* The input is in separate planes, so we need to read one
-               scanline for the reds, another for the greens, then another
-               for the blues.
-            */
-
-            int col;
-
-            if (photomet != PHOTOMETRIC_RGB)
-                pm_error("This is a multiple-plane file, but is not an RGB "
-                         "file.  This program does not "
-                         "know how to handle that.");
-            else {
-                /* First, clear the buffer so we can add red, green,
-                   and blue one at a time.  
-                */
-                for (col = 0; col < cols; ++col) 
-                    PPM_ASSIGN(xelrow[col], 0, 0, 0);
-
-                readscanline(tif, samplebuf, scanbuf, row, 0, cols, bps, spp, 
-                             fillorder);
-                
-                /* Read the reds */
-                for (col = 0; col < cols; ++col) 
-                    PPM_PUTR(xelrow[col], samplebuf[col]);
-                
-                /* Next the greens */
-                readscanline(tif, samplebuf, scanbuf, row, 1,
-                             cols, bps, spp, fillorder);
-                for (col = 0; col < cols; ++col) 
-                    PPM_PUTG( xelrow[col], samplebuf[col] );
+        } else 
+            convertMultiPlaneRow(tif, xelrow, alpharow, cols, maxval, row,
+                                 photomet, bps, spp, fillorder,
+                                 scanbuf, samplebuf);
             
-                /* And finally the blues */
-                readscanline(tif, samplebuf, scanbuf, row, 2,
-                             cols, bps, spp, fillorder);
-                for (col = 0; col < cols; ++col) 
-                    PPM_PUTB(xelrow[col], samplebuf[col]);
-
-                /* Could there be an alpha plane?  (We assume no.  But if so,
-                   here is where to read it) 
-                */
-                for (col = 0; col < cols; ++col) 
-                    alpharow[col] = 0;
-            }
-        }
-
         if (imageoutFile != NULL) 
             pnm_writepnmrow( imageoutFile, 
                              xelrow, cols, (xelval) maxval, format, 0 );
@@ -710,8 +769,6 @@ convertTiffRaster(uint32 *        const raster,
    the files *imageoutFile and *alphaFile in format 'format' with maxval
    'maxval'.  The raster is 'cols' wide by 'rows' high.
 -----------------------------------------------------------------------------*/
-
-    uint32* lins;
     xel* xelrow;
         /* The ppm-format row of the image row we are
            presently converting 
@@ -725,14 +782,13 @@ convertTiffRaster(uint32 *        const raster,
     xelrow = pnm_allocrow(cols);
     alpharow = pgm_allocrow(cols);
 
-    lins = raster + (rows - 1) * cols ;
-    
     for (row = 0; row < rows; ++row) {
-        uint32* rp ;
+        uint32* rp;  
+            /* Address of pixel in 'raster' we are presently converting */
         int col;
 
-        rp = lins ;
-        lins -= cols;
+        /* Start at beginning of row: */
+        rp = raster + (rows - row - 1) * cols;
     
         for (col = 0; col < cols; ++col) {
             uint32 const tiffPixel = *rp++;
@@ -745,9 +801,9 @@ convertTiffRaster(uint32 *        const raster,
         }
         
         if (imageoutFile != NULL) 
-            pnm_writepnmrow( imageoutFile, xelrow, cols, maxval, format, 0 );
+            pnm_writepnmrow(imageoutFile, xelrow, cols, maxval, format, 0);
         if (alphaFile != NULL) 
-            pgm_writepgmrow( alphaFile, alpharow, cols, maxval, 0);
+            pgm_writepgmrow(alphaFile, alpharow, cols, maxval, 0);
     }
     
     pgm_freerow(alpharow);
@@ -756,7 +812,8 @@ convertTiffRaster(uint32 *        const raster,
 
 
 
-enum convertDisp {CONV_DONE, CONV_OOM, CONV_UNABLE, CONV_FAILED};
+enum convertDisp {CONV_DONE, CONV_OOM, CONV_UNABLE, CONV_FAILED, 
+                  CONV_NOTATTEMPTED};
 
 static void
 convertRasterInMemory(FILE *         const imageoutFile, 
@@ -836,13 +893,14 @@ convertRasterInMemory(FILE *         const imageoutFile,
 
 
 
-int
-main(int argc, char * argv[]) {
+static void
+convertIt(TIFF*              const tif,
+          FILE *             const alphaFile, 
+          FILE *             const imageoutFile,
+          struct cmdlineInfo const cmdline) {
 
     unsigned int cols, rows;
     int format;
-    TIFF* tif;
-    FILE *alphaFile, *imageoutFile;
     xelval maxval;
     unsigned short bps, spp;
 
@@ -850,6 +908,52 @@ main(int argc, char * argv[]) {
     unsigned short photomet, planarconfig, fillorderTag;
     unsigned short fillorder;
 
+    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
+                   &cols, &rows, 
+                   cmdline.headerdump);
+
+
+    computeFillorder(fillorderTag, &fillorder, cmdline.respectfillorder);
+
+    analyze_image_type(tif, bps, spp, photomet, 
+                       &maxval, &format, colormap, cmdline.headerdump);
+
+    if (imageoutFile != NULL) 
+        pnm_writepnminit( imageoutFile, 
+                          cols, rows, (xelval) maxval, format, 0 );
+    if (alphaFile != NULL) 
+        pgm_writepgminit( alphaFile, cols, rows, (gray) maxval, 0 );
+
+    {
+        enum convertDisp status;
+        if (cmdline.byrow)
+            status = CONV_NOTATTEMPTED;
+        else {
+            convertRasterInMemory(
+                imageoutFile, alphaFile, cols, rows, maxval, format, 
+                tif, photomet, planarconfig, bps, spp, fillorder,
+                colormap, &status);
+        }
+        if (status != CONV_DONE) {
+            if (status != CONV_NOTATTEMPTED)
+                pm_message("In-memory conversion failed; "
+                           "using more primitive row-by-row conversion.");
+            
+            convertRasterByRows(
+                imageoutFile, alphaFile, cols, rows, maxval, format, 
+                tif, photomet, planarconfig, bps, spp, fillorder, colormap);
+        }            
+    }
+}
+
+
+
+int
+main(int argc, char * argv[]) {
+
+    TIFF* tif;
+    FILE *alphaFile, *imageoutFile;
+
     pnm_init( &argc, argv );
 
     parseCommandLine(argc, argv, &cmdline);
@@ -877,36 +981,7 @@ main(int argc, char * argv[]) {
     else
         imageoutFile = stdout;
 
-    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
-                   &cols, &rows, 
-                   cmdline.headerdump);
-
-
-    computeFillorder(fillorderTag, &fillorder, cmdline.respectfillorder);
-
-    analyze_image_type(tif, bps, spp, photomet, 
-                       &maxval, &format, colormap, cmdline.headerdump);
-
-    if (imageoutFile != NULL) 
-        pnm_writepnminit( imageoutFile, 
-                          cols, rows, (xelval) maxval, format, 0 );
-    if (alphaFile != NULL) 
-        pgm_writepgminit( alphaFile, cols, rows, (gray) maxval, 0 );
-
-    {
-        enum convertDisp status;
-        convertRasterInMemory(
-            imageoutFile, alphaFile, cols, rows, maxval, format, 
-            tif, photomet, planarconfig, bps, spp, fillorder,
-            colormap, &status);
-        if (status != CONV_DONE) {
-            pm_message("In-memory conversion failed; "
-                       "using more primitive row-by-row conversion.");
-            convertRasterByRows(
-                imageoutFile, alphaFile, cols, rows, maxval, format, 
-                tif, photomet, planarconfig, bps, spp, fillorder, colormap);
-        }            
-    }
+    convertIt(tif, alphaFile, imageoutFile, cmdline);
 
     if (imageoutFile != NULL) 
         pm_close( imageoutFile );
