@@ -8,7 +8,9 @@
 #include <math.h>
 #include <ctype.h>
 #include <string.h>
+
 #include "pbm.h"
+#include "nstring.h"
 
 #define MAXPKCHAR 256
 #define MAXOPTLINE 200
@@ -717,7 +719,7 @@ optionfile(const char * const name) {
         char *here = buffer;
       
         if (fgets(buffer, MAXOPTLINE, fp) == NULL) break ;
-        while (isspace(*here)) here++ ;
+        while (ISSPACE(*here)) here++ ;
         if (*here && *here == '=') {
             if (sscanf(here+1, "%d", &car) != 1)
                 pm_error("bad option file line %s", buffer) ;
