@@ -49,13 +49,12 @@
 #include <string.h>
 #include "bmp.h"
 #include "ppm.h"
-#include "ppmcmap.h"
 #include "shhopt.h"
 #include "bitio.h"
 
 #define MAXCOLORS 256
 
-enum colortype {TRUECOLOR, PALLETTE};
+enum colortype {TRUECOLOR, PALETTE};
 /*
  * Utilities
  */
@@ -82,7 +81,7 @@ parse_command_line(int argc, char ** argv,
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optStruct));
+    optEntry *option_def = malloc(100*sizeof(optEntry));
         /* Instructions to OptParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -134,7 +133,7 @@ parse_command_line(int argc, char ** argv,
 
 
 static void
-PutByte(FILE *fp, unsigned char v)
+PutByte(FILE *fp, char v)
 {
         if (putc(v, fp) == EOF)
         {
@@ -316,9 +315,9 @@ BMPwritecolormap(FILE *fp, const int class, const int bpp,
  * returns the number of bytes written, or -1 on error.
  */
 static int
-BMPwriterow_pallette(FILE *fp, const pixel * const row, 
-                     const unsigned long cx, 
-                     const unsigned short bpp, const colorhash_table cht)
+BMPwriterow_palette(FILE *fp, const pixel * const row, 
+                    const unsigned long cx, 
+                    const unsigned short bpp, const colorhash_table cht)
 {
         BITSTREAM       b;
         unsigned        nbyte = 0;
@@ -364,8 +363,7 @@ BMPwriterow_pallette(FILE *fp, const pixel * const row,
 static int
 BMPwriterow_truecolor(FILE *        const fp, 
                       const pixel * const row, 
-                      unsigned long const cols,
-                      pixval        const maxval) {
+                      unsigned long const cols) {
 /*----------------------------------------------------------------------------
   Write a row of a truecolor BMP image to the file 'fp'.  The row is 
   'row', which is 'cols' columns long.
@@ -378,7 +376,7 @@ BMPwriterow_truecolor(FILE *        const fp,
        general case (which is only hypothetical -- this program doesn't
        write any truecolor images except 24 bit and apparently no one
        else does either), you would move this function into 
-       BMPwriterow_pallette, which writes arbitrary bit strings.  But
+       BMPwriterow_palette, which writes arbitrary bit strings.  But
        that would be a lot slower and less robust.
     */
 
@@ -387,9 +385,9 @@ BMPwriterow_truecolor(FILE *        const fp,
         
     nbyte = 0;  /* initial value */
     for (col = 0; col < cols; col++) {
-        PutByte(fp, PPM_GETB(row[col]) * 255 / maxval);
-        PutByte(fp, PPM_GETG(row[col]) * 255 / maxval);
-        PutByte(fp, PPM_GETR(row[col]) * 255 / maxval);
+        PutByte(fp, PPM_GETB(row[col]));
+        PutByte(fp, PPM_GETG(row[col]));
+        PutByte(fp, PPM_GETR(row[col]));
         nbyte += 3;
     }
 
@@ -411,9 +409,7 @@ static int
 BMPwritebits(FILE *fp, const unsigned long cx, const unsigned long cy, 
              const enum colortype colortype,
              const unsigned short cBitCount, 
-             const pixel ** const pixels, 
-             const pixval maxval,
-             const colorhash_table cht)
+             const pixel ** const pixels, const colorhash_table cht)
 {
         int             nbyte = 0;
         long            y;
@@ -431,11 +427,11 @@ BMPwritebits(FILE *fp, const unsigned long cx, const unsigned long cy,
         for (y = cy - 1; y >= 0; y--)
         {
                 int rc;
-                if (colortype == PALLETTE)
-                    rc = BMPwriterow_pallette(fp, pixels[y], cx, 
-                                              cBitCount, cht);
+                if (colortype == PALETTE)
+                    rc = BMPwriterow_palette(fp, pixels[y], cx, 
+                                             cBitCount, cht);
                 else 
-                    rc = BMPwriterow_truecolor(fp, pixels[y], cx, maxval);
+                    rc = BMPwriterow_truecolor(fp, pixels[y], cx);
 
                 if(rc == -1)
                 {
@@ -463,22 +459,22 @@ static void
 BMPEncode(FILE *fp, const int class, const enum colortype colortype,
           const int bpp,
           const int x, const int y, 
-          const pixel ** const pixels, const pixval maxval, const int colors, 
+          const pixel ** const pixels, const int colors, 
           const colorhash_table cht, 
           const unsigned char R[], const unsigned char G[], 
           const unsigned char B[])
 {
         unsigned long   nbyte = 0;
 
-        if (colortype == PALLETTE)
-            pm_message("Writing %d bits per pixel with a color pallette", bpp);
+        if (colortype == PALETTE)
+            pm_message("Writing %d bits per pixel with a color palette", bpp);
         else
-            pm_message("Writing %d bits per pixel truecolor (no pallette)", 
+            pm_message("Writing %d bits per pixel truecolor (no palette)", 
                        bpp);
 
         nbyte += BMPwritefileheader(fp, class, bpp, x, y);
         nbyte += BMPwriteinfoheader(fp, class, bpp, x, y);
-        if (colortype == PALLETTE)
+        if (colortype == PALETTE)
             nbyte += BMPwritecolormap(fp, class, bpp, colors, R, G, B);
 
         if(nbyte !=     ( BMPlenfileheader(class)
@@ -488,7 +484,7 @@ BMPEncode(FILE *fp, const int class, const enum colortype colortype,
                 pm_error(er_internal, "BMPEncode 1");
         }
 
-        nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, maxval, cht);
+        nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, cht);
         if(nbyte != BMPlenfile(class, bpp, -1, x, y))
         {
                 pm_error(er_internal, "BMPEncode 2");
@@ -514,13 +510,13 @@ analyze_colors(const pixel ** const pixels,
   the minimum number of bits per pixel it will take to represent all
   the colors in BMP format.
 
-  If there are few enough colors to represent with a pallette, also
-  return the number of colors as *colors_p and a suitable pallette
+  If there are few enough colors to represent with a palette, also
+  return the number of colors as *colors_p and a suitable palette
   (colormap) and a hash table in which to look up indexes into that
-  pallette as Red[], Green[], Blue[], and *cht_p.  Use only the first
+  palette as Red[], Green[], Blue[], and *cht_p.  Use only the first
   *colors_p entries of the Red[], etc. array.
 
-  If there are too many colors for a pallette, return *cht_p == NULL.
+  If there are too many colors for a palette, return *cht_p == NULL.
 
   Issue informational messages.
 
@@ -581,7 +577,7 @@ choose_colortype_bpp(const struct cmdline_info cmdline,
     if (*bits_per_pixel_p > 8) 
         *colortype_p = TRUECOLOR;
     else
-        *colortype_p = PALLETTE;
+        *colortype_p = PALETTE;
 }
 
 
@@ -589,7 +585,7 @@ choose_colortype_bpp(const struct cmdline_info cmdline,
 int
 main(int argc, char **argv)
 {
-        FILE *ifp = stdin;
+        FILE *ifp;
 
         int rows;
         int cols;
@@ -624,7 +620,7 @@ main(int argc, char **argv)
                              &bits_per_pixel);
 
         BMPEncode(stdout, cmdline.class, colortype, bits_per_pixel,
-                  cols, rows, (const pixel**)pixels, maxval, colors, cht,
+                  cols, rows, (const pixel**)pixels, colors, cht,
                   Red, Green, Blue);
 
         if (cht) 
