@@ -4,7 +4,7 @@
 **
 ** Copyright (C) 1992,93 by S.Petra Zeidler
 ** Minor modifications by Ingo Wilken:
-**  - mymalloc() and check_and_rotate() functions for often used
+x**  - mymalloc() and check_and_rotate() functions for often used
 **    code fragments.  Reduces code size by a few KB.
 **  - use pm_error() instead of fprintf(stderr)
 **  - localized allocation of colorhastable
@@ -21,12 +21,52 @@
 */
 
 #include <string.h>
+
 #include "ppm.h"
-#include "ppmcmap.h"
+#include "nstring.h"
 
 #include "mitsu.h"
 
 
+#include <stdio.h>
+
+#define HASHSIZE 2048
+#define myhash(x) ((PPM_GETR(x)*3 + PPM_GETG(x)*5 + PPM_GETB(x)*7) % HASHSIZE)
+
+typedef struct hashinfo {
+        pixel     color;
+        long      flag;
+        struct hashinfo *next;
+} hashinfo;
+
+#ifdef __STDC__
+static void lineputinit(int cols, int rows, int sharpness, int enlarge, int
+                        copy, struct mediasize medias);
+static void frametransferinit(int cols, int rows, int sharpness, int enlarge,
+                              int copy, struct mediasize medias);
+static void lookuptableinit(int sharpness, int enlarge, int copy,
+                            struct mediasize medias);
+static void lookuptabledata(int cols, int rows, int enlarge,
+                            struct mediasize medias);
+static void check_and_rotate(int cols, int rows, int enlarge,
+                             struct mediasize medias);
+static void * mymalloc(long bytes);
+#define CONST const
+#else /*__STDC__*/
+static int lineputinit();
+static int lookuptableinit();
+static int lookuptabledata();
+static int frametransferinit();
+static int check_and_rotate();
+static char *mymalloc();
+#define CONST
+#endif
+
+#define cmd(arg)           fputc((arg), stdout)
+#define datum(arg)         fputc((char)(arg), stdout)
+#define data(arg,num)      fwrite((arg), sizeof(char), (num), stdout)
+
+
 #ifdef __STDC__
 int main(int argc, char *argv[] )
 #else
@@ -77,7 +117,7 @@ int main( argc, argv )
         ++argn;
         if (argn == argc || sscanf(argv[argn], "%15s", media) < 1)
             pm_usage(usage);
-        else if (mytoupper(media[0]) != 'A')
+        else if (TOUPPER(media[0]) != 'A')
             pm_usage(usage);
     }
     else if (pm_keymatch(argv[argn], "-copy", 2)) {
@@ -106,13 +146,13 @@ int main( argc, argv )
     if (argn != argc)
         pm_usage(usage);
 
-    if (mytoupper(media[0]) == 'A')
-        switch (mytoupper(media[1])) {
+    if (TOUPPER(media[0]) == 'A')
+        switch (TOUPPER(media[1])) {
         case 'S':
             medias = MSize_AS;
             break;
         case '4':
-            if(mytoupper(media[2]) == 'S')
+            if(TOUPPER(media[2]) == 'S')
                 medias = MSize_A4S;
             else {
                 medias = MSize_A4;
