@@ -1,6 +1,6 @@
 /* ppmtowinicon.c - read portable pixmap file(s) and write a MS Windows .ico
 **
-** Copyright (C) 2000 by Lee Benfield - lee@recoil.org
+** Copyright (C) 2000 by Lee Benfield - lee@benf.org
 **
 ** Permission to use, copy, modify, and distribute this software and its
 ** documentation for any purpose and without fee is hereby granted, provided
@@ -14,12 +14,12 @@
  * Changelog
  *
  * 2002/01/25 - Use pbm instead of ppm for alpha maps and fix alpha
- *		sense. (black = transparent, white = opaque).
- *		Option renamed to -andpbms; -andppms retained
- *		for compatibility.
- *		ppmtowinicon should now be the true reverse of winicontoppm.
- *		Filled some "ignored" fields in ICO output for maximum
- *		compatibility.  MVB.
+ *      sense. (black = transparent, white = opaque).
+ *      Option renamed to -andpbms; -andppms retained
+ *      for compatibility.
+ *      ppmtowinicon should now be the true reverse of winicontoppm.
+ *      Filled some "ignored" fields in ICO output for maximum
+ *      compatibility.  MVB.
  *
  * 2000/05/24 - using colorhash_table instead of a straight array search
  *              Though it's a little slower for files due to the low # of
@@ -31,7 +31,6 @@
 
 #include "winico.h"
 #include "ppm.h"
-#include "ppmcmap.h"
 
 #define MAJVERSION 0
 #define MINVERSION 3
@@ -54,7 +53,7 @@ PutByte(int const v) {
    
    if (putc(v, ofp) == EOF)
      {
-	pm_error(er_write, outfname);
+    pm_error(er_write, outfname);
      }
 }
    
@@ -63,7 +62,7 @@ PutShort(short const v) {
    
    if (pm_writelittleshort(ofp, v) == -1)
      {
-	pm_error(er_write, outfname);
+    pm_error(er_write, outfname);
      }
    
 }
@@ -73,7 +72,7 @@ PutLong(long const v) {
    
    if (pm_writelittlelong(ofp, v) == -1)
      {
-	pm_error(er_write, outfname);
+    pm_error(er_write, outfname);
      }
    
 }
@@ -143,19 +142,19 @@ createAndBitmap (gray ** const ba, int const cols, int const rows,
        * Check there's a bit array, otherwise we're just faking this...
        */
       if (ba) {
-	 for (x=0;x<cols;x++) {
+     for (x=0;x<cols;x++) {
             /* Black (bit clear) is transparent in PGM alpha maps,
              * in ICO bit *set* is transparent.
              */
             if (ba[y][x] <= maxval/2) row[byteOn] |= bitOn;
 
-	    if (bitOn == 1) {
-	       byteOn++;
-	       bitOn = 128;
-	    } else {
-	       bitOn >>= 1;
-	    }
-	 }
+        if (bitOn == 1) {
+           byteOn++;
+           bitOn = 128;
+        } else {
+           bitOn >>= 1;
+        }
+     }
       }
    }
    return icBitmap;
@@ -204,26 +203,26 @@ create1Bitmap (pixel ** const pa, int const cols, int const rows,
        * Check there's a pixel array, otherwise we're just faking this...
        */
       if (pa) {
-	 for (x=0;x<cols;x++) {
-	    /*
-	     * So we've got a colorhash_table with two colors in it.
-	     * Which is black?!
-	     * 
-	     * Unless the hashing function changes, 0's black.
-	     */
-	    value = ppm_lookupcolor(cht, &pa[y][x]);
-	    if (!value) {
-	       /* leave black. */
-	    } else {
-	       row[byteOn] |= bitOn;
-	    }
-	    if (bitOn == 1) {
-	       byteOn++;
-	       bitOn = 128;
-	    } else {
-	       bitOn >>= 1;
-	    }
-	 }
+     for (x=0;x<cols;x++) {
+        /*
+         * So we've got a colorhash_table with two colors in it.
+         * Which is black?!
+         * 
+         * Unless the hashing function changes, 0's black.
+         */
+        value = ppm_lookupcolor(cht, &pa[y][x]);
+        if (!value) {
+           /* leave black. */
+        } else {
+           row[byteOn] |= bitOn;
+        }
+        if (bitOn == 1) {
+           byteOn++;
+           bitOn = 128;
+        } else {
+           bitOn >>= 1;
+        }
+     }
       }
    }
    return icBitmap;
@@ -262,22 +261,22 @@ create4Bitmap (pixel ** const pa, int const cols, int const rows,
        * Check there's a pixel array, otherwise we're just faking this...
        */
       if (pa) {
-	 for (x=0;x<cols;x++) {
-	    value = ppm_lookupcolor(cht, &pa[y][x]);
-	    /*
-	     * Shift it, if we're putting it in the high nibble.
-	     */
-	    if (nibble) {
-	       value <<= 4;
-	    }
-	    row[byteOn] |= value;
-	    if (nibble) {
-	       nibble = 0;
-	    } else {
-	       nibble = 1;
-	       byteOn++;
-	    }
-	 }
+     for (x=0;x<cols;x++) {
+        value = ppm_lookupcolor(cht, &pa[y][x]);
+        /*
+         * Shift it, if we're putting it in the high nibble.
+         */
+        if (nibble) {
+           value <<= 4;
+        }
+        row[byteOn] |= value;
+        if (nibble) {
+           nibble = 0;
+        } else {
+           nibble = 1;
+           byteOn++;
+        }
+     }
       }
    }
    return icBitmap;
@@ -313,9 +312,9 @@ create8Bitmap (pixel ** const pa, int const cols, int const rows,
        * Check there's a pixel array, otherwise we're just faking this...
        */
       if (pa) {
-	 for (x=0;x<cols;x++) {
-	    row[x] = ppm_lookupcolor(cht, &pa[y][x]);
-	 }
+     for (x=0;x<cols;x++) {
+        row[x] = ppm_lookupcolor(cht, &pa[y][x]);
+     }
       }
    }
    return icBitmap;
@@ -440,27 +439,27 @@ addEntryToIcon (MS_Ico const MSIconData, const char * const xorPPM,
                                  &colors);
    if (xorChv == (colorhist_vector) 0)
      pm_error("%s has too many colors - try doing a 'pnmquant %d'"
-	      , xorPPM, MAXCOLORS);
+          , xorPPM, MAXCOLORS);
    if (verbose) pm_message("%d colors found", colors);
 
    if (verbose && (xorMaxval > 255))
        pm_message("maxval is not 255 - automatically rescaling colors");
    for (i = 0; i < colors; ++i)
      {
-	if (xorMaxval == 255)
-	  {
-	     addColorToPalette(palette,i,
-			       PPM_GETR(xorChv[i].color),
-			       PPM_GETG(xorChv[i].color),
-			       PPM_GETB(xorChv[i].color));
-	  }
-	else
-	  {
-	     addColorToPalette(palette,i,
-			       PPM_GETR(xorChv[i].color) * 255 / xorMaxval,
-			       PPM_GETG(xorChv[i].color) * 255 / xorMaxval,
-			       PPM_GETB(xorChv[i].color) * 255 / xorMaxval);
-	  }
+    if (xorMaxval == 255)
+      {
+         addColorToPalette(palette,i,
+                   PPM_GETR(xorChv[i].color),
+                   PPM_GETG(xorChv[i].color),
+                   PPM_GETB(xorChv[i].color));
+      }
+    else
+      {
+         addColorToPalette(palette,i,
+                   PPM_GETR(xorChv[i].color) * 255 / xorMaxval,
+                   PPM_GETG(xorChv[i].color) * 255 / xorMaxval,
+                   PPM_GETB(xorChv[i].color) * 255 / xorMaxval);
+      }
      }
    
    /* And make a hash table for fast lookup. */
@@ -627,7 +626,7 @@ writeMS_Ico (MS_Ico const MSIconData, char * const outFname) {
    for (x=0;x<MSIconData->count;x++) {
       writeIC_InfoHeader(MSIconData->entries[x]->ih);
       for (y=0;y<(MSIconData->entries[x]->color_count);y++) {
-	 writeIC_Color(MSIconData->entries[x]->colors[y]);
+     writeIC_Color(MSIconData->entries[x]->colors[y]);
       }
       if (verbose) pm_message("writing xor bitmap\n");
       writeBitmap(MSIconData->entries[x]->xorBitmapOut,
@@ -644,88 +643,82 @@ writeMS_Ico (MS_Ico const MSIconData, char * const outFname) {
 
 
 int 
-main (int argc, char ** argv) {
-   MS_Ico MSIconData = createIconFile();
-   int iconOn = 1;
-   int argn;
-   int offset;
-   const char * const usage = "[-andppms] [-output output.ico] [icon1.ppm icon2.ppm ... ]";
+main(int argc, char ** argv) {
+    MS_Ico MSIconData = createIconFile();
+    int iconOn = 1;
+    int argn;
+    int offset;
    
-   /*
-    * usage:
-    * 
-    * ppmtowinicon [-andppms] icon1.ppm [icon2.ppm icon3.ppm ... ] output.ico
-    * 
-    * Use -a flag if you supply your own and bitmaps.
-    */
-   ppm_init ( &argc, argv);
-   /*
-    * Parse command line arguments.
-    */
-   argn = 1;
-   while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
-      if (pm_keymatch(argv[argn], "-verbose", 2))
-	verbose++;
-      else if (pm_keymatch(argv[argn], "-andpgms", 2) 
-               || pm_keymatch(argv[argn], "-andppms", 2)) {
-	readAndMaps++;
-      }
-      else if (pm_keymatch(argv[argn], "-output", 2)) {
-	 if (argc - argn > 1) {
-	    outfname = argv[argn+1];
-	    argn++;
-	 } else {
-	    pm_error ("-output must be supplied a filename");
-	 }
-      }
-      else pm_usage(usage);
-      ++argn;
-   }
+    ppm_init ( &argc, argv);
+    /*
+     * Parse command line arguments.
+     */
+    argn = 1;
+    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
+        if (pm_keymatch(argv[argn], "-verbose", 2))
+            verbose++;
+        else if (pm_keymatch(argv[argn], "-andpgms", 2) 
+                 || pm_keymatch(argv[argn], "-andppms", 2)) {
+            readAndMaps++;
+        }
+        else if (pm_keymatch(argv[argn], "-output", 2)) {
+            if (argc - argn > 1) {
+                outfname = argv[argn+1];
+                argn++;
+            } else {
+                pm_error ("-output must be supplied a filename");
+            }
+        }
+        else pm_error("Invalid option '%s'", argv[argn]);
+        ++argn;
+    }
    
-   if (argn < argc) {
-      /*
-       * If there's cmd line args left over, fine. 
-       * Use them later.
-       */
-   } else {
-      ifp = stdin;
-      infname = "noname";
-      readFromStdin++;
-   }
+    if (argn < argc) {
+        /*
+         * If there's cmd line args left over, fine. 
+         * Use them later.
+         */
+    } else {
+        ifp = stdin;
+        infname = "noname";
+        readFromStdin++;
+    }
 
    
-   if (readFromStdin) {
-      addEntryToIcon(MSIconData, "-", "");
-      if (verbose) pm_message ("Added entry from stdin.\n");
-   } else {
-      /*
-       * If we're not using fake and maps, then we skip 1 each time.
-       */
-      for ( iconOn = argn; iconOn < argc ; iconOn += (readAndMaps ? 2 : 1) ) {
-          const char * xorPPM;
-          const char * andPPM;
-          xorPPM = argv[iconOn];
-          andPPM = (readAndMaps ? argv[iconOn+1] : "");
-          addEntryToIcon(MSIconData, xorPPM, andPPM);
-      }
-   }
-   /*
-    * Now we have to go through and calculate the offsets.
-    * The first infoheader starts at 6 + count*16 bytes.
-    */
-   offset = (MSIconData->count * 16) + 6;
-   for ( iconOn = 0; iconOn < MSIconData->count; iconOn++ ) {
-      IC_Entry entry = MSIconData->entries[iconOn];
-      entry->file_offset = offset;
-      /* 
-       * Increase the offset by the size of this offset & data.
-       * this includes the size of the color data.
-       */
-      offset += entry->size_in_bytes;
-   }
-   /*
-    * And now, we have to actually SAVE the .ico!
-    */
-   writeMS_Ico(MSIconData,argv[argc-1]);
-   return 0;
+    if (readFromStdin) {
+        addEntryToIcon(MSIconData, "-", "");
+        if (verbose) pm_message ("Added entry from stdin.\n");
+    } else {
+        /*
+         * If we're not using fake and maps, then we skip 1 each time.
+         */
+        for ( iconOn = argn; 
+              iconOn < argc ; 
+              iconOn += (readAndMaps ? 2 : 1) ) {
+            const char * xorPPM;
+            const char * andPPM;
+            xorPPM = argv[iconOn];
+            andPPM = (readAndMaps ? argv[iconOn+1] : "");
+            addEntryToIcon(MSIconData, xorPPM, andPPM);
+        }
+    }
+    /*
+     * Now we have to go through and calculate the offsets.
+     * The first infoheader starts at 6 + count*16 bytes.
+     */
+    offset = (MSIconData->count * 16) + 6;
+    for ( iconOn = 0; iconOn < MSIconData->count; iconOn++ ) {
+        IC_Entry entry = MSIconData->entries[iconOn];
+        entry->file_offset = offset;
+        /* 
+         * Increase the offset by the size of this offset & data.
+         * this includes the size of the color data.
+         */
+        offset += entry->size_in_bytes;
+    }
+    /*
+     * And now, we have to actually SAVE the .ico!
+     */
+    writeMS_Ico(MSIconData,argv[argc-1]);
+    return 0;
 }
