@@ -38,8 +38,8 @@
 #include <ctype.h>
 #include <string.h>
 #include "ppm.h"
-#include "ppmcmap.h"
 #include "shhopt.h"
+#include "nstring.h"
 
 /* Max number of entries we will put in the XPM color map 
    Don't forget the one entry for transparency.
@@ -60,20 +60,15 @@
 */
 #define MAXCOLORS    256
 
-/* Max number of rgb mnemonics allowed in rgb text file. */
-#define MAX_RGBNAMES 1024
-
 #define MAXPRINTABLE 92         /* number of printable ascii chars
-                     * minus \ and " for string compat
-                     * and ? to avoid ANSI trigraphs. */
+                                 * minus \ and " for string compat
+                                 * and ? to avoid ANSI trigraphs. */
 
 static const char * const printable =
 " .XoO+@#$%&*=-;:>,<1234567890qwertyuipasdfghjklzxcvbnmMNBVCZ\
 ASDFGHJKLPIUYTREWQ!~^/()_`'][{}|";
 
 
-#define max(a,b) ((a) > (b) ? (a) : (b))
-
 struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
@@ -82,163 +77,92 @@ struct cmdline_info {
     const char *name;
     const char *rgb;
     const char *alpha_filename;
-    int verbose;
+    unsigned int hexonly;
 };
 
+
+
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdline_info * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
+    optEntry *option_def = malloc(100*sizeof(optStruct));
+        /* Instructions to OptParseOptions3 on how to parse our options.
          */
-    optStruct2 opt;
+    optStruct3 opt;
 
     unsigned int option_def_index;
-    char *nameOpt;
+    const char * nameOpt;
+    unsigned int nameSpec;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "alphamask",   OPT_STRING, &cmdline_p->alpha_filename, 0);
-    OPTENTRY(0,   "name",        OPT_STRING, &nameOpt,                   0);
-    OPTENTRY(0,   "rgb",         OPT_STRING, &cmdline_p->rgb,            0);
+    OPTENT3(0,   "alphamask",   OPT_STRING, &cmdlineP->alpha_filename, 
+            NULL, 0);
+    OPTENT3(0,   "name",        OPT_STRING, &nameOpt,                   
+            &nameSpec, 0);
+    OPTENT3(0,   "rgb",         OPT_STRING, &cmdlineP->rgb,            
+            NULL, 0);
+    OPTENT3(0,   "hexonly",     OPT_FLAG, NULL,
+            &cmdlineP->hexonly, 0);
 
     /* Set the defaults */
-    cmdline_p->alpha_filename = NULL;  /* no transparency */
-    nameOpt = NULL;      /* no name */
-    cmdline_p->rgb = NULL;      /* no rgb file specified */
+    cmdlineP->alpha_filename = NULL;  /* no transparency */
+    cmdlineP->rgb = NULL;      /* no rgb file specified */
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We may have parms that are negative numbers */
 
-    optParseOptions2(&argc, argv, opt, 0);
-        /* Uses and sets argc, argv, and some of *cmdline_p and others. */
+    optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (argc-1 == 0) 
-        cmdline_p->input_filespec = "-";
+        cmdlineP->input_filespec = "-";
     else if (argc-1 != 1)
         pm_error("Program takes zero or one argument (filename).  You "
                  "specified %d", argc-1);
     else
-        cmdline_p->input_filespec = argv[1];
+        cmdlineP->input_filespec = argv[1];
 
     /* If output filename not specified, use input filename as default. */
-    if (nameOpt)
-        cmdline_p->name = nameOpt;
-    else if (strcmp(cmdline_p->input_filespec, "-") == 0)
-        cmdline_p->name = "noname";
+    if (nameSpec)
+        cmdlineP->name = nameOpt;
+    else if (strcmp(cmdlineP->input_filespec, "-") == 0)
+        cmdlineP->name = "noname";
     else {
         static char name[80+1];
         char *cp;
 
-        strncpy(name, cmdline_p->input_filespec, sizeof(name));
-        name[sizeof(name)] = '\0';
+        STRSCPY(name, cmdlineP->input_filespec);
         cp = strchr(name, '.');
         if (cp)
             *cp = '\0';     /* remove extension */
-        cmdline_p->name = name;
+        cmdlineP->name = name;
     }
 }
 
 
 typedef struct {            /* rgb values and ascii names (from
-                     * rgb text file) */
+                             * rgb text file) */
     int r, g, b;            /* rgb values, range of 0 -> 65535 */
     char *name;             /* color mnemonic of rgb value */
-}      rgb_names;
+} rgb_names;
 
 typedef struct {            
     /* Information for an XPM color map entry */
     char *cixel;    
-        /* XPM color number, as might appear in the XPM raster */
-    char *rgbname;  
+       /* XPM color number, as might appear in the XPM raster */
+    const char *rgbname;  
        /* color name, e.g. "blue" or "#01FF23" */
 } cixel_map;
 
 
 
-/*---------------------------------------------------------------------------*/
-/* This routine reads a rgb text file.  It stores the rgb values (0->65535)
-   and the rgb mnemonics (malloc'ed) into the "rgbn" array.  Returns the
-   number of entries stored in "rgbn_max". */
-static void
-read_rgb_names(rgb_fname, rgbn, rgbn_max)
-    char *rgb_fname;
-    rgb_names rgbn[MAX_RGBNAMES];
-    int *rgbn_max;
-
-{
-    FILE *rgbf;
-    int i, items, red, green, blue;
-    char line[512], name[512], *rgbname;
-
-    /* Open the rgb text file.  Abort if error. */
-    if ((rgbf = fopen(rgb_fname, "r")) == NULL)
-    pm_error("error opening rgb text file \"%s\"", rgb_fname);
-
-    /* Loop reading each line in the file. */
-    for (i = 0; fgets(line, sizeof(line), rgbf); i++) {
-
-    /* Quit if rgb text file is too large. */
-    if (i == MAX_RGBNAMES) {
-        fprintf(stderr,
-        "Too many entries in rgb text file, truncated to %d entries.\n",
-            MAX_RGBNAMES);
-        fflush(stderr);
-        break;
-    }
-    /* Read the line.  Skip if bad. */
-    items = sscanf(line, "%d %d %d %[^\n]\n", &red, &green, &blue, name);
-    if (items != 4) {
-        fprintf(stderr, "rgb text file syntax error on line %d\n", i + 1);
-        fflush(stderr);
-        i--;
-        continue;
-    }
-    /* Make sure rgb values are within 0->255 range.  Skip if bad. */
-    if (red < 0 || red > 0xFF ||
-        green < 0 || green > 0xFF ||
-        blue < 0 || blue > 0xFF) {
-        fprintf(stderr, "rgb value for \"%s\" out of range, ignoring it\n",
-            name);
-        fflush(stderr);
-        i--;
-        continue;
-    }
-    /* Allocate memory for ascii name.  Abort if error. */
-    if (!(rgbname = (char *) malloc(strlen(name) + 1)))
-        pm_error("out of memory allocating rgb name");
-        
-#ifdef NAMESLOWCASE
-    /* Copy string to ascii name and lowercase it. */
-    for (n = name, m = rgbname; *n; n++)
-        *m++ = isupper(*n) ? tolower(*n) : *n;
-    *m = '\0';
-#else
-    strcpy(rgbname, name);
-#endif
-
-    /* Save the rgb values and ascii name in the array. */
-    rgbn[i].r = red << 8;
-    rgbn[i].g = green << 8;
-    rgbn[i].b = blue << 8;
-    rgbn[i].name = rgbname;
-    }
-
-    /* Return the max number of rgb names. */
-    *rgbn_max = i - 1;
-
-    fclose(rgbf);
-
-}                   /* read_rgb_names */
-
-
-
 static char *
-gen_numstr(int const input, int const digits) {
+genNumstr(int const input, int const digits) {
 /*---------------------------------------------------------------------------
    Given a number and a base (MAXPRINTABLE), this routine prints the
    number into a malloc'ed string and returns it.  The length of the
@@ -276,22 +200,37 @@ gen_numstr(int const input, int const digits) {
 
 
 static void
-gen_cmap(colorhist_vector const chv, int const ncolors, 
-         pixval const maxval, int map_rgb_names, 
-         rgb_names rgbn[MAX_RGBNAMES], int const rgbn_max,
-         cixel_map ** const cmapP, int const charsPerPixel) {
+genCmap(colorhist_vector const chv, 
+        int              const ncolors, 
+        pixval           const maxval, 
+        colorhash_table  const colornameHash,
+        const char *           colornames[],
+        cixel_map **     const cmapP, 
+        int              const charsPerPixel) {
 /*----------------------------------------------------------------------------
    Generate the XPM color map in cixel_map format (which is just a step
-   away from the actual text that needs to be written the XPM file).
+   away from the actual text that needs to be written the XPM file).  The
+   color map is defined by 'chv', which contains 'ncolors' colors which
+   have maxval 'maxval'.
 
    Output is in newly malloc'ed storage, with its address returned as
    *cmapP.
 
    This map includes an entry for transparency, whether the raster uses
    it or not.  Its index is ncolors (so cmap's dimension is ncolors +1).
+
+   In the map, identify colors by the names give by 'colornameHash' and
+   colornames[].  'colornameHash' maps a color in 'pixel' form to an
+   index into colornames[]; colornames[] contains the text to identify the
+   color in the XPM format.  The colors in 'colornameHash' have maxval 255.
+   If a color is not in 'colornameHash', use hexadecimal notation in the
+   output colormap.
+
+   But if 'colornameHash' is null, don't use color names at all.  Just use
+   hexadecimal notation.
 -----------------------------------------------------------------------------*/
-    int i, j, mval, red, green, blue, r, g, b, matched;
-    char *str;
+    int cmapIndex;
+    int mval;
     cixel_map * cmap;  /* malloc'ed */
 
     cmap = malloc(sizeof(cixel_map) * (ncolors+1));
@@ -302,120 +241,74 @@ gen_cmap(colorhist_vector const chv, int const ncolors,
      * Determine how many hex digits we'll be normalizing to if the rgb
      * value doesn't match a color mnemonic. 
      */
-    mval = (int) maxval;
+    mval = (unsigned int) maxval;
     if (mval <= 0x000F)
-    mval = 0x000F;
+        mval = 0x000F;
     else if (mval <= 0x00FF)
-    mval = 0x00FF;
+        mval = 0x00FF;
     else if (mval <= 0x0FFF)
-    mval = 0x0FFF;
+        mval = 0x0FFF;
     else
-    mval = 0xFFFF;
-
+        mval = 0xFFFF;
+    
     /*
      * Generate the character-pixel string and the rgb name for each
      * colormap entry. 
      */
-    for (i = 0; i < ncolors; i++) {
-
-    /*
-     * The character-pixel string is simply a printed number in base
-     * MAXPRINTABLE where the digits of the number range from
-     * printable[0] .. printable[MAXPRINTABLE-1] and the printed length
-     * of the number is 'charsPerPixel'. 
-     */
-    cmap[i].cixel = gen_numstr(i, charsPerPixel);
-
-    /* Fetch the rgb value of the current colormap entry. */
-    red = PPM_GETR(chv[i].color);
-    green = PPM_GETG(chv[i].color);
-    blue = PPM_GETB(chv[i].color);
-
-    /*
-     * If the ppm color components are not relative to 15, 255, 4095,
-     * 65535, normalize the color components here. 
-     */
-    if (mval != (int) maxval) {
-        red = (red * mval) / (int) maxval;
-        green = (green * mval) / (int) maxval;
-        blue = (blue * mval) / (int) maxval;
-    }
-
-    /*
-     * If the "-rgb <rgbfile>" option was specified, attempt to map the
-     * rgb value to a color mnemonic. 
-     */
-    if (map_rgb_names) {
+    for (cmapIndex = 0; cmapIndex < ncolors; ++cmapIndex) {
+        pixel const color = chv[cmapIndex].color;
 
+        pixel color255;
+            /* The color, scaled to maxval 255 */
+        const char * colorname;  /* malloc'ed */
         /*
-         * The rgb values of the color mnemonics are normalized relative
-         * to 255 << 8, (i.e. 0xFF00).  [That's how the original MIT
-         * code did it, really should have been "v * 65535 / 255"
-         * instead of "v << 8", but have to use the same scheme here or
-         * else colors won't match...]  So, if our rgb values aren't
-         * already 16-bit values, need to shift left. 
+         * The character-pixel string is simply a printed number in base
+         * MAXPRINTABLE where the digits of the number range from
+         * printable[0] .. printable[MAXPRINTABLE-1] and the printed length
+         * of the number is 'charsPerPixel'. 
          */
-        if (mval == 0x000F) {
-        r = red << 12;
-        g = green << 12;
-        b = blue << 12;
-        /* Special case hack for "white". */
-        if (0xF000 == r && r == g && g == b)
-            r = g = b = 0xFF00;
-        } else if (mval == 0x00FF) {
-        r = red << 8;
-        g = green << 8;
-        b = blue << 8;
-        } else if (mval == 0x0FFF) {
-        r = red << 4;
-        g = green << 4;
-        b = blue << 4;
-        } else {
-        r = red;
-        g = green;
-        b = blue;
-        }
+        cmap[cmapIndex].cixel = genNumstr(cmapIndex, charsPerPixel);
+        
+        PPM_DEPTH(color255, color, maxval, 255);
 
-        /*
-         * Just perform a dumb linear search over the rgb values of the
-         * color mnemonics.  One could speed things up by sorting the
-         * rgb values and using a binary search, or building a hash
-         * table, etc... 
-         */
-        for (matched = 0, j = 0; j <= rgbn_max; j++)
-        if (r == rgbn[j].r && g == rgbn[j].g && b == rgbn[j].b) {
+        if (colornameHash == NULL)
+            colorname = NULL;
+        else {
+            int colornameIndex;
+            colornameIndex = ppm_lookupcolor(colornameHash, &color255);
+            if (colornameIndex >= 0)
+                colorname = strdup(colornames[colornameIndex]);
+            else
+                colorname = NULL;
+        }
+        if (colorname)
+            cmap[cmapIndex].rgbname = colorname;
+        else {
+            /* Color has no name; represent it in hexadecimal */
 
-            /* Matched.  Allocate string, copy mnemonic, and exit. */
-            if (!(str = (char *) malloc(strlen(rgbn[j].name) + 1)))
-            pm_error("out of memory");
-            strcpy(str, rgbn[j].name);
-            cmap[i].rgbname = str;
-            matched = 1;
-            break;
+            pixel scaledColor;
+            const char * hexString;  /* malloc'ed */
+
+            PPM_DEPTH(scaledColor, color, maxval, mval);
+
+            asprintfN(&hexString, mval == 0x000F ? "#%X%X%X" :
+                      mval == 0x00FF ? "#%02X%02X%02X" :
+                      mval == 0x0FFF ? "#%03X%03X%03X" :
+                      "#%04X%04X%04X", 
+                      PPM_GETR(scaledColor),
+                      PPM_GETG(scaledColor),
+                      PPM_GETB(scaledColor)
+                );
+
+            if (hexString == NULL)
+                pm_error("Unable to allocate storage for hex string");
+            cmap[cmapIndex].rgbname = hexString;
         }
-        if (matched)
-        continue;
     }
 
-    /*
-     * Either not mapping to color mnemonics, or didn't find a match.
-     * Generate an absolute #RGB value string instead. 
-     */
-    if (!(str = (char *) malloc(mval == 0x000F ? 5 :
-                    mval == 0x00FF ? 8 :
-                    mval == 0x0FFF ? 11 :
-                    14)))
-        pm_error("out of memory");
-
-    sprintf(str, mval == 0x000F ? "#%X%X%X" :
-        mval == 0x00FF ? "#%02X%02X%02X" :
-        mval == 0x0FFF ? "#%03X%03X%03X" :
-        "#%04X%04X%04X", red, green, blue);
-    cmap[i].rgbname = str;
-    }
     /* Add the special transparency entry to the colormap */
     cmap[ncolors].rgbname = strdup("None");
-    cmap[ncolors].cixel = gen_numstr(ncolors, charsPerPixel);
+    cmap[ncolors].cixel = genNumstr(ncolors, charsPerPixel);
 
     *cmapP = cmap;
 }
@@ -423,15 +316,15 @@ gen_cmap(colorhist_vector const chv, int const ncolors,
 
 
 static void
-destroy_cmap(cixel_map * const cmap, int const ncolors) {
+destroyCmap(cixel_map * const cmap, int const ncolors) {
 
     int i;
     /* Free the real color entries */
     for (i = 0; i < ncolors; i++) {
-        free(cmap[i].rgbname);
+        strfree(cmap[i].rgbname);
         free(cmap[i].cixel);
     }
-    free(cmap[ncolors].rgbname);  /* The transparency entry */
+    strfree(cmap[ncolors].rgbname);  /* The transparency entry */
     free(cmap[ncolors].cixel);  /* The transparency entry */
 
     free(cmap);
@@ -495,8 +388,8 @@ writeXpmFile(FILE * const outfile, pixel ** const pixels,
 
 
 static void
-read_alpha(const char filespec[], gray *** const alphaP,
-           int const cols, int const rows, pixval * const alphamaxvalP) {
+readAlpha(const char filespec[], gray *** const alphaP,
+          int const cols, int const rows, pixval * const alphamaxvalP) {
 
     FILE * alpha_file;
     int alphacols, alpharows;
@@ -513,10 +406,10 @@ read_alpha(const char filespec[], gray *** const alphaP,
     
 
 static void
-compute_colormap(pixel ** const pixels, int const cols, int const rows,
-                 colorhist_vector * const chvP, 
-                 colorhash_table * const chtP,
-                 int * const ncolorsP) {
+computeColormap(pixel ** const pixels, int const cols, int const rows,
+                colorhist_vector * const chvP, 
+                colorhash_table * const chtP,
+                int * const ncolorsP) {
 /*----------------------------------------------------------------------------
    Compute the color map for the image 'pixels', which is 'cols' by 'rows',
    in Netpbm data structures (a colorhist_vector for index-to-color lookups
@@ -551,13 +444,16 @@ main(int argc, char *argv[]) {
     colorhash_table cht;
     colorhist_vector chv;
 
+    colorhash_table colornameHash;
+        /* Hash table to map colors to their names */
+    const char ** colornames;
+        /* Table of color names; 'colornameHash' yields an index into this
+           array.
+        */
+
     pixel **pixels;
     gray **alpha;
 
-    /* Used for rgb value -> rgb mnemonic mapping */
-    rgb_names rgbn[MAX_RGBNAMES];
-    int rgbn_max;
-
     /* Used for rgb value -> character-pixel string mapping */
     cixel_map *cmap;  /* malloc'ed */
     int charsPerPixel;  
@@ -566,24 +462,23 @@ main(int argc, char *argv[]) {
 
     ppm_init(&argc, argv);
 
-    parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
 
     ifp = pm_openr(cmdline.input_filespec);
     pixels = ppm_readppm(ifp, &cols, &rows, &maxval);
     pm_close(ifp);
 
     if (cmdline.alpha_filename) 
-        read_alpha(cmdline.alpha_filename, &alpha, cols, rows, &alphamaxval);
+        readAlpha(cmdline.alpha_filename, &alpha, cols, rows, &alphamaxval);
     else
         alpha = NULL;
 
-    compute_colormap(pixels, cols, rows, &chv, &cht, &ncolors);
-    /*
-     * If a rgb text file was specified, read in the rgb mnemonics. Does not
-     * return if fatal error occurs. 
-     */
-    if (cmdline.rgb)
-        read_rgb_names(cmdline.rgb, rgbn, &rgbn_max);
+    computeColormap(pixels, cols, rows, &chv, &cht, &ncolors);
+
+    if (cmdline.hexonly)
+        colornameHash = NULL;
+    else
+        ppm_readcolornamefile(cmdline.rgb, FALSE, &colornameHash, &colornames);
 
     { 
         /* Compute characters per pixel in the XPM file */
@@ -592,17 +487,20 @@ main(int argc, char *argv[]) {
             j /= MAXPRINTABLE;
     }
     /* Now generate the character-pixel colormap table. */
-    gen_cmap(chv, ncolors, maxval, cmdline.rgb != NULL, rgbn, rgbn_max,
-             &cmap, charsPerPixel);
+    genCmap(chv, ncolors, maxval, colornameHash, colornames,
+            &cmap, charsPerPixel);
 
     writeXpmFile(stdout, pixels, alpha, alphamaxval,
                  cmdline.name, cols, rows, ncolors + 1, 
                  charsPerPixel, cmap, cht);
     
-    destroy_cmap(cmap, ncolors);
+    if (colornameHash) {
+        ppm_freecolorhash(colornameHash);
+        ppm_freecolornames(colornames);
+    }
+    destroyCmap(cmap, ncolors);
     ppm_freearray(pixels, rows);
     if (alpha) pgm_freearray(alpha, rows);
 
     return 0;
 }
-
