@@ -1,6 +1,6 @@
 /* winicontoppm.c - read a MS Windows .ico file and write portable pixmap(s)
 **
-** Copyright (C) 2000 by Lee Benfield - lee@recoil.org
+** Copyright (C) 2000,2003 by Lee Benfield - lee@benf.org
 **
 ** Permission to use, copy, modify, and distribute this software and its
 ** documentation for any purpose and without fee is hereby granted, provided
@@ -8,29 +8,108 @@
 ** copyright notice and this permission notice appear in supporting
 ** documentation.  This software is provided "as is" without express or
 ** implied warranty.
+**
+** Changes:
+** 
+** 03/2003 - Added 24+32 bpp support.
 */
 
 #include <math.h>
 #include <string.h>
+#include <assert.h>
 
 #include "ppm.h"
+#include "shhopt.h"
+#include "nstring.h"
 #include "winico.h"
 
 #define MAJVERSION 0
-#define MINVERSION 3
+#define MINVERSION 4
 
-static int      writeToFile = 0;
-static int      verbose = 0;
-static int      allicons = 0;
-static int      writeands = 0;
-static int      bestqual = 0;
-static int      multippm = 0;
-static int      file_offset = 0;    /* not actually used, but useful for debug */
+static int file_offset = 0;    /* not actually used, but useful for debug */
 static const char     er_read[] = "%s: read error";
 static const char *   infname;
-static const char *   outfname;
 static FILE *   ifp;
 
+struct cmdlineInfo {
+    /* All the information the user supplied in the command line,
+       in a form easy for the program to use.
+    */
+    const char * inputFilespec;
+    const char * outputFilespec;
+    unsigned int allicons;
+    unsigned int bestqual;
+    unsigned int writeands;
+    unsigned int multippm;
+    unsigned int verbose;
+};
+
+
+
+
+static void
+parseCommandLine ( int argc, char ** argv,
+                   struct cmdlineInfo *cmdlineP ) {
+/*----------------------------------------------------------------------------
+   parse program command line described in Unix standard form by argc
+   and argv.  Return the information in the options as *cmdlineP.  
+
+   If command line is internally inconsistent (invalid options, etc.),
+   issue error message to stderr and abort program.
+
+   Note that the strings we return are stored in the storage that
+   was passed to us as the argv array.  We also trash *argv.
+-----------------------------------------------------------------------------*/
+    optEntry *option_def = malloc(100*sizeof(optEntry));
+        /* Instructions to optParseOptions3 on how to parse our options.
+         */
+    optStruct3 opt;
+
+    unsigned int option_def_index;
+
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0, "allicons",     OPT_FLAG,   NULL,                  
+            &cmdlineP->allicons,       0 );
+    OPTENT3(0, "bestqual",     OPT_FLAG,   NULL,                  
+            &cmdlineP->bestqual,       0 );
+    OPTENT3(0, "writeands",    OPT_FLAG,   NULL,                  
+            &cmdlineP->writeands,      0 );
+    OPTENT3(0, "multippm",     OPT_FLAG,   NULL,                  
+            &cmdlineP->multippm,       0 );
+    OPTENT3(0, "verbose",      OPT_FLAG,   NULL,                  
+            &cmdlineP->verbose,        0 );
+
+    opt.opt_table = option_def;
+    opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
+    opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
+
+    optParseOptions3( &argc, argv, opt, sizeof(opt), 0);
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
+
+
+    if (argc-1 < 1) 
+        cmdlineP->inputFilespec = "-";
+    else
+        cmdlineP->inputFilespec = argv[1];
+
+    if (argc-1 < 2) {
+        cmdlineP->outputFilespec = "-";
+        
+        if (cmdlineP->writeands || cmdlineP->allicons)
+            pm_error("If you specify the -writeands or -allicons option, "
+                     "you must also specify an output file name argument.");
+    } else
+        cmdlineP->outputFilespec = argv[2];
+
+    if (argc-1 > 2)
+        pm_error("Too many arguments (%d).  Input filespec and "
+                 "output filespec are the only possible arguments.",
+                 argc-1);
+}
+
+
+
+
 static int 
 GetByte(void) {
     int v;
@@ -114,19 +193,18 @@ readICEntry (void)
     entry->bitcount      = readU2();
     entry->size_in_bytes = readU4();
     entry->file_offset   = readU4();
-
-    /*
-     * Spec says 0 color count is really 256.
-     */
-    if (entry->color_count == 0) entry->color_count = 256;
-   
+    entry->colors        = NULL;
+    entry->ih            = NULL;
+    entry->xorBitmap     = NULL;
+    entry->andBitmap     = NULL;
+    
     return entry;
 }
 
 
 
 static IC_InfoHeader 
-readInfoHeader (void) 
+readInfoHeader (IC_Entry entry) 
 {
     IC_InfoHeader ih = malloc ( sizeof (* ih) );
     ih->size            = readU4();
@@ -140,6 +218,13 @@ readInfoHeader (void)
     ih->y_pixels_per_m  = readU4();
     ih->colors_used     = readU4();
     ih->colors_important = readU4();
+    
+    if (!entry->bitcount) entry->bitcount = ih->bitcount;
+    if (entry->color_count == 0 && 
+        entry->bitcount <= 8) entry->color_count = 256;
+    if (ih->compression) {
+        pm_error("Can't handle compressed icons");
+    }
     return ih;
 }
 
@@ -189,7 +274,8 @@ read1Bitmap (int width, int height)
         int xOrVal = 128;
         u1 * row = readU1String(xBytes);
         for (x = 0; x< width; x++) {
-            *(bitmap+((height-tmp-1)*width) + (x)) = (row[rowByte] & xOrVal) / xOrVal;
+            *(bitmap+((height-tmp-1)*width) + (x)) = 
+                (row[rowByte] & xOrVal) / xOrVal;
             if (xOrVal == 1) {
                 xOrVal = 128;
                 rowByte++;
@@ -197,6 +283,7 @@ read1Bitmap (int width, int height)
                 xOrVal >>= 1;
             }
         }
+        free(row);
     }
     return bitmap;
 }
@@ -225,13 +312,15 @@ read4Bitmap (int width, int height)
              * 2 nibbles, 2 values.
              */
             if (bottom) {
-                *(bitmap+((height-tmp-1)*width) + (x)) = (row[rowByte] & 0xF0) >> 4;
+                *(bitmap+((height-tmp-1)*width) + (x)) = 
+                    (row[rowByte] & 0xF0) >> 4;
             } else {
                 *(bitmap+((height-tmp-1)*width) + (x)) = (row[rowByte] & 0xF);
                 rowByte++;
             }
             bottom = !bottom;
         }
+    free(row);
     }
     return bitmap;
 }
@@ -245,7 +334,6 @@ read8Bitmap (int width, int height)
     unsigned int xBytes;
     unsigned int wt = width;
     u1 * bitmap = malloc ( (width * height ) * (sizeof (u1)) );
-
    
     if (wt & 3) {
         wt = (wt & ~3) + 4;
@@ -259,15 +347,36 @@ read8Bitmap (int width, int height)
             *(bitmap+((height-tmp-1)*width) + (x)) = row[rowByte];
             rowByte++;
         }
+        free(row);
     }
     return bitmap;
 }
-   
 
+/*
+ * Read a true color bitmap. (24/32 bits)
+ * 
+ * The output routine deplanarizes it for us, we keep it flat here.
+ */
+static u1 *
+readXBitmap (int width, int height, int bpp)
+{
+    int tmp = 0;
+    int bytes = bpp >> 3;
+    u1 * bitmap = malloc ( (bytes * width * height ) * (sizeof (u1)) );
+    /* remember - bmp (dib) stored upside down, so reverse */
+    u1 * bitcurptr = bitmap + (bytes * width * (height-1) ) * (sizeof (u1));
+    unsigned int xBytes = width * bytes;
+
+    for (tmp = 0;tmp < height; tmp++, bitcurptr -= xBytes) {
+        u1 * row = readU1String(xBytes);
+        memcpy(bitcurptr, row, xBytes);
+        free(row);
+    }
+    return bitmap;
+}
 
 static MS_Ico 
-readIconFile (void) 
-{
+readIconFile (bool const verbose) {
     int iter,iter2;
 
     MS_Ico MSIconData= malloc( sizeof (* MSIconData) );
@@ -302,34 +411,48 @@ readIconFile (void)
     if (verbose) fprintf (stderr,"#\tColors\tBPP\tWidth\tHeight\n");
     for (iter = 0;iter < MSIconData->count ; iter++ ) {
         int bpp;
-        MSIconData->entries[iter]->ih = readInfoHeader ();
-        MSIconData->entries[iter]->colors = 
-            malloc (MSIconData->entries[iter]->color_count * 
-                    sizeof(IC_Color *));
-        for (iter2 = 0;
-             iter2 < MSIconData->entries[iter]->color_count ; 
-             iter2++ ) {
-            MSIconData->entries[iter]->colors[iter2] = readICColor();
+        MSIconData->entries[iter]->ih =
+            readInfoHeader (MSIconData->entries[iter]);
+       
+        /* What's the bits per pixel? */
+        bpp = MSIconData->entries[iter]->bitcount; 
+        /* Read the palette, if appropriate */
+        switch (bpp) {
+        case 24:
+        case 32:
+            /* 24/32 bpp icon has no palette */
+            break;
+        default:
+            MSIconData->entries[iter]->colors = 
+                malloc (MSIconData->entries[iter]->color_count * 
+                        sizeof(IC_Color *));
+            for (iter2 = 0;
+                 iter2 < MSIconData->entries[iter]->color_count ; 
+                 iter2++ ) {
+                MSIconData->entries[iter]->colors[iter2] = readICColor();
+            }
+            break;
         }
-        /* What's the bits per pixel?  Bit confusing, since there's a
-         * field in entry, and in the infoheader.  I'll use both but
-         * let the entry field take precedence.  
-         */
-        bpp = MSIconData->entries[iter]->bitcount ? 
-            MSIconData->entries[iter]->bitcount : 
-            MSIconData->entries[iter]->ih->bitcount;
-        if (verbose) 
+        if (verbose) {
+            char cols_text[10];
+            sprintf (cols_text, "%d", MSIconData->entries[iter]->color_count);
             fprintf (stderr,
-                     "%d\t%d\t%d\t%d\t%d\n", iter,
-                     MSIconData->entries[iter]->color_count, 
+                     "%d\t%s\t%d\t%d\t%d\n", iter,
+                     MSIconData->entries[iter]->color_count ? 
+                     cols_text : "TRUE",
                      bpp, MSIconData->entries[iter]->width, 
                      MSIconData->entries[iter]->height);
+        }
         /* Pixels are stored bottom-up, left-to-right. Pixel lines are
          * padded with zeros to end on a 32bit (4byte) boundary. Every
          * line will have the same number of bytes. Color indices are
          * zero based, meaning a pixel color of 0 represents the first
          * color table entry, a pixel color of 255 (if there are that
          * many) represents the 256th entry.  
+         * 
+         * 24+32 bit (16 is an abomination, which I'll avoid, and expect
+         * no-one to mind) are stored 1byte/plane with a spare (alpha?)
+         * byte for 32 bit.
          */
         {
             /*
@@ -351,8 +474,14 @@ readIconFile (void)
                     read8Bitmap(MSIconData->entries[iter]->width,
                                 MSIconData->entries[iter]->height);
                 break;
+            case 24:
+            case 32:
+                MSIconData->entries[iter]->xorBitmap = 
+                    readXBitmap(MSIconData->entries[iter]->width,
+                                MSIconData->entries[iter]->height,bpp);
+                break;
             default:
-                pm_error("Uncatered bit depth %d\n",bpp);
+                pm_error("Uncatered bit depth %d",bpp);
             }
             /*
              * Read AND Bitmap
@@ -369,7 +498,7 @@ readIconFile (void)
 
 
 static char * 
-trimOutputName (char * inputName)
+trimOutputName(const char inputName[])
 {
     /*
      * Just trim off the final ".ppm", if there is one, else return as is.
@@ -408,12 +537,13 @@ getBestQualityIcon(MS_Ico MSIconData)
     return best;
 }
 
-
-
 static void
-writeXors(bool const writeToFile, FILE * const multiOutF,
-          char outputFileBase[], IC_Entry const entry,
-          int const entryNum, bool const multiple, bool const xor) {
+writeXors(FILE *   const multiOutF,
+          char           outputFileBase[], 
+          IC_Entry const entry,
+          int      const entryNum,
+          bool     const multiple, 
+          bool     const xor) {
 /*----------------------------------------------------------------------------
    Write an "xor" image (i.e. the main image) out.
 
@@ -431,23 +561,25 @@ writeXors(bool const writeToFile, FILE * const multiOutF,
     FILE * outF;
     pixel ** ppm_array;
     int row;
-    char *outputFile;
+    int pel_size;
+    const char *outputFile;
+    int maxval;
+    int forcetext;
 
-    outputFile = malloc(sizeof(char) * (strlen (outputFileBase) + 20));
-
-    if (multiOutF)
+    if (multiOutF) {
         outF = multiOutF;
-    else {
-        if (writeToFile) {
+        outputFile = strdup("");
+    } else {
+        if (outputFileBase) {
             if (multiple) {
-                sprintf(outputFile, "%s%s_%d.ppm",
-                        outputFileBase,(xor ? "_xor" : ""), entryNum);
+                asprintfN(&outputFile, "%s%s_%d.ppm",
+                          outputFileBase,(xor ? "_xor" : ""), entryNum);
             } else { 
-                sprintf(outputFile, "%s%s.ppm",
-                        outputFileBase,(xor ? "_xor" : ""));
+                asprintfN(&outputFile, "%s%s.ppm",
+                          outputFileBase,(xor ? "_xor" : ""));
             }
         } else
-            strcpy(outputFile, "-");
+            outputFile = strdup("-");
         
         outF = pm_openw(outputFile);
     }
@@ -460,20 +592,39 @@ writeXors(bool const writeToFile, FILE * const multiOutF,
     for (row=0; row < entry->height; row++) {
         u1 * xorRow;
         int col;
-        xorRow = entry->xorBitmap + row * entry->width;
-        for (col=0; col < entry->width; col++) {
-            int colorIndex;
-            IC_Color color;
-            colorIndex  = xorRow[col];
-            color = entry->colors[colorIndex];
-            PPM_ASSIGN(ppm_array[row][col],
-                       color->red,color->green,color->blue);
+        switch (entry->bitcount) {
+        case 24:
+        case 32:
+            pel_size = entry->bitcount >> 3;
+            xorRow = entry->xorBitmap + row * entry->width * pel_size;
+            for (col=0; col < entry->width*pel_size;col+=pel_size) {
+                PPM_ASSIGN(ppm_array[row][col/pel_size],
+                           xorRow[col+2],xorRow[col+1],xorRow[col]);
+            }
+            break;
+        default:
+            xorRow = entry->xorBitmap + row * entry->width;
+            for (col=0; col < entry->width; col++) {
+                int colorIndex;
+                IC_Color color;
+                colorIndex  = xorRow[col];
+                color = entry->colors[colorIndex];
+                PPM_ASSIGN(ppm_array[row][col],
+                           color->red,color->green,color->blue);
+            }
+            break;
         }
     }    
+    
+    maxval = 255;
+    forcetext = 0;
+
     ppm_writeppm(outF,ppm_array,entry->width, entry->height, 
-                 (pixval) 255, 0);
+                 (pixval) maxval, forcetext);
     ppm_freearray(ppm_array,entry->height);
 
+    strfree(outputFile);
+    
     if (!multiOutF) 
         pm_close(outF);
 }
@@ -485,7 +636,7 @@ writeAnds(FILE * const multiOutF,
           char outputFileBase[], IC_Entry const entry, int const entryNum, 
           bool multiple) {
 /*----------------------------------------------------------------------------
-   Write an "and" image (i.e. the alpha mask) out.
+   Write the "and" image (i.e. the alpha mask) of the image 'IC_Entry' out.
 
    'multiple' means this is one of multiple images that are being written.
    'entryNum' is the sequence number within the winicon file of the image
@@ -504,16 +655,16 @@ writeAnds(FILE * const multiOutF,
     if (multiOutF)
         outF = multiOutF;
     else {
-        char *outputFile;
+        const char *outputFile;
 
-        outputFile =  malloc ( sizeof (char) * (strlen (outputFileBase) + 20));
+        assert(outputFileBase);
 
-        if (allicons) 
-            sprintf(outputFile, "%s_and_%d.pbm", outputFileBase, entryNum);
+        if (multiple) 
+            asprintfN(&outputFile, "%s_and_%d.pbm", outputFileBase, entryNum);
         else 
-            sprintf(outputFile, "%s_and.pbm", outputFileBase);
+            asprintfN(&outputFile, "%s_and.pbm", outputFileBase);
         outF = pm_openw(outputFile);
-        free(outputFile);
+        strfree(outputFile);
     }
     pbm_array = pbm_allocarray(entry->width, entry->height);
     for (row=0; row < entry->height; row++) {
@@ -529,28 +680,30 @@ writeAnds(FILE * const multiOutF,
 
     pbm_freearray(pbm_array, entry->height);
     if (!multiOutF)
-        pm_close (outF);	 
+        pm_close (outF);     
 }
 
 
 
 static void
-openMultiXor(char outputFileBase[], FILE ** const multiOutFP) {
+openMultiXor(char          outputFileBase[], 
+             bool    const writeands,
+             FILE ** const multiOutFP) {
 
-    char *outputFile;
+    const char *outputFile;
+
+    if (outputFileBase) {
+        asprintfN(&outputFile, "%s%s.ppm",
+                  outputFileBase, (writeands ? "_xor" : ""));
+    } else
+        outputFile = strdup("-");
 
-    outputFile =  malloc ( sizeof (char) * (strlen (outputFileBase) + 20));
     /*
      * Open the output file now, it'll stay open the whole time.
      */
-    if (writeToFile) {
-        sprintf(outputFile, "%s%s.ppm",
-                outputFileBase, (writeands ? "_xor" : ""));
-    } else {
-        sprintf(outputFile,"-");
-    }
     *multiOutFP = pm_openw(outputFile);
-    free(outputFile);
+
+    strfree(outputFile);
 }
 
 
@@ -558,84 +711,67 @@ openMultiXor(char outputFileBase[], FILE ** const multiOutFP) {
 static void
 openMultiAnd(char outputFileBase[], FILE ** const multiAndOutFP) {
 
-    char *outputFile;
+    const char *outputFile;
 
-    outputFile =  malloc ( sizeof (char) * (strlen (outputFileBase) + 20));
-    
-    sprintf(outputFile, "%s_and.pbm", outputFileBase);
+    assert(outputFileBase);
+
+    asprintfN(&outputFile, "%s_and.pbm", outputFileBase);
     
     *multiAndOutFP = pm_openw(outputFile);
-    free(outputFile);
+
+    strfree(outputFile);
+}
+
+static void free_iconentry(IC_Entry entry) {
+    int x;
+    if (entry->colors && entry->color_count) {
+        for (x=0;x<entry->color_count;x++) free(entry->colors[x]);
+        free(entry->colors);
+    }
+    if (entry->andBitmap) free(entry->andBitmap);
+    if (entry->xorBitmap) free(entry->xorBitmap);
+    if (entry->ih) free(entry->ih);
+    free(entry);
+}
+
+static void free_icondata(MS_Ico MSIconData)
+{
+    int x;
+    for (x=0;x<MSIconData->count;x++) {
+    free_iconentry(MSIconData->entries[x]);
+    }
+    free(MSIconData);
 }
 
 
+int 
+main(int argc, char *argv[]) {
 
-
-int main (int argc, char *argv[]) {
-
-    int argn;
+    struct cmdlineInfo cmdline;
     int startEntry, endEntry;
-    const char * const usage = "[-writeands] [-allicons|-bestqual] [-multippm] "
-        "[-verbose] [iconfile] [ppmdestfile]";
     MS_Ico MSIconData;
     char * outputFileBase;
-    char * outputFile;
     FILE * multiOutF;
     FILE * multiAndOutF;
    
     ppm_init (&argc, argv);
-    /*
-     * Parse command line arguments.
-     */
-    argn = 1;
-    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
-        if (pm_keymatch(argv[argn], "-verbose", 2))
-            verbose++;
-        else if (pm_keymatch(argv[argn], "-allicons", 2))
-            allicons++;
-        else if (pm_keymatch(argv[argn], "-bestqual", 2))
-            bestqual++;
-        else if (pm_keymatch(argv[argn], "-writeands", 2))
-            writeands++;
-        else if (pm_keymatch(argv[argn], "-multippm", 2))
-            multippm++;
-        else pm_usage(usage);
-        ++argn;
-    }
+
+    parseCommandLine(argc, argv, &cmdline);
+
+    if (cmdline.bestqual && cmdline.allicons)
+        pm_message("-bestqual doesn't make sense with -allicons.  "
+                   "Ignoring -bestqual.");
    
-    if (bestqual && allicons) {
-        pm_message ("bestqual flag ignored.");
-        bestqual--;
-    }
-   
-    if (argn < argc) {
-        ifp = pm_openr(argv[argn]);
-        infname = argv[argn];
-        ++argn;
-    } else {
-        ifp = stdin;
-        infname = "noname";
-    }
-   
-    if (argn < argc && strcmp(argv[argn],"-")) {
-        outputFileBase = trimOutputName(argv[argn]);
-        outfname = argv[argn];
-        outputFile =  malloc ( sizeof (char) * (strlen (outputFileBase) + 20));
-        writeToFile++;
-        ++argn;
-    } else {
-        outfname = "noname";
-        if (allicons || writeands) {
-            pm_error("When using -allicons or -writeands, "
-                     "please supply an output filename.\n");
-        }
-        if (argn < argc) argn++;
-    }
-   
-    if (argn != argc)
-        pm_usage(usage);
-   
-    MSIconData = readIconFile ();
+    if (strcmp(cmdline.outputFilespec, "-") == 0)
+        outputFileBase = NULL;
+    else
+        outputFileBase = trimOutputName(cmdline.outputFilespec);
+
+    ifp = pm_openr(cmdline.inputFilespec);
+
+    infname = cmdline.inputFilespec;
+
+    MSIconData = readIconFile(cmdline.verbose);
     /*
      * Now we've read the icon file in (Hopefully! :)
      * Go through each of the entries, and write out files of the
@@ -650,7 +786,7 @@ int main (int argc, char *argv[]) {
      * If allicons is set, we want everything, if not, just go through once.
      */
     startEntry = 0;
-    if (allicons) {
+    if (cmdline.allicons) {
         endEntry = MSIconData->count;
     } else {
         endEntry = 1;
@@ -658,17 +794,17 @@ int main (int argc, char *argv[]) {
     /*
      * If bestqual is set, find the icon with highest size & bpp.
      */
-    if (bestqual) {
+    if (cmdline.bestqual) {
         startEntry = getBestQualityIcon(MSIconData);
         endEntry = startEntry+1;
     }
    
-    if (multippm) 
-        openMultiXor(outputFileBase, &multiOutF);
+    if (cmdline.multippm) 
+        openMultiXor(outputFileBase, cmdline.writeands, &multiOutF);
     else
         multiOutF = NULL;
 
-    if (writeands && multippm) 
+    if (cmdline.writeands && cmdline.multippm) 
         openMultiAnd(outputFileBase, &multiAndOutF);
     else
         multiAndOutF = NULL;
@@ -679,17 +815,19 @@ int main (int argc, char *argv[]) {
         for (entryNum = startEntry ; entryNum < endEntry ; entryNum++ ) {
             IC_Entry const entry = MSIconData->entries[entryNum];
 
-            writeXors(writeToFile, multiOutF, outputFileBase, entry, entryNum, 
-                      allicons, writeands);
-            if (writeands)
+            writeXors(multiOutF, outputFileBase, entry, entryNum, 
+                      cmdline.allicons, cmdline.writeands);
+            if (cmdline.writeands)
                 writeAnds(multiAndOutF, outputFileBase, 
-                          entry, entryNum, allicons);
+                          entry, entryNum, cmdline.allicons);
         }
     }
     if (multiOutF)
-        pm_close (multiOutF);	 
+        pm_close (multiOutF);    
     if (multiAndOutF)
         pm_close(multiAndOutF);
+    
+    /* free up the image data here. */
+    free_icondata(MSIconData);
     return 0;
 }
-
