@@ -33,6 +33,7 @@
 
 #include "ppm.h"
 #include "shhopt.h"
+#include "nstring.h"
 
 #define MAX_LINE 2048
   /* The maximum size XPM input line we can handle. */
@@ -172,8 +173,8 @@ getword(char * const output, char ** const cursorP) {
     char *t1;
     char *t2;
 
-    for (t1=*cursorP; isspace(*t1); t1++); /* skip white space */
-    for (t2 = t1; !isspace(*t2) && *t2 != '"' && *t2 != '\0'; t2++);
+    for (t1=*cursorP; ISSPACE(*t1); t1++); /* skip white space */
+    for (t2 = t1; !ISSPACE(*t2) && *t2 != '"' && *t2 != '\0'; t2++);
         /* Move to next white space, ", or eol */
     if (t2 > t1)
         strncpy(output, t1, t2 - t1);
