@@ -47,15 +47,111 @@ static const char* const version="$VER: pgmabel 1.009 (24 Jan 2002)";
 #define TRUE 1
 #define FALSE 0
 
-/* Prototype Declaration */
-static void   abel ARGS(( float* , int, double* ));
-static double Sum ARGS(( int , double* , int , double*));
-static double dr ARGS(( int, double, int ));
-static void   help ARGS(( void  ));
-
 /* some global variables */
 static double *aldl, *ardl;                /* pointer for weighting factors */
 
+/* ----------------------------------------------------------------------------
+** procedure for calculating the sum of the calculated surfaces with the
+** weight of the surface
+**      n     <-  index of end point of the summation
+**      N     <-  width of the calculated row
+**      xr    <-  array of the calculated elements of the row
+**      adl   <-  pre-calculated surface coefficient for each segment
+*/
+static double 
+Sum ( int n, double *xr, int N, double *adl)
+{
+    int k;
+    double result=0.0;
+
+    if (n==0) return(0.0);             /* outer ring  is 0 per definition    */
+    for (k=0 ; k<=(n-1) ; k++)
+    {
+         result += xr[k] * ( adl[k*N+n] - adl[(k+1)*N+n]);
+/*       result += xr[k] * ( dr(k,n+0.5,N) - dr(k+1,n+0.5,N));   */
+    }
+    return(result);
+}
+
+/* ----------------------------------------------------------------------------
+** procedure for calculating the surface coefficient for the Integration
+**      R, N  <-  indizes of the coefficient
+**      r     <-  radial position of the center of the surface
+*/
+static double 
+dr ( int R, double r,  int N)
+{
+    double a;
+    double b;
+    a=(double) N-R ;
+    b=(double) N-r ;
+    return(sqrt(a*a-b*b));
+}
+
+/* ----------------------------------------------------------------------------
+** procedure for making the Abel integration for deconvolution of the image
+**        y    <-> array with values for deconvolution and results
+**        N    <-  width of the array
+**        adl  <-  array with pre-calculated weighting factors
+*/
+static void 
+abel ( float *y, int N, double *adl)
+{
+    register int n;
+    double *rho, *rhop;       /* results and new index                       */
+    float  *yp;               /* new indizes for the y-array                 */
+
+    rho = (double *) malloc( N*sizeof(double)); /* memory for  temporary     */
+    if( !rho )
+        pm_error( "out of memory" );
+    rhop = rho;
+    yp  = y;
+
+    for (n=0 ; n<N ; n++)
+    {
+        *(rhop++) = ((*yp++) - Sum(n,rho,N,adl))/(adl[n*N+n]);
+/*    *(rhop++) = ((*yp++) - Sum(n,rho,N))/(dr(n,n+0.5,N));  old version */
+        if ( *rhop < 0.0 ) *rhop = 0.0;         /*  error correction !       */
+/*   if (n > 2) rhop[n-1] = (rho[n-2]+rho[n-1]+rho[n])/3.0;  stabilization*/
+    }
+    for (n=0 ; n<N ; n++)
+        {
+            if (( n>=1 )&&( n<N-1 ))
+	       (*y++) = ((rho[n-1]*0.5+rho[n]+rho[n+1]*0.5)/2.0);/*1D median filter*/
+            else (*y++) = rho[n];
+        }
+    free(rho);
+}
+
+/* ----------------------------------------------------------------------------
+** printing a help message if Option -h(elp) is chosen
+*/
+static void 
+help()
+{
+    pm_message("-----------------------------------------------------------------");
+    pm_message("| pgmabel                                                       |");
+    pm_message("| make a deconvolution with vertical axis as symmetry-line      |");
+    pm_message("| usage:                                                        |");
+    pm_message("| pgmabel [-help] [-axis N] [-factor N] [-left|-right]          |");
+    pm_message("|         [-pixsize] [-verbose] [pgmfile]                       |");
+    pm_message("|   axis    : horizontal position of the axis                   |");
+    pm_message("|   factor  : user defines stretch-factor for the gray levels   |");
+    pm_message("|   pixsize : size of one pixel in mm (default = 0.1)           |");
+    pm_message("|   left    : calculating only the left (or right) side         |");
+    pm_message("|   verbose : output of useful data                             |");
+    pm_message("|   pgmfile : Name of a pgmfile (optional)                      |");
+    pm_message("|                                                               |");
+    pm_message("| for further information please contact the manpage            |"); 
+    pm_message("-----------------------------------------------------------------");
+    pm_message("%s",version);     /* telling the version      */
+    exit(-1);                     /* retur-code for no result */
+}
+
+
+
+
+
 /* ----------------------------------------------------------------------------
 ** main program
 */
@@ -217,96 +313,3 @@ int main( argc, argv )
     exit( 0 );                       /* end of procedure                     */
 }
 
-/* ----------------------------------------------------------------------------
-** procedure for making the Abel integration for deconvolution of the image
-**        y    <-> array with values for deconvolution and results
-**        N    <-  width of the array
-**        adl  <-  array with pre-calculated weighting factors
-*/
-static void abel ( float *y, int N, double *adl)
-{
-    register int n;
-    double *rho, *rhop;       /* results and new index                       */
-    float  *yp;               /* new indizes for the y-array                 */
-
-    rho = (double *) malloc( N*sizeof(double)); /* memory for  temporary     */
-    if( !rho )
-        pm_error( "out of memory" );
-    rhop = rho;
-    yp  = y;
-
-    for (n=0 ; n<N ; n++)
-    {
-        *(rhop++) = ((*yp++) - Sum(n,rho,N,adl))/(adl[n*N+n]);
-/*    *(rhop++) = ((*yp++) - Sum(n,rho,N))/(dr(n,n+0.5,N));  old version */
-        if ( *rhop < 0.0 ) *rhop = 0.0;         /*  error correction !       */
-/*   if (n > 2) rhop[n-1] = (rho[n-2]+rho[n-1]+rho[n])/3.0;  stabilization*/
-    }
-    for (n=0 ; n<N ; n++)
-        {
-            if (( n>=1 )&&( n<N-1 ))
-	       (*y++) = ((rho[n-1]*0.5+rho[n]+rho[n+1]*0.5)/2.0);/*1D median filter*/
-            else (*y++) = rho[n];
-        }
-    free(rho);
-}
-
-/* ----------------------------------------------------------------------------
-** procedure for calculating the sum of the calculated surfaces with the
-** weight of the surface
-**      n     <-  index of end point of the summation
-**      N     <-  width of the calculated row
-**      xr    <-  array of the calculated elements of the row
-**      adl   <-  pre-calculated surface coefficient for each segment
-*/
-static double Sum ( int n, double *xr, int N, double *adl)
-{
-    int k;
-    double result=0.0;
-
-    if (n==0) return(0.0);             /* outer ring  is 0 per definition    */
-    for (k=0 ; k<=(n-1) ; k++)
-    {
-         result += xr[k] * ( adl[k*N+n] - adl[(k+1)*N+n]);
-/*       result += xr[k] * ( dr(k,n+0.5,N) - dr(k+1,n+0.5,N));   */
-    }
-    return(result);
-}
-
-/* ----------------------------------------------------------------------------
-** procedure for calculating the surface coefficient for the Integration
-**      R, N  <-  indizes of the coefficient
-**      r     <-  radial position of the center of the surface
-*/
-static double dr ( int R, double r,  int N)
-{
-    double a;
-    double b;
-    a=(double) N-R ;
-    b=(double) N-r ;
-    return(sqrt(a*a-b*b));
-}
-
-/* ----------------------------------------------------------------------------
-** printing a help message if Option -h(elp) is chosen
-*/
-static void help()
-{
-    pm_message("-----------------------------------------------------------------");
-    pm_message("| pgmabel                                                       |");
-    pm_message("| make a deconvolution with vertical axis as symmetry-line      |");
-    pm_message("| usage:                                                        |");
-    pm_message("| pgmabel [-help] [-axis N] [-factor N] [-left|-right]          |");
-    pm_message("|         [-pixsize] [-verbose] [pgmfile]                       |");
-    pm_message("|   axis    : horizontal position of the axis                   |");
-    pm_message("|   factor  : user defines stretch-factor for the gray levels   |");
-    pm_message("|   pixsize : size of one pixel in mm (default = 0.1)           |");
-    pm_message("|   left    : calculating only the left (or right) side         |");
-    pm_message("|   verbose : output of useful data                             |");
-    pm_message("|   pgmfile : Name of a pgmfile (optional)                      |");
-    pm_message("|                                                               |");
-    pm_message("| for further information please contact the manpage            |"); 
-    pm_message("-----------------------------------------------------------------");
-    pm_message("%s",version);     /* telling the version      */
-    exit(-1);                     /* retur-code for no result */
-}
