@@ -100,6 +100,7 @@ parseCommandLine(int argc, char ** argv,
 }
 
 
+
 static void
 buildGamma(xelval table[], xelval const maxval, double const gamma) {
 /*----------------------------------------------------------------------------
@@ -113,14 +114,14 @@ buildGamma(xelval table[], xelval const maxval, double const gamma) {
    identities.
 -----------------------------------------------------------------------------*/
     xelval i;
-    double const one_over_gamma = 1.0 / gamma;
-    double const q = (double) maxval;
+    double const oneOverGamma = 1.0 / gamma;
 
     for (i = 0 ; i <= maxval; ++i) {
-        double const ind = ((double) i) / q;
+        double const normalized = ((double) i) / maxval;
             /* Xel sample value normalized to 0..1 */
-        double const v = (q * pow(ind, one_over_gamma));
-        table[i] = MIN((xelval)(v + 0.5), maxval);  /* round and clip */
+        double const v = pow(normalized, oneOverGamma);
+        table[i] = MIN((xelval)(v * maxval + 0.5), maxval);  
+            /* denormalize, round and clip */
     }
 }
 
@@ -169,6 +170,7 @@ buildCie709GammaInverse(xelval table[], xelval const maxval,
 
    'gamma' must be 1/0.45 for true Rec. 709.
 -----------------------------------------------------------------------------*/
+    double const oneOverGamma = 1.0 / gamma;
     xelval i;
 
     /* This transfer function is linear for sample values 0
@@ -178,7 +180,7 @@ buildCie709GammaInverse(xelval table[], xelval const maxval,
     */
     xelval const linearCutoff = (xelval) maxval * 0.018 + 0.5;
     double const linearExpansion = 
-        (1.099 * pow(0.018, gamma) - 0.099) / 0.018;
+        (1.099 * pow(0.018, oneOverGamma) - 0.099) / 0.018;
 
     for (i = 0; i <= linearCutoff * linearExpansion; ++i) 
         table[i] = i / linearExpansion;
