@@ -261,20 +261,27 @@ computeEndValues(FILE *   const ifp,
    'ifp', which is positioned just past the header (at the raster).
    Leave it positioned arbitrarily.
 -----------------------------------------------------------------------------*/
-    unsigned int hist[PNM_OVERALLMAXVAL+1];
+    unsigned int * hist;  /* malloc'ed */
 
-    buildHistogram(ifp, cols, rows, maxval, format, hist, BRIGHT_LUMIN);
-    
-    if (cmdline.bvalueSpec)
-        *bvalueP = cmdline.bvalue;
-    else 
-        computeBottomPercentile(hist, maxval, cols*rows, cmdline.bpercent, 
+    hist = malloc((PNM_OVERALLMAXVAL+1)*sizeof(hist[0]));
+
+    if (hist == NULL)
+        pm_error("Unable to allocate storage for intensity histogram.");
+    else {
+        buildHistogram(ifp, cols, rows, maxval, format, hist, BRIGHT_LUMIN);
+        
+        if (cmdline.bvalueSpec)
+            *bvalueP = cmdline.bvalue;
+        else 
+            computeBottomPercentile(hist, maxval, cols*rows, cmdline.bpercent, 
                                 bvalueP);
-    if (cmdline.wvalueSpec) 
-        *wvalueP = cmdline.wvalue;
-    else
-        computeTopPercentile(hist, maxval, cols*rows, cmdline.wpercent, 
-                             wvalueP);
+        if (cmdline.wvalueSpec) 
+            *wvalueP = cmdline.wvalue;
+        else
+            computeTopPercentile(hist, maxval, cols*rows, cmdline.wpercent, 
+                                 wvalueP);
+        free(hist);
+    }
 }
 
 
@@ -366,7 +373,7 @@ main(int argc, char *argv[]) {
 
     struct cmdlineInfo cmdline;
     FILE *ifp;
-    int imagePos;
+    pm_filepos imagePos;
     xelval maxval;
     int rows, cols, format;
     xelval bvalue, wvalue;
@@ -379,7 +386,7 @@ main(int argc, char *argv[]) {
 
     /* Rescale so that bvalue maps to 0, wvalue maps to maxval. */
     pnm_readpnminit(ifp, &cols, &rows, &maxval, &format);
-    imagePos = pm_tell(ifp);
+    pm_tell2(ifp, &imagePos, sizeof(imagePos));
 
     computeEndValues(ifp, cols, rows, maxval, format, cmdline, 
                      &bvalue, &wvalue);
@@ -399,7 +406,7 @@ main(int argc, char *argv[]) {
 
         computeTransferFunction(bvalue, wvalue, maxval, newIntensity);
 
-        pm_seek(ifp, imagePos);
+        pm_seek2(ifp, &imagePos, sizeof(imagePos));
         pnm_writepnminit(stdout, cols, rows, maxval, format, 0);
 
         rowbuf = pnm_allocrow(cols);
