@@ -358,7 +358,10 @@ searchColormapCloseSimple(struct pam * const pamP,
                           int          const colors,
                           tuple        const tuple,
                           int *        const colormapIndexP) {
-
+/*----------------------------------------------------------------------------
+   Same as searchColorMap(), except works only where the maxval of the
+   color map is the same as that of the search tuple and less than 256.
+-----------------------------------------------------------------------------*/
     unsigned int i;
     unsigned long dist;
     dist = ULONG_MAX;
@@ -385,12 +388,14 @@ searchColormapCloseComplex(struct pam * const pamP,
                            sample       const colormapMaxval,
                            tuple        const tuple,
                            int *        const colormapIndexP) {
-
+/*----------------------------------------------------------------------------
+   Same as searchColorMap().  Not as fast as searchColorMapSimple().
+-----------------------------------------------------------------------------*/
     unsigned int i;
     unsigned int const divider = colormapMaxval < 256 ? 1 : 2;
-    /* We divide our sum by this to make sure it fits in a "long" */
+        /* We divide our sum by this to make sure it fits in a "long" */
     unsigned long dist;
-    /* The closest distance we've found so far between the value of
+        /* The closest distance we've found so far between the value of
            tuple 'tuple' and a tuple in the colormap.  This is measured as
            the square of the cartesian distance between the tuples, except
            that it's divided by 'divider' (above) to make sure it will fit
@@ -434,6 +439,22 @@ searchColormapClose(struct pam * const pamP,
    Search colormap 'colormap', which contains 'colors' colors (at
    least 1) for the color closest to that of tuple 'tuple'.  Return
    its index as *colormapIndexP.
+
+   We compute distance between colors simply as the cartesian distance
+   between them in the RGB space.  An alternative would be to look at
+   the chromaticities and luminosities of the colors.  In experiments
+   in 2003, we found that this was actually worse in many cases.  One
+   might think that two colors are closer if they have similar hues
+   than when they are simply different brightnesses of the same hue.
+   Human subjects asked to compare two colors normally say so.  But
+   when replacing the color of a pixel in an image, the luminosity is
+   much more important, because you need to retain the luminosity
+   relationship between adjacent pixels.  If you replace a pixel with
+   one that has the same chromaticity as the original, but much
+   darker, it may stand out among its neighbors in a way the original
+   pixel did not.  In fact, on an image with blurred edges, we say
+   ugly effects at the edges when we substituted colors using a
+   chromaticity-first color closeness formula.
 -----------------------------------------------------------------------------*/
     if (pamP->maxval == colormapMaxval && colormapMaxval < 256) 
         /* Do a fast computation for this usual case */
@@ -508,7 +529,7 @@ lookupThroughHash(struct pam * const pamP,
    unless *usehashP is false, and if that makes 'colorhash' full, we set
    *usehashP false.
 -----------------------------------------------------------------------------*/
-    bool found;
+    int found;
 
     /* Check hash table to see if we have already matched this color. */
     pnm_lookuptuple(pamP, colorhash, tuple, &found, colormapIndexP);
@@ -524,15 +545,15 @@ lookupThroughHash(struct pam * const pamP,
         } else 
             searchColormapClose(pamP, colormap, colormapSize, 
                                 colormapMaxval, tuple, colormapIndexP);
-    }        
-    if (*usehashP) {
-        bool fits;
-        pnm_addtotuplehash(pamP, colorhash, tuple, *colormapIndexP, 
-                           &fits);
-        if (!fits) {
-            pm_message("out of memory adding to hash table; "
-                       "proceeding without it");
-            *usehashP = FALSE;
+        if (*usehashP) {
+            bool fits;
+            pnm_addtotuplehash(pamP, colorhash, tuple, *colormapIndexP, 
+                               &fits);
+            if (!fits) {
+                pm_message("out of memory adding to hash table; "
+                           "proceeding without it");
+                *usehashP = FALSE;
+            }
         }
     }
 }
