@@ -20,13 +20,15 @@ struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *text;    /* text from command line or NULL if none */
-    char *font;    /* -font option value or NULL if none */
-    char *builtin; /* -builtin option value or NULL if none */
-    int dump;   /* undocumented dump flag for installing a new built-in font */
+    const char *text;    /* text from command line or NULL if none */
+    const char *font;    /* -font option value or NULL if none */
+    const char *builtin; /* -builtin option value or NULL if none */
+    unsigned int dump;   
+        /* undocumented dump option for installing a new built-in font */
     float space;   /* -space option value or default */
     unsigned int width;     /* future -width option value or zero */
     int lspace;    /* lspace option value or default */
+    unsigned int nomargins;     /* -nomargins */
 };
 
 
@@ -39,25 +41,25 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
+    optEntry *option_def = malloc(100*sizeof(optEntry));
+        /* Instructions to OptParseOptions3 on how to parse our options.
          */
-    optStruct2 opt;
+    optStruct3 opt;
 
     unsigned int option_def_index;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "font",     OPT_STRING,   &cmdline_p->font,         0);
-    OPTENTRY(0,   "builtin",  OPT_STRING,   &cmdline_p->builtin,      0);
-    OPTENTRY(0,   "dump",     OPT_UINT,     &cmdline_p->dump,         0);
-    OPTENTRY(0,   "space",    OPT_FLOAT,    &cmdline_p->space,        0);
-    OPTENTRY(0,   "width",    OPT_UINT,     &cmdline_p->width,        0);
-    OPTENTRY(0,   "lspace",   OPT_INT,      &cmdline_p->lspace,       0);
+    OPTENT3(0, "font",      OPT_STRING, &cmdline_p->font, NULL,        0);
+    OPTENT3(0, "builtin",   OPT_STRING, &cmdline_p->builtin, NULL,     0);
+    OPTENT3(0, "dump",      OPT_FLAG,   NULL, &cmdline_p->dump,        0);
+    OPTENT3(0, "space",     OPT_FLOAT,  &cmdline_p->space, NULL,       0);
+    OPTENT3(0, "width",     OPT_UINT,   &cmdline_p->width, NULL,       0);
+    OPTENT3(0, "lspace",    OPT_INT,    &cmdline_p->lspace, NULL,      0);
+    OPTENT3(0, "nomargins", OPT_FLAG,   NULL, &cmdline_p->nomargins,   0);
 
     /* Set the defaults */
     cmdline_p->font = NULL;
     cmdline_p->builtin = NULL;
-    cmdline_p->dump = FALSE;
     cmdline_p->space = 0.0;
     cmdline_p->width = 0;
     cmdline_p->lspace = 0;
@@ -66,34 +68,36 @@ parse_command_line(int argc, char ** argv,
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    optParseOptions2(&argc, argv, opt, 0);
+    optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
     if (argc-1 == 0)
         cmdline_p->text = NULL;
     else {
+        char *text;
         int i;
         int totaltextsize;
 
         totaltextsize = 1;  /* initial value */
         
-        cmdline_p->text = malloc(totaltextsize);
-        cmdline_p->text[0] = '\0';
+        text = malloc(totaltextsize);  /* initial allocation */
+        text[0] = '\0';
         
         for (i = 1; i < argc; i++) {
             if (i > 1) {
                 totaltextsize += 1;
-                cmdline_p->text = realloc(cmdline_p->text, totaltextsize);
-                if (cmdline_p->text == NULL)
-                    pm_error("out of memory");
-                strcat(cmdline_p->text, " ");
+                text = realloc(text, totaltextsize);
+                if (text == NULL)
+                    pm_error("out of memory allocating space for input text");
+                strcat(text, " ");
             } 
             totaltextsize += strlen(argv[i]);
-            cmdline_p->text = realloc(cmdline_p->text, totaltextsize);
-            if (cmdline_p->text == NULL)
-                pm_error("out of memory");
-            strcat(cmdline_p->text, argv[i]);
+            text = realloc(text, totaltextsize);
+            if (text == NULL)
+                pm_error("out of memory allocating space for input text");
+            strcat(text, argv[i]);
         }
+        cmdline_p->text = text;
     }
 }
 
@@ -119,11 +123,11 @@ fix_control_chars(char * const buf, struct font * const fn) {
             for ( ; i < n; ++i )
                 buf[i] = ' ';
             --i;
-	    }
+        }
         else if ( !fn->glyph[(unsigned char)buf[i]] )
             /* Turn unknown chars into a single space. */
             buf[i] = ' ';
-	}
+    }
 }
 
 
@@ -408,8 +412,8 @@ get_text(const char cmdline_text[], struct font * const fn,
                 pm_error( "out of memory" );
             strcpy(input_text[*linesP], buf);
             ++(*linesP);
-	    }
-	}
+        }
+    }
     *input_textP = (const char **) input_text;
 }
 
@@ -457,7 +461,7 @@ main(int argc, char *argv[]) {
     parse_command_line(argc, argv, &cmdline);
     
     if (cmdline.font)
-	    fn = pbm_loadfont(cmdline.font);
+        fn = pbm_loadfont(cmdline.font);
     else {
         if (cmdline.builtin)
             fn = pbm_defaultfont(cmdline.builtin);
@@ -468,17 +472,22 @@ main(int argc, char *argv[]) {
     if (cmdline.dump) {
         pbm_dumpfont(fn);
         exit(0);
-	}
+    }
 
     get_text(cmdline.text, fn, &input_text, &lines);
     
-    if (lines == 1) {
-        vmargin = fn->maxheight / 2;
-        hmargin = fn->maxwidth;
-	} else {
-        vmargin = fn->maxheight;
-        hmargin = 2 * fn->maxwidth;
-	}
+    if (cmdline.nomargins) {
+        vmargin = 0;
+        hmargin = 0;
+    } else {
+        if (lines == 1) {
+            vmargin = fn->maxheight / 2;
+            hmargin = fn->maxwidth;
+        } else {
+            vmargin = fn->maxheight;
+            hmargin = 2 * fn->maxwidth;
+        }
+    }
 
     if (cmdline.width > 0) {
         /* Flow or truncate lines to meet user's width request */
@@ -512,6 +521,3 @@ main(int argc, char *argv[]) {
 
     exit(0);
 }
-
-
-
