@@ -25,7 +25,7 @@
 #define FILENAME_LEN 200
 static FILE *psfile = 0;
 char psfname[FILENAME_LEN];
-static int pbmfile = 0;
+static FILE *pbmfile = 0;
 char pbmfname[FILENAME_LEN];
 
 #define BUFFER_SIZE 2048
@@ -66,7 +66,7 @@ parseCommandLine(int argc, char ** argv,
     Note that the file spec array we return is stored in the storage that
     was passed to us as the argv array.
     -------------------------------------------------------------------------*/
-  optEntry *option_def = malloc(100*sizeof(optStruct));
+  optEntry *option_def = malloc(100*sizeof(optEntry));
   /* Instructions to OptParseOptions2 on how to parse our options.
    */
   optStruct3 opt;
@@ -242,6 +242,7 @@ cleanup(void)
     unlink(psfname);
   }
   if(pbmfile) {
+    pm_close(pbmfile);
     unlink(pbmfname);
   }
 }
@@ -262,7 +263,6 @@ create_outputfile(struct cmdlineInfo *cmdl)
 
   sprintf(pbmfname, template, getpid(), "pbm");
 
-  pbmfile = 1;
   if(! (com = strdup(gs_command(cmdl))))
     pm_error("Can't allocate ghostscript command");
 
@@ -283,6 +283,8 @@ create_outputfile(struct cmdlineInfo *cmdl)
 
   free(com);
 
+  pbmfile = pm_openw(pbmfname);
+
   com = crop_command();
   if(com) {
     if (cmdl->verbose)
