@@ -322,7 +322,9 @@ static void spectralsynth(x, n, h)
 
 static void initseed()
 {
-    int i = time((long *) 0) ^ 0xF37C;
+    int i;
+
+    i = time(NULL) ^ 0xF37C;
     V srand(i);
     for (i = 0; i < 7; i++)
         V rand();
@@ -908,7 +910,7 @@ int main(argc, argv)
                 pm_error("already specified a random seed");
             }
             i++;
-            if ((i == argc) || (sscanf(argv[i], "%d", &rseed) != 1))
+            if ((i == argc) || (sscanf(argv[i], "%u", &rseed) != 1))
                 pm_usage(usage);
             seedspec = TRUE;
         } else if (pm_keymatch(argv[i], "-xsize", 2) ||
