@@ -6,12 +6,16 @@
 -----------------------------------------------------------------------------*/
 
 #include <string.h>
+#include <limits.h>
+
+#include "nstring.h"
 #include "pam.h"
 #include "ppm.h"
 #include "libpbm.h"
 #include "libpgm.h"
 #include "libppm.h"
 #include "colorname.h"
+#include "fileio.h"
 
 
 char
@@ -31,9 +35,9 @@ stripeq(const char * const comparand, const char * const comparator) {
      */
 
   p = (char *) comparand;
-  while (isspace(*p)) p++;
+  while (ISSPACE(*p)) p++;
   q = (char *) comparator;
-  while (isspace(*q)) q++;
+  while (ISSPACE(*q)) q++;
 
   /* Make px and qx point to the last non-blank character in each string.
      If there are no nonblank characters (which implies the string is
@@ -43,13 +47,13 @@ stripeq(const char * const comparand, const char * const comparator) {
   if (*p == '\0') px = p;
   else {
     px = p + strlen(p) - 1;
-    while (isspace(*px)) px--;
+    while (ISSPACE(*px)) px--;
   }
 
   if (*q == '\0') qx = q;
   else {
     qx = q + strlen(q) - 1;
-    while (isspace(*qx)) qx--;
+    while (ISSPACE(*qx)) qx--;
   }
 
   equal = TRUE;   /* initial assumption */
@@ -69,6 +73,42 @@ stripeq(const char * const comparand, const char * const comparator) {
 
 
 
+static void
+validateComputableSize(struct pam * const pamP) {
+/*----------------------------------------------------------------------------
+   Validate that the dimensions of the image are such that it can be
+   processed in typical ways on this machine without worrying about
+   overflows.  Note that in C, arithmetic is always modulus arithmetic,
+   so if your values are too big, the result is not what you expect.
+   That failed expectation can be disastrous if you use it to allocate
+   memory.
+
+   It is very normal to allocate space for a tuplerow, so we make sure
+   the size of a tuple row, in bytes, can be represented by an 'int'.
+
+   Another common operation is adding 1 or 2 to the highest row, column,
+   or plane number in the image, so we make sure that's possible.
+-----------------------------------------------------------------------------*/
+    if (pamP->depth > INT_MAX/sizeof(sample))
+        pm_error("image depth (%u) too large to be processed", pamP->depth);
+    else if (pamP->depth * sizeof(sample) > INT_MAX/pamP->width)
+        pm_error("image width and depth (%u, %u) too large "
+                 "to be processed.", pamP->width, pamP->depth);
+    else if (pamP->width * (pamP->depth * sizeof(sample)) >
+             INT_MAX - pamP->depth * sizeof(tuple *))
+        pm_error("image width and depth (%u, %u) too large "
+                 "to be processed.", pamP->width, pamP->depth);
+
+    if (pamP->depth > INT_MAX - 2)
+        pm_error("image depth (%u) too large to be processed", pamP->depth);
+    if (pamP->width > INT_MAX - 2)
+        pm_error("image width (%u) too large to be processed", pamP->width);
+    if (pamP->height > INT_MAX - 2)
+        pm_error("image height (%u) too large to be processed", pamP->height);
+}
+
+
+
 int
 pnm_tupleequal(const struct pam * const pamP, 
                tuple              const comparand, 
@@ -158,13 +198,18 @@ createBlackTuple(const struct pam * const pamP, tuple * const blackTupleP) {
 
 
 tuple *
-pnm_allocpamrow(struct pam * const pamP) {
+pnm_allocpamrow(const struct pam * const pamP) {
+/*----------------------------------------------------------------------------
+   We assume that the dimensions of the image are such that arithmetic
+   overflow will not occur in our calculations.  NOTE: pnm_readpaminit()
+   ensures this assumption is valid.
+-----------------------------------------------------------------------------*/
     const int bytes_per_tuple = pamP->depth * sizeof(sample);
     tuple * tuplerow;
 
     /* The tuple row data structure starts with 'width' pointers to
-        the tuples, immediately followed by the 'width' tuples
-        themselves.  Each tuple consists of 'depth' samples.  
+       the tuples, immediately followed by the 'width' tuples
+       themselves.  Each tuple consists of 'depth' samples.  
     */
 
     tuplerow = malloc(pamP->width * (sizeof(tuple *) + bytes_per_tuple));
@@ -193,7 +238,7 @@ pnm_allocpamrow(struct pam * const pamP) {
 
 
 tuple **
-pnm_allocpamarray(struct pam * const pamP) {
+pnm_allocpamarray(const struct pam * const pamP) {
     
     tuple **tuplearray;
     int row;
@@ -216,7 +261,7 @@ pnm_allocpamarray(struct pam * const pamP) {
 
 
 void
-pnm_freepamarray(tuple ** const tuplearray, struct pam * const pamP) {
+pnm_freepamarray(tuple ** const tuplearray, const struct pam * const pamP) {
 
     int row;
     for (row = 0; row < pamP->height; row++)
@@ -253,13 +298,13 @@ parse_header_line(const char buffer[], char label[MAX_LABEL_LENGTH+1],
 
     buffer_curs = 0;
     /* Skip initial white space */
-    while (isspace(buffer[buffer_curs])) buffer_curs++;
+    while (ISSPACE(buffer[buffer_curs])) buffer_curs++;
 
     {
         /* Read off label, put as much as will fit into label[] */
         int label_curs;
         label_curs = 0;
-        while (!isspace(buffer[buffer_curs]) && buffer[buffer_curs] != '\0') {
+        while (!ISSPACE(buffer[buffer_curs]) && buffer[buffer_curs] != '\0') {
             if (label_curs < MAX_LABEL_LENGTH) 
                 label[label_curs++] = buffer[buffer_curs];
             buffer_curs++;
@@ -268,7 +313,7 @@ parse_header_line(const char buffer[], char label[MAX_LABEL_LENGTH+1],
     }    
 
     /* Skip white space between label and value */
-    while (isspace(buffer[buffer_curs])) buffer_curs++;
+    while (ISSPACE(buffer[buffer_curs])) buffer_curs++;
 
     /* copy value into value[] */
     strncpy(value, buffer+buffer_curs, MAX_VALUE_LENGTH+1);
@@ -277,7 +322,7 @@ parse_header_line(const char buffer[], char label[MAX_LABEL_LENGTH+1],
         /* Remove trailing white space from value[] */
         int value_curs;
         value_curs = strlen(value)-1;
-        while (value_curs >= 0 && isspace(value[value_curs])) 
+        while (value_curs >= 0 && ISSPACE(value[value_curs])) 
             value[value_curs--] = '\0';
     }
 }
@@ -314,8 +359,12 @@ process_header_line(const char buffer[], struct pam * const pamP,
                          label);
             else {
                 char *endptr;
-                long numeric_value;
+                long int numeric_value;
+                errno = 0;  /* Clear errno so we can detect strtol() failure */
                 numeric_value = strtol(value, &endptr, 10);
+                if (errno != 0)
+                    pm_error("Too-large value for %s in "
+                             "PAM file header: '%s'", label, value);
                 if (*endptr != '\0') 
                     pm_error("Non-numeric value for %s in "
                              "PAM file header: '%s'", label, value);
@@ -371,7 +420,7 @@ pnm_readpaminitrest(struct pam * const pamP) {
     { int c;
     /* Read off rest of 1st line -- probably just the newline after the 
        magic number 
-       */
+    */
     while ((c = getc(pamP->file)) != -1 && c != '\n');
     }    
 
@@ -461,6 +510,8 @@ pnm_readpaminit(FILE *file, struct pam * const pamP, const int size) {
     else if (pamP->maxval >> 16 == 0) pamP->bytes_per_sample = 2;
     else if (pamP->maxval >> 24 == 0) pamP->bytes_per_sample = 3;
     else                              pamP->bytes_per_sample = 4;
+
+    validateComputableSize(pamP);
 }
 
 
@@ -501,7 +552,7 @@ get_sample(FILE *file, const int bytes) {
 
 
 void 
-pnm_readpamrow(struct pam * const pamP, tuple* const tuplerow) {
+pnm_readpamrow(const struct pam * const pamP, tuple* const tuplerow) {
 
     /* Need a special case for raw PBM because it has multiple tuples (8)
        packed into one byte.
@@ -524,7 +575,7 @@ pnm_readpamrow(struct pam * const pamP, tuple* const tuplerow) {
             int samp;
             for (samp = 0; samp < pamP->depth; samp++) {
                 if (pamP->plainformat)
-                    tuplerow[col][samp] = pbm_getint(pamP->file);
+                    tuplerow[col][samp] = pm_getuint(pamP->file);
                 else 
                     tuplerow[col][samp] = 
                         get_sample(pamP->file, pamP->bytes_per_sample);
@@ -676,7 +727,7 @@ write_sample(FILE *file, const unsigned int value, const int bytes) {
 
 
 void 
-pnm_writepamrow(struct pam *const pamP, const tuple * const tuplerow) {
+pnm_writepamrow(const struct pam *const pamP, const tuple * const tuplerow) {
 
     /* For speed, we don't check any of the inputs for consistency 
        here (unless it's necessary to avoid crashing).  Any consistency
@@ -717,18 +768,17 @@ pnm_writepamrow(struct pam *const pamP, const tuple * const tuplerow) {
 
 
 void
-pnm_checkpam(struct pam *const pamP, const enum pm_check_type check_type, 
+pnm_checkpam(const struct pam *const pamP, const enum pm_check_type check_type, 
              enum pm_check_code * const retval_p) {
 
     if (check_type != PM_CHECK_BASIC) {
         if (retval_p) *retval_p = PM_CHECK_UNKNOWN_TYPE;
     } else switch (PAM_FORMAT_TYPE(pamP->format)) {
-    case PAM_TYPE: 
-        pm_check(pamP->file, 
-                 check_type, 
-                 pamP->width * pamP->height * pamP->depth * 
-                 pamP->bytes_per_sample,
-                 retval_p);
+    case PAM_TYPE: {
+        pm_filepos const need_raster_size = 
+            pamP->width * pamP->height * pamP->depth * pamP->bytes_per_sample;
+        pm_check(pamP->file, check_type, need_raster_size, retval_p);
+    }
         break;
     case PPM_TYPE:
         pgm_check(pamP->file, check_type, pamP->format, 
@@ -767,3 +817,80 @@ pnm_YCbCrtuple(const tuple tuple,
     *CbP = (- 0.16874 * red - 0.33126 * grn + 0.50000 * blu);
     *CrP = (+ 0.50000 * red - 0.41869 * grn - 0.08131 * blu);
 }
+
+
+
+void 
+pnm_YCbCr_to_rgbtuple(const struct pam * const pamP,
+                      tuple              const tuple,
+                      double             const Y,
+                      double             const Cb, 
+                      double             const Cr,
+                      int *              const overflowP) {
+
+    double rgb[3];
+    unsigned int plane;
+    bool overflow;
+
+    rgb[PAM_RED_PLANE] = Y + 1.4022 * Cr + .5;
+    rgb[PAM_GRN_PLANE] = Y - 0.7145 * Cr - 0.3456 * Cb + .5;
+    rgb[PAM_BLU_PLANE] = Y + 1.7710 * Cb + .5;
+
+    overflow = FALSE;  /* initial assumption */
+
+    for (plane = 0; plane < 3; ++plane) {
+        if (rgb[plane] > pamP->maxval) {
+            overflow = TRUE;
+            tuple[plane] = pamP->maxval;
+        } else if (rgb[plane] < 0) {
+            overflow = TRUE;
+            tuple[plane] = 0;
+        } else 
+            tuple[plane] = rgb[plane];
+    }
+    if (overflowP)
+        *overflowP = overflow;
+}
+
+
+/*=============================================================================
+   pm_system() Standard Input feeder and Standard Output accepter functions.   
+=============================================================================*/
+
+void
+pm_feed_from_pamtuples(int    const pipeToFeedFd,
+                       void * const feederParm) {
+
+    struct pamtuples * const inputTuplesP = feederParm;
+
+    struct pam outpam;
+
+    outpam = *inputTuplesP->pamP;
+    outpam.file = fdopen(pipeToFeedFd, "w");
+
+    /* The following signals (and normally kills) the process with
+       SIGPIPE if the pipe does not take all 'inputLength' bytes.
+    */
+    pnm_writepam(&outpam, *inputTuplesP->tuplesP);
+
+    pm_close(outpam.file);
+}
+
+
+
+void
+pm_accept_to_pamtuples(int    const pipeToSuckFd,
+                       void * const accepterParm ) {
+
+    struct pamtuples * const outputTuplesP = accepterParm;
+
+    struct pam inpam;
+
+    *outputTuplesP->tuplesP =
+        pnm_readpam(fdopen(pipeToSuckFd, "r"), &inpam, sizeof(inpam));
+
+    pm_close(inpam.file);
+}
+
+
+
