@@ -15,6 +15,8 @@
 */
 
 #include <string.h>
+
+#include "nstring.h"
 #include "pbm.h"
 #include "pbmfont.h"
 
@@ -807,8 +809,7 @@ gotblankcol:
 
 
 struct font*
-pbm_loadfont( filename )
-char* filename;
+pbm_loadfont(const char * const filename)
 {
     FILE* fp;
     struct font* fn;
@@ -833,8 +834,7 @@ char* filename;
 
 
 
-struct font* pbm_loadpbmfont( filename )
-char* filename;
+struct font* pbm_loadpbmfont(const char * const filename)
 {
     FILE* ifp;
     bit** font;
@@ -942,8 +942,7 @@ static int readline ARGS((FILE* fp, char* buf, char* arg[]));
 #define expect(str) if (readline(fp, line, arg) < 0 || strcmp(arg[0], (str))) \
 	{ fclose(fp); return 0; }
 
-struct font* pbm_loadbdffont(name)
-char* name;
+struct font* pbm_loadbdffont(const char * const name)
 {
 	FILE* fp;
 	char line[1024], *arg[32], *hex;
@@ -1086,14 +1085,14 @@ int mk_max;
 
 	n = 0;
 	while (*s) {
-		if (isspace(*s)) {
+		if (ISSPACE(*s)) {
 			*s++ = '\0';
 			continue;
 		}
 		vec[n++] = s;
 		if (n >= mk_max)
 			break;
-		while (*s && !isspace(*s))
+		while (*s && !ISSPACE(*s))
 			s++;
 	}
 	vec[n] = 0;
