@@ -12,14 +12,45 @@
   (in addition to being the library for handling the PBM format).
 
 **************************************************************************/
+#define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
+#define _LARGEFILE_SOURCE 1  /* Make sure ftello, fseeko are defined */
+#define _LARGEFILE64_SOURCE 1 
+#define _FILE_OFFSET_BITS 64
+    /* This means ftello() is really ftello64() and returns a 64 bit file
+       position.  Unless the C library doesn't have ftello64(), in which 
+       case ftello() is still just ftello().
+
+       Likewise for all the other C library file functions.
+
+       And off_t and fpos_t are 64 bit types instead of 32.
+    */
+#define _LARGE_FILES  
+    /* This does for AIX what _FILE_OFFSET_BITS=64 does for GNU */
+
 #include <stdio.h>
 #include "version.h"
 #include "compile.h"
+#include "nstring.h"
 #include "shhopt.h"
 #include "pm.h"
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+#ifdef __DJGPP__
+  #include <io.h>
+#endif
+
+/* In GNU, _LFS_LARGEFILE means the "off_t" functions are available.
+   In AIX, _LARGE_FILE_API says that.
+*/
+#if defined(_LFS_LARGEFILE) || defined(_LARGE_FILE_API)
+    /* The C library has ftello() and fseeko() */
+#else
+#define ftello ftell
+#define fseeko fseek
+#endif
+
+
 
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
@@ -180,9 +211,9 @@ pm_keymatch(char *       const strarg,
         c2 = *keyword++;
         if ( c2 == '\0' )
             return 0;
-        if ( isupper( c1 ) )
+        if ( ISUPPER( c1 ) )
             c1 = tolower( c1 );
-        if ( isupper( c2 ) )
+        if ( ISUPPER( c2 ) )
             c2 = tolower( c2 );
         if ( c1 != c2 )
             return 0;
@@ -340,7 +371,7 @@ pm_init(const char * const progname, unsigned int const flags) {
 
 static void
 showVersion(void) {
-    pm_message( "Using libpbm from Netpbm Version: %s", NETPBM_VERSION );
+    pm_message( "Using libnetpbm from Netpbm Version: %s", NETPBM_VERSION );
 #if defined(COMPILE_TIME) && defined(COMPILED_BY)
     pm_message( "Compiled %s by user \"%s\"",
                 COMPILE_TIME, COMPILED_BY );
@@ -573,16 +604,18 @@ pm_openr(const char * const name) {
 #ifndef VMS
         f = fopen(name, "rb");
 #else
-        f = fopen (name, "r", "ctx=stm");
+        f = fopen(name, "r", "ctx=stm");
 #endif
-        if (f == NULL) {
-            pm_perror(name);
-            exit(1);
-        }
+        if (f == NULL) 
+            pm_error("Unable to open file '%s' for reading.  "
+                     "fopen() returns errno %d (%s)", 
+                     name, errno, strerror(errno));
     }
     return f;
 }
 
+
+
 FILE*
 pm_openw(const char * const name) {
     FILE* f;
@@ -593,12 +626,12 @@ pm_openw(const char * const name) {
 #ifndef VMS
         f = fopen(name, "wb");
 #else
-        f = fopen (name, "w", "mbc=32", "mbf=2");  /* set buffer factors */
+        f = fopen(name, "w", "mbc=32", "mbf=2");  /* set buffer factors */
 #endif
-        if (f == NULL) {
-            pm_perror(name);
-            exit(1);
-        }
+        if (f == NULL) 
+            pm_error("Unable to open file '%s' for writing.  "
+                     "fopen() returns errno %d (%s)", 
+                     name, errno, strerror(errno));
     }
     return f;
 }
@@ -679,12 +712,13 @@ pm_openr_seekable(const char name[]) {
 
 void
 pm_close(FILE * const f) {
-    fflush( f );
-    if ( ferror( f ) )
-        pm_message( "a file read or write error occurred at some point" );
-    if ( f != stdin )
-        if ( fclose( f ) != 0 )
-            pm_perror( "fclose" );
+    fflush(f);
+    if (ferror(f))
+        pm_message("A file read or write error occurred at some point");
+    if (f != stdin)
+        if (fclose(f) != 0)
+            pm_error("close of file failed with errno %d (%s)",
+                     errno, strerror(errno));
 }
 
 
@@ -861,71 +895,169 @@ char *pm_read_unknown_size(FILE * const file, long * const nread) {
 }
 
 
-unsigned int
-pm_tell(FILE * const fileP) {
-/*----------------------------------------------------------------------------
-   Return the current file position.  Abort the program if error,
-   including if *fileP isn't a file that has a position.
------------------------------------------------------------------------------*/
-    long const rc = ftell(fileP);
 
-    if (rc < 0)
-        pm_error("ftell() to get current file position failed.  "
+void
+pm_tell2(FILE *       const fileP, 
+         pm_filepos * const fileposP,
+         unsigned int const fileposSize) {
+/*----------------------------------------------------------------------------
+   Return the current file position as *filePosP, which is a buffer
+   'fileposSize' bytes long.  Abort the program if error, including if
+   *fileP isn't a file that has a position.
+-----------------------------------------------------------------------------*/
+    /* Note: ftello may actually by ftello64 or ftell, depending on the 
+       capabilities of the underlying C library.
+    */
+    pm_filepos filepos = ftello(fileP);
+    if (filepos < 0)
+        pm_error("ftello() to get current file position failed.  "
                  "Errno = %s (%d)\n", strerror(errno), errno);
 
-    return rc;
+    if (fileposSize == sizeof(pm_filepos))
+        *fileposP = filepos;
+    else if (fileposSize == sizeof(long)) {
+        if (sizeof(pm_filepos) > sizeof(long) &&
+            filepos >= (pm_filepos) 1 << (sizeof(long)*8))
+            pm_error("File size is too large to represent in the %u bytes "
+                     "that were provided to pm_tell2()", fileposSize);
+        else 
+            *(long*)fileposP = (long)filepos;
+    } else
+        pm_error("File position size passed to pm_tell() is invalid: %u.  "
+                 "Valid sizes are %u and %u", 
+                 fileposSize, sizeof(pm_filepos), sizeof(long));
 }
 
 
+
+unsigned int
+pm_tell(FILE * const fileP) {
+    
+    long filepos;
+
+    pm_tell2(fileP, (pm_filepos *)&filepos, sizeof(filepos));
+
+    return filepos;
+}
+
+
+
+void
+pm_seek2(FILE *             const fileP, 
+         const pm_filepos * const fileposP,
+         unsigned int       const fileposSize) {
+/*----------------------------------------------------------------------------
+   Position file *fileP to position *fileposP.  Abort if error, including
+   if *fileP isn't a seekable file.
+-----------------------------------------------------------------------------*/
+    if (fileposSize == sizeof(pm_filepos)) 
+        /* Note: fseeko may actually by fseeko64 or fseek, depending on the 
+           capabilities of the underlying C library.
+        */
+        fseeko(fileP, *fileposP, SEEK_SET);
+    else if (fileposSize == sizeof(long)) {
+        long const fileposLong = *(long *)fileposP;
+        fseek(fileP, fileposLong, SEEK_SET);
+    } else
+        pm_error("File position size passed to pm_seek() is invalid: %u.  "
+                 "Valid sizes are %u and %u", 
+                 fileposSize, sizeof(pm_filepos), sizeof(long));
+}
+
+
+
 void
 pm_seek(FILE * const fileP, unsigned long filepos) {
 /*----------------------------------------------------------------------------
-   Position file *fileP to position 'filepos'.  Abort if error, including
-   if *fileP isn't a seekable file.
 -----------------------------------------------------------------------------*/
-    fseek(fileP, filepos, SEEK_SET);
+
+    pm_filepos fileposBuff;
+
+    fileposBuff = filepos;
+
+    pm_seek2(fileP, &fileposBuff, sizeof(fileposBuff));
 }
 
 
 
 void
 pm_nextimage(FILE * const file, int * const eofP) {
+/*----------------------------------------------------------------------------
+   Position the file 'file' to the next image in the stream, assuming it is
+   now positioned just after the current image.  I.e. read off any white
+   space at the end of the current image's raster.  Note that the raw formats
+   don't permit such white space, but this routine tolerates it anyway, 
+   because the plain formats do permit white space after the raster.
 
-    int c;
+   Iff there is no next image, return *eofP == TRUE.
 
-    c = getc(file);
-    if (c == EOF) {
-        if (feof(file))
-            *eofP = TRUE;
-        else
-            pm_error("File error on getc() to position to image");
-    } else {
-        int rc;
-        *eofP = FALSE;
-        rc = ungetc(c, file);
-        if (rc == EOF) 
-            pm_error("File error doing ungetc() to position to image.");
+   Note that in practice, we will not normally see white space here in
+   a plain PPM or plain PGM stream because the routine to read a
+   sample from the image reads one character of white space after the
+   sample in order to know where the sample ends.  There is not
+   normally more than one character of white space (a newline) after
+   the last sample in the raster.  But plain PBM is another story.  No white
+   space is required between samples of a plain PBM image.  But the raster
+   normally ends with a newline nonetheless.  Since the sample reading code
+   will not have read that newline, it is there for us to read now.
+-----------------------------------------------------------------------------*/
+    bool eof;
+    bool nonWhitespaceFound;
+
+    eof = FALSE;
+    nonWhitespaceFound = FALSE;
+
+    while (!eof && !nonWhitespaceFound) {
+        int c;
+        c = getc(file);
+        if (c == EOF) {
+            if (feof(file)) 
+                eof = TRUE;
+            else
+                pm_error("File error on getc() to position to image");
+        } else {
+            if (!isspace(c)) {
+                int rc;
+
+                nonWhitespaceFound = TRUE;
+
+                /* Have to put the non-whitespace character back in
+                   the stream -- it's part of the next image.  
+                */
+                rc = ungetc(c, file);
+                if (rc == EOF) 
+                    pm_error("File error doing ungetc() "
+                             "to position to image.");
+            }
+        }
     }
+    *eofP = eof;
 }
 
 
 
 void
-pm_check(FILE * const file, const enum pm_check_type check_type, 
-         const unsigned int need_raster_size,
+pm_check(FILE *               const file, 
+         enum pm_check_type   const check_type, 
+         pm_filepos           const need_raster_size,
          enum pm_check_code * const retval_p) {
-
+/*----------------------------------------------------------------------------
+   This is not defined for use outside of libnetpbm.
+-----------------------------------------------------------------------------*/
     struct stat statbuf;
-    int curpos;  /* Current position of file; -1 if none */
+    pm_filepos curpos;  /* Current position of file; -1 if none */
     int rc;
 
-    curpos = ftell(file);
+    /* Note: ftello may actually by ftello64 or ftell, depending on the 
+       capabilities of the underlying C library.
+    */
+    curpos = ftello(file);
     if (curpos >= 0) {
         /* This type of file has a current position */
             
         rc = fstat(fileno(file), &statbuf);
         if (rc != 0) 
-            pm_error("fstat() failed to get size of file, though ftell() "
+            pm_error("fstat() failed to get size of file, though ftello() "
                      "successfully identified\n"
                      "the current position.  Errno=%s (%d)",
                      strerror(errno), errno);
@@ -933,13 +1065,14 @@ pm_check(FILE * const file, const enum pm_check_type check_type,
             /* Not a regular file; we can't know its size */
             if (retval_p) *retval_p = PM_CHECK_UNCHECKABLE;
         } else {
-            const unsigned int have_raster_size = statbuf.st_size - curpos;
-                
+            pm_filepos const have_raster_size = statbuf.st_size - curpos;
+            
             if (have_raster_size < need_raster_size)
                 pm_error("File has invalid format.  The raster should "
-                         "contain %d bytes, but\n"
-                         "the file ends after only %d bytes.",
-                         need_raster_size, have_raster_size);
+                         "contain %u bytes, but\n"
+                         "the file ends after only %u bytes.",
+                         (unsigned int) need_raster_size, 
+                         (unsigned int) have_raster_size);
             else if (have_raster_size > need_raster_size) {
                 if (retval_p) *retval_p = PM_CHECK_TOO_LONG;
             } else {
