@@ -19,6 +19,7 @@
 #include "libpgm.h"
 #include "pbm.h"
 #include "libpbm.h"
+#include "fileio.h"
 
 void
 ppm_init( argcP, argv )
@@ -35,20 +36,18 @@ ppm_nextimage(FILE *file, int * const eofP) {
 
 
 void
-ppm_readppminitrest( file, colsP, rowsP, maxvalP )
-    FILE* file;
-    int* colsP;
-    int* rowsP;
-    pixval* maxvalP;
-    {
-    int maxval;
+ppm_readppminitrest(FILE *   const file, 
+                    int *    const colsP, 
+                    int *    const rowsP, 
+                    pixval * const maxvalP ) {
+        unsigned int maxval;
 
     /* Read size. */
-    *colsP = pbm_getint( file );
-    *rowsP = pbm_getint( file );
+    *colsP = (int)pm_getuint( file );
+    *rowsP = (int)pm_getuint( file );
 
     /* Read maxval. */
-    maxval = pbm_getint( file );
+    maxval = pm_getuint( file );
     if ( maxval > PPM_OVERALLMAXVAL )
 	pm_error("maxval (%d) is too large.\n"
              "The maximum allowed by the PPM is %d.",
@@ -87,42 +86,36 @@ ppm_readppminit( file, colsP, rowsP, maxvalP, formatP )
 	}
     }
 
-#if __STDC__
 void
-ppm_readppmrow( FILE* file, pixel* pixelrow, int cols, pixval maxval, int format )
-#else /*__STDC__*/
-void
-ppm_readppmrow( file, pixelrow, cols, maxval, format )
-    FILE* file;
-    pixel* pixelrow;
-    int cols, format;
-    pixval maxval;
-#endif /*__STDC__*/
-    {
-    register int col;
-    register pixel* pP;
-    register pixval r, g, b;
+ppm_readppmrow(FILE*  const file, 
+               pixel* const pixelrow, 
+               int    const cols, 
+               pixval const maxval, 
+               int    const format) {
+    int col;
+    pixel* pP;
+    pixval r, g, b;
     gray* grayrow;
-    register gray* gP;
+    gray* gP;
     bit* bitrow;
-    register bit* bP;
+    bit* bP;
 
     switch ( format )
 	{
 	case PPM_FORMAT:
         for ( col = 0, pP = pixelrow; col < cols; ++col, ++pP ) 
         {
-            r = pbm_getint( file );
+            r = pm_getuint( file );
 #ifdef DEBUG
             if ( r > maxval )
                 pm_error( "r value out of bounds (%u > %u)", r, maxval );
 #endif /*DEBUG*/
-            g = pbm_getint( file );
+            g = pm_getuint( file );
 #ifdef DEBUG
             if ( g > maxval )
                 pm_error( "g value out of bounds (%u > %u)", g, maxval );
 #endif /*DEBUG*/
-            b = pbm_getint( file );
+            b = pm_getuint( file );
 #ifdef DEBUG
             if ( b > maxval )
                 pm_error( "b value out of bounds (%u > %u)", b, maxval );
@@ -210,6 +203,11 @@ ppm_check(FILE * file, const enum pm_check_type check_type,
           const int format, const int cols, const int rows, const int maxval,
           enum pm_check_code * const retval_p) {
 
+    if (rows < 0)
+        pm_error("Invalid number of rows passed to ppm_check(): %d", rows);
+    if (cols < 0)
+        pm_error("Invalid number of columns passed to ppm_check(): %d", cols);
+    
     if (check_type != PM_CHECK_BASIC) {
         if (retval_p) *retval_p = PM_CHECK_UNKNOWN_TYPE;
     } else if (PPM_FORMAT_TYPE(format) == PBM_TYPE) {
@@ -219,9 +217,8 @@ ppm_check(FILE * file, const enum pm_check_type check_type,
     } else if (format != RPPM_FORMAT) {
         if (retval_p) *retval_p = PM_CHECK_UNCHECKABLE;
     } else {        
-        const unsigned int bytes_per_row =
-            cols * 3 * (maxval > 255 ? 2 : 1);
-        const unsigned int need_raster_size = rows * bytes_per_row;
+        pm_filepos const bytes_per_row = cols * 3 * (maxval > 255 ? 2 : 1);
+        pm_filepos const need_raster_size = rows * bytes_per_row;
         
         pm_check(file, check_type, need_raster_size, retval_p);
     }
