@@ -13,7 +13,6 @@
 */
 
 #include "ppm.h"
-#include "ppmcmap.h"
 #include "libppm.h"
 
 #define HASH_SIZE 20023
@@ -360,11 +359,12 @@ ppm_lookupcolor( const colorhash_table cht, const pixel * const colorP )
     return -1;
     }
 
+
+
 void
-ppm_freecolorhist( colorhist_vector chv )
-    {
-    free( (char*) chv );
-    }
+ppm_freecolorhist(colorhist_vector const chv) {
+    free(chv);
+}
 
 
 
