@@ -150,7 +150,7 @@ parseCommandLine (int argc, char ** argv,
                 long int const newcolors = strtol(argv[1], &tail, 10);
                 if (*tail != '\0')
                     pm_error("The number of colors argument '%s' is not "
-                             "a number.", argv[1]);
+                             "a number or 'all'", argv[1]);
                 else if (newcolors < 1)
                     pm_error("The number of colors must be positive");
                 else if (newcolors == 1)
@@ -224,17 +224,20 @@ sumcompare(const void * const b1, const void * const b2) {
 */
 
 static tupletable
-newColorMap(const struct pam * const pamP, 
-            unsigned int       const newcolors) {
+newColorMap(unsigned int const newcolors,
+            unsigned int const depth) {
 
     tupletable colormap;
     unsigned int i;
+    struct pam pam;
 
-    colormap = pnm_alloctupletable(pamP, newcolors);
+    pam.depth = depth;
+
+    colormap = pnm_alloctupletable(&pam, newcolors);
 
     for (i = 0; i < newcolors; ++i) {
         unsigned int plane;
-        for (plane = 0; plane < pamP->depth; ++plane) 
+        for (plane = 0; plane < depth; ++plane) 
             colormap[i]->tuple[plane] = 0;
     }
     return colormap;
@@ -261,10 +264,10 @@ newBoxVector(int const colors, int const sum, int const newcolors) {
 
 
 static void
-findBoxBoundaries(struct pam * const pamP, 
-                  tupletable   const colorfreqtable,
-                  unsigned int const indx,
-                  unsigned int const clrs,
+findBoxBoundaries(tupletable   const colorfreqtable,
+                  unsigned int const depth,
+                  unsigned int const boxStart,
+                  unsigned int const boxSize,
                   sample             minval[],
                   sample             maxval[]) {
 /*----------------------------------------------------------------------------
@@ -274,13 +277,13 @@ findBoxBoundaries(struct pam * const pamP,
     unsigned int plane;
     unsigned int i;
 
-    for (plane = 0; plane < pamP->depth; ++plane) 
-        minval[plane] = maxval[plane] = colorfreqtable[indx]->tuple[plane];
+    for (plane = 0; plane < depth; ++plane) 
+        minval[plane] = maxval[plane] = colorfreqtable[boxStart]->tuple[plane];
     
-    for (i = 1; i < clrs; ++i) {
+    for (i = 1; i < boxSize; ++i) {
         unsigned int plane;
-        for (plane = 0; plane < pamP->depth; ++plane) {
-            sample const v = colorfreqtable[indx + i]->tuple[plane];
+        for (plane = 0; plane < depth; ++plane) {
+            sample const v = colorfreqtable[boxStart + i]->tuple[plane];
             if (v < minval[plane]) minval[plane] = v;
             if (v > maxval[plane]) maxval[plane] = v;
         } 
@@ -290,14 +293,14 @@ findBoxBoundaries(struct pam * const pamP,
 
 
 static unsigned int
-largestByNorm(struct pam * const pamP, sample minval[], sample maxval[]) {
+largestByNorm(sample minval[], sample maxval[], unsigned int const depth) {
     
     unsigned int largestDimension;
     unsigned int plane;
 
     sample largestSpreadSoFar = 0;  
     largestDimension = 0;
-    for (plane = 0; plane < pamP->depth; ++plane) {
+    for (plane = 0; plane < depth; ++plane) {
         sample const spread = maxval[plane]-minval[plane];
         if (spread > largestSpreadSoFar) {
             largestDimension = plane;
@@ -310,7 +313,8 @@ largestByNorm(struct pam * const pamP, sample minval[], sample maxval[]) {
 
 
 static unsigned int 
-largestByLuminosity(struct pam * const pamP, sample minval[], sample maxval[]) {
+largestByLuminosity(sample minval[], sample maxval[], 
+                    unsigned int const depth) {
 /*----------------------------------------------------------------------------
    This subroutine presumes that the tuple type is either 
    BLACKANDWHITE, GRAYSCALE, or RGB (which implies pamP->depth is 1 or 3).
@@ -318,7 +322,7 @@ largestByLuminosity(struct pam * const pamP, sample minval[], sample maxval[]) {
 -----------------------------------------------------------------------------*/
     unsigned int retval;
 
-    if (pamP->depth == 1)
+    if (depth == 1)
         retval = 0;
     else {
         /* An RGB tuple */
@@ -344,22 +348,22 @@ largestByLuminosity(struct pam * const pamP, sample minval[], sample maxval[]) {
 
 
 static void
-centerBox(struct pam * const pamP, 
-          int          const indx,
-          int          const clrs,
+centerBox(int          const boxStart,
+          int          const boxSize,
           tupletable   const colorfreqtable, 
+          unsigned int const depth,
           tuple        const newTuple) {
 
     unsigned int plane;
 
-    for (plane = 0; plane < pamP->depth; ++plane) {
+    for (plane = 0; plane < depth; ++plane) {
         int minval, maxval;
         unsigned int i;
 
-        minval = maxval = colorfreqtable[indx]->tuple[plane];
+        minval = maxval = colorfreqtable[boxStart]->tuple[plane];
         
-        for (i = 1; i < clrs; ++i) {
-            int const v = colorfreqtable[indx + i]->tuple[plane];
+        for (i = 1; i < boxSize; ++i) {
+            int const v = colorfreqtable[boxStart + i]->tuple[plane];
             minval = MIN( minval, v);
             maxval = MAX( maxval, v);
         }
@@ -370,34 +374,34 @@ centerBox(struct pam * const pamP,
 
 
 static void
-averageColors(struct pam * const pamP, 
-              int          const indx,
-              int          const clrs,
+averageColors(int          const boxStart,
+              int          const boxSize,
               tupletable   const colorfreqtable, 
+              unsigned int const depth,
               tuple        const newTuple) {
 
     unsigned int plane;
 
-    for (plane = 0; plane < pamP->depth; ++plane) {
+    for (plane = 0; plane < depth; ++plane) {
         sample sum;
         int i;
 
         sum = 0;
 
-        for (i = 0; i < clrs; ++i) 
-            sum += colorfreqtable[indx+i]->tuple[plane];
+        for (i = 0; i < boxSize; ++i) 
+            sum += colorfreqtable[boxStart+i]->tuple[plane];
 
-        newTuple[plane] = sum / clrs;
+        newTuple[plane] = sum / boxSize;
     }
 }
 
 
 
 static void
-averagePixels(struct pam * const pamP, 
-              int          const indx,
-              int          const clrs,
+averagePixels(int          const boxStart,
+              int          const boxSize,
               tupletable   const colorfreqtable, 
+              unsigned int const depth,
               tuple        const newTuple) {
 
     unsigned int n;
@@ -407,19 +411,19 @@ averagePixels(struct pam * const pamP,
 
     /* Count the tuples in question */
     n = 0;  /* initial value */
-    for (i = 0; i < clrs; ++i)
-        n += colorfreqtable[indx + i]->value;
+    for (i = 0; i < boxSize; ++i)
+        n += colorfreqtable[boxStart + i]->value;
 
 
-    for (plane = 0; plane < pamP->depth; ++plane) {
+    for (plane = 0; plane < depth; ++plane) {
         sample sum;
         int i;
 
         sum = 0;
 
-        for (i = 0; i < clrs; ++i) 
-            sum += colorfreqtable[indx+i]->tuple[plane]
-                * colorfreqtable[indx+i]->value;
+        for (i = 0; i < boxSize; ++i) 
+            sum += colorfreqtable[boxStart+i]->tuple[plane]
+                * colorfreqtable[boxStart+i]->value;
 
         newTuple[plane] = sum / n;
     }
@@ -428,11 +432,11 @@ averagePixels(struct pam * const pamP,
 
 
 static tupletable
-colormapFromBv(struct pam *      const pamP, 
-               unsigned int      const newcolors,
+colormapFromBv(unsigned int      const newcolors,
                boxVector         const bv, 
                unsigned int      const boxes,
                tupletable        const colorfreqtable, 
+               unsigned int      const depth,
                enum methodForRep const methodForRep) {
     /*
     ** Ok, we've got enough boxes.  Now choose a representative color for
@@ -445,20 +449,20 @@ colormapFromBv(struct pam *      const pamP,
     tupletable colormap;
     unsigned int bi;
 
-    colormap = newColorMap(pamP, newcolors);
+    colormap = newColorMap(newcolors, depth);
 
     for (bi = 0; bi < boxes; ++bi) {
         switch (methodForRep) {
         case REP_CENTER_BOX: 
-            centerBox(pamP, bv[bi].ind, bv[bi].colors, colorfreqtable, 
+            centerBox(bv[bi].ind, bv[bi].colors, colorfreqtable, depth, 
                       colormap[bi]->tuple);
             break;
         case REP_AVERAGE_COLORS:
-            averageColors(pamP, bv[bi].ind, bv[bi].colors, colorfreqtable,
+            averageColors(bv[bi].ind, bv[bi].colors, colorfreqtable, depth,
                           colormap[bi]->tuple);
             break;
         case REP_AVERAGE_PIXELS:
-            averagePixels(pamP, bv[bi].ind, bv[bi].colors, colorfreqtable,
+            averagePixels(bv[bi].ind, bv[bi].colors, colorfreqtable, depth,
                           colormap[bi]->tuple);
             break;
         default:
@@ -471,100 +475,142 @@ colormapFromBv(struct pam *      const pamP,
 
 
 
+static unsigned int
+freqTotal(tupletable const freqtable, unsigned int const entries) {
+    
+    unsigned int i;
+    unsigned int sum;
+
+    sum = 0;
+
+    for (i = 0; i < entries; ++i)
+        sum += freqtable[i]->value;
+
+    return sum;
+}
+
+
+static void
+splitBox(boxVector             const bv, 
+         unsigned int *        const boxesP, 
+         unsigned int          const bi,
+         tupletable            const colorfreqtable, 
+         unsigned int          const depth,
+         enum methodForLargest const methodForLargest) {
+/*----------------------------------------------------------------------------
+   Split Box 'bi' in the box vector bv (so that bv contains one more box
+   than it did as input).  Split it so that each new box represents about
+   half of the pixels in the distribution given by 'colorfreqtable' for 
+   the colors in the original box, but with distinct colors in each of the
+   two new boxes.
+
+   Assume the box contains at least two colors.
+-----------------------------------------------------------------------------*/
+    unsigned int const boxStart = bv[bi].ind;
+    unsigned int const boxSize = bv[bi].colors;
+    unsigned int const sm = bv[bi].sum;
+
+    sample minval[MAXDEPTH];
+    sample maxval[MAXDEPTH];
+    unsigned int largestDimension;
+    /* number of the plane with the largest spread */
+    unsigned int medianIndex;
+    int lowersum;
+    /* Number of pixels whose value is "less than" the median */
+
+    findBoxBoundaries(colorfreqtable, depth, boxStart, boxSize, 
+                      minval, maxval);
+
+    /*
+            ** Find the largest dimension, and sort by that component.  I have
+            ** included two methods for determining the "largest" dimension;
+            ** first by simply comparing the range in RGB space, and second
+            ** by transforming into luminosities before the comparison. 
+            */
+    switch (methodForLargest) {
+    case LARGE_NORM: 
+        largestDimension = largestByNorm(minval, maxval, depth);
+        break;
+    case LARGE_LUM: 
+        largestDimension = largestByLuminosity(minval, maxval, depth);
+        break;
+    }
+                                                    
+    /* TODO: I think this sort should go after creating a box,
+       not before splitting.  Because you need the sort to use
+       the REP_CENTER_BOX method of choosing a color to
+       represent the final boxes 
+    */
+
+    qsort((char*) &colorfreqtable[boxStart], boxSize, 
+          sizeof(colorfreqtable[boxStart]), 
+          samplecompare[largestDimension].fn);
+            
+    {
+        /* Now find the median based on the counts, so that
+           about half the pixels (not colors, pixels) are in
+           each subdivision. 
+                */
+        unsigned int i;
+
+        lowersum = colorfreqtable[boxStart]->value; /* initial value */
+        for (i = 1; i < boxSize - 1 && lowersum < sm/2; ++i) {
+            lowersum += colorfreqtable[boxStart + i]->value;
+        }
+        medianIndex = i;
+    }
+    /* Split the box, and sort to bring the biggest boxes to the top.  */
+
+    bv[bi].colors = medianIndex;
+    bv[bi].sum = lowersum;
+    bv[*boxesP].ind = boxStart + medianIndex;
+    bv[*boxesP].colors = boxSize - medianIndex;
+    bv[*boxesP].sum = sm - lowersum;
+    ++(*boxesP);
+    qsort((char*) bv, *boxesP, sizeof(struct box), sumcompare);
+}
+
+
+
 static tupletable
-mediancut(struct pam * const pamP,
-          tupletable   const colorfreqtable, 
-          int          const colors, 
-          int          const newcolors,
+mediancut(tupletable            const colorfreqtable, 
+          int                   const colors, 
+          unsigned int          const depth,
+          int                   const newcolors,
           enum methodForLargest const methodForLargest,
           enum methodForRep     const methodForRep) {
 /*----------------------------------------------------------------------------
-   If *pamP describes an image with 'colors' distinct tuple values in it,
-   and colorfreqtable[] gives how many times each of those tuple values
-   occurs, compute a set of only 'newcolors' colors that are close to the
-   colors in colorfreqtable[].
+   Compute a set of only 'newcolors' colors that best represent an image
+   whose pixels are summarized by the histogram 'colorfreqtable'.  There are
+   'colors' colors in that table.  Each tuple in it has depth 'depth'.
+   colorfreqtable[i] tells the number of pixels in the subject image have a
+   particular color.
 
    As a side effect, sort 'colorfreqtable'.
 -----------------------------------------------------------------------------*/
-
     tupletable colormap;
     boxVector bv;
-    int bi;
-    int boxes;
+    unsigned int bi;
+    unsigned int boxes;
+    bool multicolorBoxesExist;
+        /* There is at least one box that contains at least 2 colors; ergo,
+           there is more splitting we can do.
+        */
 
-    bv = newBoxVector(colors, pamP->width*pamP->height, newcolors);
+    bv = newBoxVector(colors, freqTotal(colorfreqtable, colors), newcolors);
     boxes = 1;
-
+    multicolorBoxesExist = (colors > 1);
 
     /* Main loop: split boxes until we have enough. */
-    while (boxes < newcolors) {
-        unsigned int indx;
-        unsigned int clrs;
-        unsigned int sm;
-        sample minval[MAXDEPTH];
-        sample maxval[MAXDEPTH];
-        unsigned int largestDimension;
-            /* number of the plane with the largest spread */
-        unsigned int medianIndex;
-        int lowersum;
-            /* Number of pixels whose value is "less than" the median */
-
+    while (boxes < newcolors && multicolorBoxesExist) {
         /* Find the first splittable box. */
         for (bi = 0; bi < boxes && bv[bi].colors < 2; ++bi);
         if (bi >= boxes)
-            break;  /* ran out of colors! */
-        indx = bv[bi].ind;
-        clrs = bv[bi].colors;
-        sm = bv[bi].sum;
-
-        findBoxBoundaries(pamP, colorfreqtable, indx, clrs, minval, maxval);
-
-        /*
-        ** Find the largest dimension, and sort by that component.  I have
-        ** included two methods for determining the "largest" dimension;
-        ** first by simply comparing the range in RGB space, and second
-        ** by transforming into luminosities before the comparison. 
-        */
-        switch (methodForLargest) {
-        case LARGE_NORM: 
-            largestDimension = largestByNorm(pamP, minval, maxval);
-            break;
-        case LARGE_LUM: 
-            largestDimension = largestByLuminosity(pamP, minval, maxval);
-            break;
-        }
-                                                    
-        /* TODO: I think this sort should go after creating a box, not before
-           splitting.  Because you need the sort to use the REP_CENTER_BOX
-           method of choosing a color to represent the final boxes 
-        */
-        qsort((char*) &colorfreqtable[indx], clrs, 
-              sizeof(colorfreqtable[indx]), 
-              samplecompare[largestDimension].fn);
-
-        {
-            /* Now find the median based on the counts, so that about half the
-               pixels (not colors, pixels) are in each subdivision.
-            */
-            unsigned int i;
-
-            lowersum = colorfreqtable[indx]->value;  /* initial value */
-            for (i = 1; i < clrs - 1 && lowersum < sm/2; ++i) {
-                lowersum += colorfreqtable[indx + i]->value;
-            }
-            medianIndex = i;
-        }
-        /* Split the box, and sort to bring the biggest boxes to the top. */
-        bv[bi].colors = medianIndex;
-        bv[bi].sum = lowersum;
-        bv[boxes].ind = indx + medianIndex;
-        bv[boxes].colors = clrs - medianIndex;
-        bv[boxes].sum = sm - lowersum;
-        ++boxes;
-        qsort((char*) bv, boxes, sizeof(struct box), sumcompare);
+            multicolorBoxesExist = FALSE;
+        else 
+            splitBox(bv, &boxes, bi, colorfreqtable, depth, methodForLargest);
     }
-
-    colormap = colormapFromBv(pamP, newcolors, bv, boxes, colorfreqtable, 
+    colormap = colormapFromBv(newcolors, bv, boxes, colorfreqtable, depth,
                               methodForRep);
 
     free(bv);
@@ -576,80 +622,92 @@ mediancut(struct pam * const pamP,
 
 static void
 computeHistogram(struct pam *   const pamP, 
-                 tuple **       const tuples,
                  tupletable *   const colorfreqtableP, 
-                 unsigned int * const colorsP) {
+                 unsigned int * const colorsP,
+                 sample *       const freqtableMaxvalP) {
 /*----------------------------------------------------------------------------
-  Attempt to make a histogram of the colors, unclustered.  If at first
-  we don't succeed, lower maxval to increase color coherence and try
-  again.  This will eventually terminate, with maxval at worst 15,
-  since 32^3 is approximately MAXCOLORS.
+  Attempt to make a histogram of the colors in the image represented by
+  *pamP, whose file is seekable and is now positioned to the raster.
 
-  We actually change the maxval of the input 'tuples' and update the input
-  *pamP to reflect the new maxval.
------------------------------------------------------------------------------*/
+  If at first we don't succeed, rescale the color resolution to half
+  the present maxval and try again, repeatedly.  This will eventually
+  terminate, with maxval at worst 16, which is half of 32, and 32^3 is
+  just slightly larger than MAXCOLORS.
+----------------------------------------------------------------------------*/
     tupletable colorfreqtable;
-    unsigned int colors;
+    sample maxval;
+    pm_filepos rasterPos;
+
+    pm_message("making histogram...");
+
+    pm_tell2(pamP->file, &rasterPos, sizeof(rasterPos));
 
     colorfreqtable = NULL;  /* initial value */
     
-    while (colorfreqtable == NULL)
+    for (maxval = pamP->maxval;
+         colorfreqtable == NULL && maxval > 0;
+         maxval /= 2)
     {
-        pm_message( "making histogram..." );
-        colorfreqtable = pnm_computetuplefreqtable(pamP, tuples, 
-                                                   MAXCOLORS, &colors);
-        if (colorfreqtable == NULL) {
-            sample const newmaxval = pamP->maxval / 2;
-            int row;
-
-            pm_message("too many colors!");
-            pm_message("scaling colors from maxval=%ld to maxval=%ld "
-                       "to improve clustering...",
-                       pamP->maxval, newmaxval );
-            for (row = 0; row < pamP->height; ++row) {
-                int col;
-                for (col = 0; col < pamP->width; ++col)
-                    pnm_scaletuple(pamP, tuples[row][col], tuples[row][col], 
-                                   newmaxval);
-            }
-            pamP->maxval = newmaxval;
-        }
+        pm_seek2(pamP->file, &rasterPos, sizeof(rasterPos));
+        colorfreqtable = 
+            pnm_computetuplefreqtable2(pamP, NULL, MAXCOLORS, maxval, colorsP);
+        if (colorfreqtable)
+            *freqtableMaxvalP = maxval;
     }
-    pm_message( "%d colors found", colors );
+
+    /* colorfreqtable is now non-null.  The "maxval > 0" in the for loop 
+       limit above is just for robustness.
+    */
+
+    if (*freqtableMaxvalP != pamP->maxval) 
+        pm_message("Too many colors (more than %u)!  "
+                   "scaled color resolution from maxval %lu "
+                   "to maxval %lu before choosing",
+                   MAXCOLORS, pamP->maxval, *freqtableMaxvalP);
+
+    pm_message("%u colors found", *colorsP);
     *colorfreqtableP = colorfreqtable;
-    *colorsP = colors;
 }
 
 
 
 
 static void
-computeColorMapFromInput(struct pam * const pamP, tuple ** const tuples,
-                         bool const allColors, int const reqColors, 
+computeColorMapFromInput(struct pam *          const pamP, 
+                         bool                  const allColors, 
+                         int                   const reqColors, 
                          enum methodForLargest const methodForLargest,
-                         enum methodForRep const methodForRep,
-                         tupletable * const colormapP,
-                         int * const colorsP) {
+                         enum methodForRep     const methodForRep,
+                         tupletable *          const colormapP,
+                         int *                 const colorsP,
+                         sample *              const colormapMaxvalP) {
 /*----------------------------------------------------------------------------
-   Produce a colormap containing the colors that we will use in the output.
-   Figure it out using the median cut technique, based on the colors (and
-   their frequencies) in the input image described by *pamP and tuples[][].
+   Produce a colormap containing the best colors to represent the
+   image represented by *pamP, whose file is seekable and now
+   positioned to the raster.  Figure it out using the median cut
+   technique.
 
-   The output map will have 'reqcolors' or fewer colors in it, unless
-   'allcolors' is true, in which case it will have all the colors that are
-   in the input.
+   The colormap will have 'reqcolors' or fewer colors in it, unless
+   'allcolors' is true, in which case it will have all the colors that
+   are in the input.
 
-   Modify the maxval of 'tuples', and update *pamP to reflect it, if 
-   necessary.
+   The colormap has the same maxval as the input, unless the input contains
+   more than MAXCOLORS colors.  In that case, we compute the map from a
+   maxval-reduced version of the input.  The colormap has that reduced 
+   maxval.  Note that in this case, 'allColors' does not cause the colormap
+   to have all the colors from the input; rather, it contains all the colors
+   from the input once the input's color resolution is reduced to the new
+   maxval.
 
    Put the colormap in newly allocated storage as a tupletable 
-   and return its address as *colormapP.
+   and return its address as *colormapP.  Return the number of colors in
+   it as *colorsP and its maxval as *colormapMaxvalP.
 -----------------------------------------------------------------------------*/
     tupletable colorfreqtable;
     unsigned int colors;
+    sample colormapMaxval;
 
-    computeHistogram(pamP, tuples, &colorfreqtable, &colors);
-       /* May modify tuples[][]/pamP to reduce the maxval */
+    computeHistogram(pamP, &colorfreqtable, &colors, &colormapMaxval);
     
     if (allColors) {
         *colormapP = colorfreqtable;
@@ -662,12 +720,14 @@ computeColorMapFromInput(struct pam * const pamP, tuple ** const tuples,
             *colorsP = colors;
         } else {
             pm_message("choosing %d colors...", reqColors);
-            *colormapP = mediancut(pamP, colorfreqtable, colors, reqColors,
+            *colormapP = mediancut(colorfreqtable, colors, pamP->depth, 
+                                   reqColors,
                                    methodForLargest, methodForRep);
             *colorsP = reqColors;
             pnm_freetupletable(pamP, colorfreqtable);
         }
     }
+    *colormapMaxvalP = colormapMaxval;
 }
 
 
@@ -789,6 +849,7 @@ colormapToImage(struct pam * const inpamP,
                 struct pam * const outpamP, 
                 tupletable   const colormap,
                 unsigned int const colormapSize, 
+                sample       const colormapMaxval,
                 bool         const sort,
                 bool         const square,
                 tuple ***    const outputRasterP) {
@@ -803,7 +864,7 @@ colormapToImage(struct pam * const inpamP,
     outpamP->format = inpamP->format;
     outpamP->plainformat = FALSE;
     outpamP->depth = inpamP->depth;
-    outpamP->maxval = inpamP->maxval;
+    outpamP->maxval = colormapMaxval;
     outpamP->bytes_per_sample = inpamP->bytes_per_sample;
     strcpy(outpamP->tuple_type, inpamP->tuple_type);
 
@@ -822,12 +883,12 @@ int
 main(int argc, char * argv[] ) {
 
     struct cmdlineInfo cmdline;
-    FILE* ifp;
+    FILE* ifP;
     struct pam inpam, outpam;
-    tuple** tuples;
     tuple** colormapRaster;
     int newcolors;
     tupletable colormap;
+    sample colormapMaxval;
 
     pnm_init(&argc, argv);
 
@@ -835,22 +896,20 @@ main(int argc, char * argv[] ) {
  
     parseCommandLine(argc, argv, &cmdline);
 
-    ifp = pm_openr(cmdline.inputFilespec);
+    ifP = pm_openr_seekable(cmdline.inputFilespec);
 
-    tuples = pnm_readpam(ifp, &inpam, sizeof(inpam));
+    pnm_readpaminit(ifP, &inpam, sizeof(inpam));
     
-    pm_close(ifp);
-
-    computeColorMapFromInput(&inpam, tuples, 
+    computeColorMapFromInput(&inpam,
                              cmdline.allcolors, cmdline.newcolors, 
                              cmdline.methodForLargest, 
                              cmdline.methodForRep,
-                             &colormap, &newcolors);
-        /* May modify maxval of 'tuples', inpam */
+                             &colormap, &newcolors, &colormapMaxval);
 
-    pnm_freepamarray(tuples, &inpam);
+    pm_close(ifP);
 
-    colormapToImage(&inpam, &outpam, colormap, newcolors, cmdline.sort,
+    colormapToImage(&inpam, &outpam, colormap, newcolors, 
+                    colormapMaxval, cmdline.sort,
                     cmdline.square, &colormapRaster);
 
     if (cmdline.verbose)
